/*
Name:   draw.c
Zweck:  Operation zur grafischen Darstellung der Voting-Struktur
Autor:  Andreas Seifert
Datum:
Update:
*/


/* motif.h laesst sich nicht direkt inkludieren: linker fehler wegen doppelten
 * definitionen von z.B. zoomOption oder charset
 */
void wprint(const char *, ...);
#include "draw.h"


/*
 * AU: Zeichnen der Objekte
 * ST: drawRubberRectangle()      zeichnet ein "gestricheltes" Rechteck
 *     drawRubberArc()            zeichnet einen "gestrichelten" Kreis
 *     cleanWorkbench()           gibt GC's und Pixmap frei
 *     Intersect()                entscheidet, ob sich  2 Rechtecke schneiden
 *     setCursor()                bestimmt die Form des Cursors
 *     moveCursor()               versetzt den Mauszeiger
 *     initGraphik()              initialisiert Zeichengeraete
 *     createGC()                 erzeugt Grafikkontexte
 *     loadZoomFonts()            liest Zeichensaetze ein
 *     printFontPath()            Ausgabe des Suchpfades fuer Zeichensaetze
 *     createPixmap()             erzeugt die Pixmaps der Rechnersymbole 
 *     drawStringsTerminal()      zeichnet ein Rechnersymbol und Beschriftungen
 *     draw75Llines()             zeichnet die Verbindungslinien bei Zoom 75%
 *     draw50Lines()              zeichnet die Verbindungslinien bei Zoom 50%
 *     draw100Lines()             zeichnet die Verbindungslinien bei normaler
 *                                Darstellung
 *     drawArcWeight()            zeichnet die Kantengewichte 
 *     redrawArea()               gibt die Zeichenflaeche aus
 *     clearArea()                loescht die Zeichenflaeche
 *     delDependingExposeEvents() unterdrueckt ueberzaehlige Expose-Events
 */


/****************************************************************************/
/****************                             *******************************/
/****************    Beginn der Funktionen    *******************************/
/****************                             *******************************/
/****************************************************************************/



void drawRubberRectangle(x_koord, y_koord)
     int x_koord, y_koord;                        /* Koordinaten Rechteck */
/*
 * AU: Rechteck im Modus Dragging/Rubberbanding zeichnen, d.h. zweimaliges 
 *     Anwenden der Funktion stellt den Originalzustand der Zeichenflaeche 
 *     wieder her
 * VB: 1. x_koord und y_koord bezeichnen die linke obere Ecke des Rechtecks
 *     2. Breite des Rechtecks betraegt 100 Pixel
 *     3. Hoehe des Rechtecks betraegt 70 Pixel
 */
{  
   switch (zoomOption)
   {
          case ZOOM_100 :
               XDrawRectangle(display, drawingWindow, drawkit.writeGCs[RUBBER], 
                              x_koord - 5, y_koord - 17,
                              RECT_WIDTH - 4, RECT_HEIGHT - 1);
               break;

          case ZOOM_75 :
               XDrawRectangle(display, drawingWindow, drawkit.writeGCs[RUBBER],
                              x_koord - 5, y_koord - 9,
			      RECT_WIDTH - 22, RECT_HEIGHT - 15);
               break;

          case ZOOM_50 :
               XDrawRectangle(display, drawingWindow, drawkit.writeGCs[RUBBER],
	                      x_koord - 2, y_koord - 9,
	                      RECT_WIDTH - 40, RECT_HEIGHT - 23);
               break;

          default :
               XBell(display, 0);
               wprint(unknown_event, "draw.c : drawRubberRectangle");
               break;
   }
}


void drawRubberArc(x_koord, y_koord)
     int x_koord, y_koord;               /* Koordinaten umgebendes Rechteck */
/*
 * AU: Ellipsenbogen im Modus Dragging/Rubberbanding zeichnen, d.h. 
 *     zweimaliges Anwenden der Funktion stellt den Originalzustand der 
 *     Zeichenflaeche wieder her 
 * VB: 1. x_koord und y_koord bezeichnen die linke obere Ecke des umgebenden
 *        Rechtecks der Ellipse
 *     2. Breite des Rechtecks betraegt 100 Pixel
 *     3. Hoehe des Rechtecks betraegt 70 Pixel
 *     4. Anfang des Ellipsenbogens als Winkel ist 0
 *     5. Oeffnung des Bogens als Winkel, wobei der Winkel in 1/64 Grad
 *        angegeben ist (360 * 64 = 23040)
 */
{
   switch (zoomOption)
   {      
          case ZOOM_100 :
               XDrawArc(display, drawingWindow, drawkit.writeGCs[RUBBER],
                        x_koord - 9, y_koord - 16, 
                        RECT_WIDTH, RECT_HEIGHT, 0, 23040);
               break;

          case ZOOM_75 :      
	       XDrawArc(display, drawingWindow, drawkit.writeGCs[RUBBER],
                        x_koord - 10, y_koord - 11, 
                        RECT_WIDTH - 5, RECT_HEIGHT - 10, 0, 23040);
               break;
   
          case ZOOM_50 :      
	       XDrawArc(display, drawingWindow, drawkit.writeGCs[RUBBER], 
                        x_koord - 12, y_koord - 10, 
	                RECT_WIDTH - 23, RECT_HEIGHT - 19, 0, 23040); 
               break;
  
          default :
	       XBell(display, 0);
	       wprint(unknown_event, "draw.c : drawRubberArc");
	       break;
   }    							    
}



void clean_Workbench()
/*
 * AU: Gibt nicht mehr benoetigte Grafikkontexte und die Pixmaps wieder frei
 */
{ 
   XtReleaseGC(Widgetliste[DRAW_AREA], drawkit.writeGCs[RUBBER]);
   XtReleaseGC(Widgetliste[DRAW_AREA], drawkit.writeGCs[FILL]);

   XFreeGC(display, drawkit.writeGCs[ZOOM_50]);
   XFreeGC(display, drawkit.writeGCs[ZOOM_75]);
   XFreeGC(display, drawkit.writeGCs[ZOOM_100]);

   XFreePixmap(display, drawkit.terminals[ZOOM_50]);
   XFreePixmap(display, drawkit.terminals[ZOOM_75]);
   XFreePixmap(display, drawkit.terminals[ZOOM_100]);

}


Bool Intersect(x1, y1, x2, y2)
     int x1, y1, x2, y2;                       /* Koordinaten der Rechtecke */
/*
 * AU: Pruefung ob sich zwei Rechtecke ueberschneiden
 * VB: 1. x1,y1 und x2,y2 bezeichnen die jeweils linke obere Ecke des Rechtecks
 *     2. Breite der Rechtecke betraegt 90 Pixel
 *     3. Hoehe der Rechtecke betraegt 53 Pixel
 * NB: -- ueberschneiden sich die Rechtecke ist der Rueckgabewert False
 *     -- sonst ist der Rueckgabewert True
 */
{
   switch (zoomOption)
   {
          case ZOOM_100 :
               if (x1 + RECT_WIDTH < x2 || y1 + RECT_HEIGHT < y2) 
                  return False;
               if (x1 > x2 + RECT_WIDTH || y1 > y2 + RECT_HEIGHT) 
                  return False;
               return True;

          case ZOOM_75 :
               if (x1 + RECT_WIDTH - 5 < x2 || y1 + RECT_HEIGHT - 10 < y2) 
                  return False;
               if (x1 > x2 + RECT_WIDTH - 5 || y1 > y2 + RECT_HEIGHT - 10)
                  return False;
               return True;

          case ZOOM_50 :
               if (x1 + RECT_WIDTH - 19 < x2 || y1 + RECT_HEIGHT - 17 < y2)
                  return False;
               if (x1 > x2 + RECT_WIDTH - 19 || y1 > y2 + RECT_HEIGHT - 17) 
                  return False;
               return True;

          default :
               XBell(display, 0);
               wprint(unknown_event, "draw.c : Intersect");
               return False;
    } 
}


void setCursor(widget, shape)
     Widget   widget;                        /* Zuordnung gilt fuer widget */
     unsigned int shape;                     /* Nummer des Cursors im Font */
/*        
 * AU: einem Bildschirmbereich wird eine best. Form des Maus-Cursors 
 *     zugeordnet
 */              
{
   Cursor  cursor;
 
   /* Cursor erzeugen */
   cursor = XCreateFontCursor(display, shape);         

   /* Cursor zuordnen */
   if (cursor) XDefineCursor(display, XtWindow(widget), cursor);
      
   /* Cursor freigeben */
   XFreeCursor(display, cursor);                     
}



void moveCursor(move_x, move_y)
     int move_x, move_y;
/*
 * AU: Versetzen des Mauszeigers zur Liste der Rechner   
 */
{
   Window   win;             /* Eingabewindow */
   Window   child;           /* zurueckgegebener Sohn */
   int      root_x, root_y;  /* Zeiger relativ zum Ursprung des Rootwindow */
   int      win_x, win_y;    /* Zeigerposition relativ zum Eingabewindow win */
   unsigned int key_buttons; /* nicht benutzt */

   /* aktuelle Zeigerposition ablesen */
   XQueryPointer(display, rootWindow, &win, &child, &root_x, &root_y, 
                 &win_x, &win_y, &key_buttons);

   /* springen des Cursors zu seiner neuen Position */
   XWarpPointer(display, None, rootWindow , 0, 0, 0, 0,
                root_x + move_x, root_y + move_y);
}



void initGraphik()
/*
 * AU: Initialisierung    
 * ST: ruft createGC(), setCursor() und createPixmap() auf
 */
{
   Bool status;
                        
   /* Bildschirm */
   screen = XtScreen(Widgetliste[DRAW_AREA]);

   /* Bildschirmtiefe */
   screendepth = DefaultDepthOfScreen(screen);
   
   /* Terminal */
   display = XtDisplay(Widgetliste[DRAW_AREA]);

   /* Ausschnitt auf dem Bildschirm */
   drawingWindow = XtWindow(Widgetliste[DRAW_AREA]);

   /* 
    * Ressource-Name des Root-Windows fuer den Bildschirm screen, alle 
    * anderen Fenster liegen hierarchisch innerhalb des Root-Windows
    */
   rootWindow = RootWindowOfScreen(screen); 

   createGC();

   /* Erzeuge die Pixmaps mit dem Rechnersymbol */
   status = createPixmap();

   demand(status, Pixmaps konnten nicht erzeugt werden);

   /* Cursor-Form fuer die Zeichenflaeche und Arbeitsbereich festlegen */
   setCursor(Widgetliste[DRAW_AREA], XC_hand1);

   setCursor(Widgetliste[EDIT_BUTTONS], XC_hand2);
}




void createGC()
/*
 * AU: Erzeugung der Grafikkontexte
 */
{
   /* Struktur zur Angabe der Attribute beim Erzeugen eines Grafikkontextes */
   XGCValues xgc_values;
   XtGCMask  valuemask;
   Arg       args[2];

   /* 
    * Vorder- und Hintergrundfarbe der Zeichenflaeche zur Erzeugung des
    * Grafikkontextes benutzen
    */
   XtSetArg(args[0], XmNforeground, &xgc_values.foreground);
   XtSetArg(args[1], XmNbackground, &xgc_values.background);
   XtGetValues(Widgetliste[DRAW_AREA], args, 2);  

   /* 
    * GC's fuer verschiedene Zoomstufen erzeugen
    * Attributbelegung fuer den Grafikkontext bei normalen Zeichenoperationen
    * 1. Verwendung des Pixel_Wertes foreground
    * 2. keine zusaetzlichen Expose-Events
    */
   xgc_values.graphics_exposures = False;
   valuemask = GCForeground | GCBackground | GCGraphicsExposures;

   drawkit.writeGCs[ZOOM_50] = XCreateGC(display, drawingWindow, 
                                         valuemask, &xgc_values);

   demand(drawkit.writeGCs[ZOOM_50],
          Grafikkontext 50% konnte nicht erzeugt werden);

   drawkit.writeGCs[ZOOM_75] = XCreateGC(display, drawingWindow, 
                                         valuemask, &xgc_values);

   demand(drawkit.writeGCs[ZOOM_75],
          Grafikkontext Zoom 75% konnte nicht erzeugt werden);

   drawkit.writeGCs[ZOOM_100] = XCreateGC(display, drawingWindow, 
                                          valuemask, &xgc_values);

   demand(drawkit.writeGCs[ZOOM_100], 
          Grafikkontext 100% konnte nicht erzeugt werden);

   /* 
    * Attributbelegung fuer den Grafikkontext fuer Zeichnungen im Modus
    * "Rubberbanding", bzw. "Dragging". Zweimaliges Zeichnen unter
    * Verwendung des Grafikkontextes rubber_gc stellt den Originalzustand
    * wieder her
    * 1. die auszufuehrende Bit-Operation zwischen der Quelle und dem Ziel 
    *    ist das "exklusive Oder" 
    * 2. Verwendung des Pixel_Wertes foreground
    * 3. Linientyp "gestrichelt"
    */       
   
   xgc_values.function = GXxor;
   xgc_values.line_style = LineOnOffDash;
   valuemask = GCForeground | GCBackground | GCFunction | GCLineStyle;
                       
   drawkit.writeGCs[RUBBER] = XtGetGC(Widgetliste[DRAW_AREA], 
                                      valuemask, &xgc_values);  

   demand(drawkit.writeGCs[RUBBER], Grafikkontext konnte nicht erzeugt werden);

   /*
    * Attributbelegung fuer den Grafikkontext zum Loeschen eines Kreises
    * 1. Vor- und Hintergrund wird mit weiss belegt;
    */

   xgc_values.foreground = xgc_values.background; 
   valuemask = GCForeground;

   drawkit.writeGCs[FILL] = XtGetGC(Widgetliste[DRAW_AREA],
                                    valuemask, &xgc_values);
   
   demand(drawkit.writeGCs[FILL], Grafikkontext konnte nicht erzeugt werden);
}



Bool loadZoomFonts()
/*
 * AU: Einlesen der Zeichensaetze und Belegung der Grafikkontexte
 * ST: Aufruf printFontPath()
 */

{
   Bool return_value = True;

   /*
    * Laden des kleinen Zeichensatzes
    */
   drawkit.font_struct[ZOOM_50] = XLoadQueryFont(display, KLEINER_FONT); 

   if (drawkit.font_struct[ZOOM_50] == NULL) {
      fprintf(stderr, fontfailed, KLEINER_FONT);
      return_value = False;
      }
   else { /* Belegung des Grafikkontexts mit dem Zeichensatz */
        XSetFont(display, drawkit.writeGCs[ZOOM_50], 
                drawkit.font_struct[ZOOM_50]->fid);
        XFreeFont(display, drawkit.font_struct[ZOOM_50]);
        }
   /*
    * Laden des mittleren Zeichensatzes
    */ 
   drawkit.font_struct[ZOOM_75] = XLoadQueryFont(display, MITTLERER_FONT); 
      
   if (drawkit.font_struct[ZOOM_75] == NULL) {
      fprintf(stderr, fontfailed, MITTLERER_FONT);
      return_value = False;
      }
   else { /* Belegung des Grafikkontexts mit dem Zeichensatz */
        XSetFont(display, drawkit.writeGCs[ZOOM_75], 
                drawkit.font_struct[ZOOM_75]->fid);
        XFreeFont(display, drawkit.font_struct[ZOOM_75]);
        }

   /*
    * Ausgabe des Suchpfades bei ergebnisloser Suche nach den
    * Zeichensaetzen
    */
   if (!return_value) {
      printFontPath(); 
      XFreePixmap(display, drawkit.terminals[ZOOM_50]);
      XFreePixmap(display, drawkit.terminals[ZOOM_75]);
      }

   return return_value;
}



void printFontPath()
/*
 * AU: Ausgabe des Suchpfades fuer Zeichensaetze
 */
{
   int  ndirs, i;      /* Anzahl der Katalognamen im Feld und Laufvariable */
   char **directories; /* Zeiger auf das Feld der Katalognamen */

   directories = XGetFontPath(display, &ndirs);
   
   fprintf(stderr, searchpath);

   for (i = 0; i < ndirs; i++) fprintf(stderr, pathis, directories[i]);

   XFreeFontPath(directories);
}



Bool createPixmap()
/*
 * AU: Erzeuge die Rechnersymbole
 */
{
   Pixmap term50bits, term75bits, term100bits;

  /*
   * Eine Pixmap ist ein Speicherbereich, der ein Rechteck aus Pixel-Werten
   * enthaelt. Die Bitmaps term??bits beinhalten die Rechnersymbole. Zur 
   * Abbildung des Rechnersymbols wird eine auf den Bildschirm abgestimmte
   * Pixmap aus drawkit.terminals[] benoetigt.
   */

   /* Bitmap aus den Bitmap-Daten erzeugen */
   if ((term100bits = 
       XCreateBitmapFromData(display, drawingWindow, 
                             term100_bits, term100_width, term100_height))
       == False) return False; 

   /* Pixmap mit den gleichen Abmessungen wie das Rechnersymbol erzeugen */
   if ((drawkit.terminals[ZOOM_100] = 
       XCreatePixmap(display, rootWindow, term100_width, term100_height, 
                     screendepth))
       == False) return False;

   /* den Inhalt der Bitmap in die Pixmap kopieren */
   XCopyPlane(display, term100bits, drawkit.terminals[ZOOM_100],
              drawkit.writeGCs[ZOOM_100], 0, 0,
              term100_width, term100_height, 0, 0, 1);


   /* Bitmap aus den Bitmap-Daten erzeugen */
   if ((term75bits = 
       XCreateBitmapFromData(display, drawingWindow, 
                             term75_bits, term75_width, term75_height))
       == False) return False;

   /* Pixmap mit den gleichen Abmessungen wie das Rechnersymbol erzeugen */   
   if ((drawkit.terminals[ZOOM_75] = 
       XCreatePixmap(display, rootWindow, term75_width, term75_height,
                     screendepth))
       == False) return False;

   /* den Inhalt der Bitmap in die Pixmap kopieren */
   XCopyPlane(display, term75bits, drawkit.terminals[ZOOM_75], 
              drawkit.writeGCs[ZOOM_75], 0, 0, 
              term75_width, term75_height, 0, 0, 1);


   /* Bitmap aus den Bitmap-Daten erzeugen */ 
   if ((term50bits = 
       XCreateBitmapFromData(display, drawingWindow, 
                             term50_bits, term50_width, term50_height))
       == False) return False;

   /* Pixmap mit den gleichen Abmessungen wie das Rechnersymbol erzeugen */
   if ((drawkit.terminals[ZOOM_50] = 
        XCreatePixmap(display, rootWindow, term50_width, term50_height,
                      screendepth))
        == False) return False;
   
   /* den Inhalt der Bitmap in die Pixmap kopieren */
   XCopyPlane(display, term50bits, drawkit.terminals[ZOOM_50], 
              drawkit.writeGCs[ZOOM_50], 0, 0, 
              term50_width, term50_height, 0, 0, 1);

   
   /* Freigabe der Bitmaps */
   XFreePixmap(display, term50bits);
   XFreePixmap(display, term75bits);
   XFreePixmap(display, term100bits); 

   return True;
}




void drawStringsTerminal(x_pos, y_pos, text_name, text_identity, 
                         vote, quor, version)
     int  x_pos, y_pos;                   /* Koordinaten des Rechnersymbols */
     char text_name[40];                  /* Bezeichnung des Rechners */
     char *text_identity;                 /* eindeutiger Rechnername */
     int  vote, quor[], version;          /* Stimme, Quoren, Versionsnummer */
/*
 * AU: Zeichnen des Rechnersymbols, seiner Beschriftung und einer Umrandung 
 */ 
{          
  char stringquor1[10]; /* Ausgabezeile zur Anzeige der Quoren 1 bis 3 */
  char stringquor2[10]; /* Ausgabezeile zur Anzeige der Quoren 4 und 5 */
  char stringQuor[3], stringVote[3];

  sprintf(stringVote, "%d", vote);

  if (version == 0) { /* physikalischer Rechner */
  
     switch(zoomOption)
     {
     case ZOOM_100 :
          /* Loesche die Zeichenflaeche */
          XClearArea(display, drawingWindow, x_pos - 5, y_pos - 17,
                    RECT_WIDTH - 4, RECT_HEIGHT - 1, False);

          /* schreibe die Rechnerbezeichnung */                   
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_100], 
                     x_pos + 10, y_pos -3, text_name, 
                     Minimum(strlen(text_name),13));

          /* zeichne ein umliegendes Rechteck um das Rechnersymbol */
          XDrawRectangle(display, drawingWindow, drawkit.writeGCs[ZOOM_100], 
                        x_pos - 5, y_pos - 17, RECT_WIDTH - 4, RECT_HEIGHT - 1);

          /* zeichne das Rechnersymbol */
          XCopyArea(display, drawkit.terminals[ZOOM_100], drawingWindow,
                   drawkit.writeGCs[ZOOM_100], 0, 0,
                   term100_width, term100_height, x_pos, y_pos);

          /* schreibe den eindeutigen Rechnernamen */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_100],
                     x_pos + 10, y_pos + 43, 
                     text_identity, strlen(text_identity));
 
          /* schreibe die Stimmenanzahl */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_100],
                     x_pos + 35, y_pos + 10,
                     stringVote, strlen(stringVote));
          break;

     case ZOOM_75 :
          /* Loesche die Zeichenflaeche */
          XClearArea(display, drawingWindow, x_pos - 5, y_pos - 9,
                    RECT_WIDTH - 22, RECT_HEIGHT - 15, False);

          /* schreibe die Rechnerbezeichnung */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                     x_pos + 6 , y_pos , text_name, 
                     Minimum(strlen(text_name),13));

          /* zeichne ein umliegendes Rechteck um das Rechnersymbol */
          XDrawRectangle(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                        x_pos - 5, y_pos - 9, RECT_WIDTH - 22, RECT_HEIGHT - 15);

          /* zeichne das Rechnersymbol */
          XCopyArea(display, drawkit.terminals[ZOOM_75], drawingWindow,
                   drawkit.writeGCs[ZOOM_75], 0, 0, 
                   term75_width, term75_height, x_pos, y_pos + 2);

          /* schreibe den eindeutigen Rechnernamen */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                     x_pos + 8, y_pos + 39,
                     text_identity, strlen(text_identity));

          /* schreibe die Stimmenanzahl */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                     x_pos + 25, y_pos + 10,
                     stringVote, strlen(stringVote));
          break;

     case ZOOM_50 :
          /* Loesche die Zeichenflaeche */
          XClearArea(display, drawingWindow, x_pos - 2, y_pos - 9,
                    RECT_WIDTH - 40, RECT_HEIGHT - 23, False);

          /* schreibe die Rechnerbezeichnung */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                     x_pos + 5, y_pos, text_name, 
                     Minimum(strlen(text_name),13));

          /* zeichne ein umliegendes Rechteck um das Rechnersymbol */
          XDrawRectangle(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                        x_pos - 2, y_pos - 9, RECT_WIDTH - 40, RECT_HEIGHT - 23);

          /* zeichne das Rechnersymbol */
          XCopyArea(display, drawkit.terminals[ZOOM_50], drawingWindow,
                   drawkit.writeGCs[ZOOM_50], 0, 0,
                   term50_width, term50_height, x_pos, y_pos + 1);

          /* schreibe den eindeutigen Rechnernamen */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                     x_pos + 5, y_pos + 30,
                     text_identity, strlen(text_identity));

          /* schreibe die Stimmenanzahl */
          XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                     x_pos + 19, y_pos + 8,
                     stringVote, strlen(stringVote));        
          break;

     default :  
          XBell(display, 0);
          wprint(unknown_event, "draw.c : drawStringsTerminal");
          break;          
     }
     }
  else { /* virtueller Rechner */

       switch(zoomOption)
       {
       case ZOOM_100 :
            /* Loesche die Zeichenflaeche */
            XFillArc(display, drawingWindow, drawkit.writeGCs[FILL], 
                    x_pos - 9, y_pos - 16, RECT_WIDTH, RECT_HEIGHT,
                    0, 23040);
    
            /* zeichne eine umliegende Ellipse um das Rechnersymbol */
            XDrawArc(display, drawingWindow, drawkit.writeGCs[ZOOM_100], 
                    x_pos - 9, y_pos - 16, RECT_WIDTH, RECT_HEIGHT, 
                    0, 23040);

            /* zeichne das Rechnersymbol */
            XCopyArea(display, drawkit.terminals[ZOOM_100], drawingWindow,
                     drawkit.writeGCs[ZOOM_100], 0, 0,
                     term100_width, term100_height, x_pos, y_pos);

            /* schreibe den eindeutigen Rechnernamen */
            XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_100],
                       x_pos + 10, y_pos + 43,
                       text_identity, strlen(text_identity));

            /* schreibe die Stimmenanzahl */
            XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_100],
                       x_pos + 35, y_pos + 10,
                       stringVote, strlen(stringVote));
            break;

       case ZOOM_75 :
            /* Loesche die Zeichenflaeche */
            XFillArc(display, drawingWindow, drawkit.writeGCs[FILL],
                    x_pos - 10, y_pos - 11, RECT_WIDTH - 5, RECT_HEIGHT - 10,
                    0, 23040);
    
            /* zeichne eine umliegende Ellipse um das Rechnersymbol */
            XDrawArc(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                    x_pos - 10, y_pos - 11, RECT_WIDTH - 5, RECT_HEIGHT - 10,
                    0, 23040);
        
            /* zeichne das Rechnersymbol */
            XCopyArea(display, drawkit.terminals[ZOOM_75], drawingWindow,
                     drawkit.writeGCs[ZOOM_75], 0, 0,
                     term75_width, term75_height, x_pos, y_pos + 2);

            /* schreibe den eindeutigen Rechnernamen */
            XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                       x_pos + 9, y_pos + 39,
                       text_identity, strlen(text_identity));

            /* schreibe die Stimmenanzahl */
            XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                       x_pos + 25, y_pos + 10,
                       stringVote, strlen(stringVote));
            break;

       case ZOOM_50 :
            /* Loesche die Zeichenflaeche */
            XFillArc(display, drawingWindow, drawkit.writeGCs[FILL],
                    x_pos - 12, y_pos - 10, RECT_WIDTH - 23, RECT_HEIGHT - 19,
                    0, 23040);

            /* zeichne eine umliegende Ellipse um das Rechnersymbol */
            XDrawArc(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                    x_pos - 12, y_pos - 10, RECT_WIDTH - 23, RECT_HEIGHT - 19,
                    0, 23040);

            /* zeichne das Rechnersymbol */
            XCopyArea(display, drawkit.terminals[ZOOM_50], drawingWindow,
                     drawkit.writeGCs[ZOOM_50], 0, 0,
                     term50_width, term50_height, x_pos, y_pos);

            /* schreibe den eindeutigen Rechnernamen */
            XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                       x_pos + 5, y_pos + 30,
                       text_identity, strlen(text_identity));

            /* schreibe die Stimmenanzahl */
            XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                       x_pos + 19, y_pos + 8,
                       stringVote, strlen(stringVote));
            break;

       default :
            XBell(display, 0);
            wprint(unknown_event, "draw.c : drawStringsTerminal");
            break;
       }
       }

  /* initialisiere die Ausgabezeilen mit dem Leerzeichen */
  strcpy(stringquor1, SPACE);
  
  strcpy(stringquor2, SPACE);

  /* belege Ausgabezeile1 mit dem Quorum Nr.1 */
  sprintf(stringQuor, "%d", quor[0]);
  strncat(stringquor1, stringQuor, 2);
 
  /* 
   * Quorengewichte sind in Abhaengigkeit von der Benutzerwahl 
   * durch Kommata getrennt anzuzeigen; bis zur Quorenanzahl 3 wird
   * die Ausgabezeile1 genutzt; die Quoren 4 und 5 sind in der 
   * Ausgabezeile2 enthalten 
   */ 
  switch (data.numb_quor)
  {
         case 5 :

         /* fuege an die Ausgabezeile1 die Quorengewichte 2 und 3 */
         strcat(stringquor1, COMMA);

         sprintf(stringQuor, "%d", quor[1]);
         strncat(stringquor1, stringQuor, 2);  
         strcat(stringquor1, COMMA);   

         sprintf(stringQuor, "%d", quor[2]);
         strncat(stringquor1, stringQuor, 2);         
         strcat(stringquor1, COMMA); 

         /* fuege an die Ausgabezeile2 die Quorengewichte 4 und 5 */
         sprintf(stringQuor, "%d", quor[3]);
         strncat(stringquor2, stringQuor, 2);                                
         strcat(stringquor2, COMMA);

         sprintf(stringQuor, "%d", quor[4]);
         strncat(stringquor2, stringQuor, 2);
       
         /* 
          * zeichne die Ausgabezeile2 in Abhaengigkeit 
          * der Zoomstufe
          */ 
         switch(zoomOption)
	 {
	       case ZOOM_100 :
                    XDrawString(display, drawingWindow, 
                               drawkit.writeGCs[ZOOM_100], 
                               x_pos + 30, y_pos + 30, 
                               stringquor2, strlen(stringquor2));
                    break;
  
	       case ZOOM_75 :
                    XDrawString(display, drawingWindow,
                               drawkit.writeGCs[ZOOM_75],
                               x_pos + 22, y_pos + 30,
                               stringquor2, strlen(stringquor2));
                    break;

	       case ZOOM_50 :
                    XDrawString(display, drawingWindow,
                               drawkit.writeGCs[ZOOM_50],
                               x_pos + 17, y_pos + 22,
                               stringquor2, strlen(stringquor2));
                    break;
	  }
         break;


         case 4 :
         
         /* fuege an die Ausgabezeile1 die Quorengewichte 2 und 3 */
         strcat(stringquor1, COMMA);

         sprintf(stringQuor, "%d", quor[1]);
         strncat(stringquor1, stringQuor, 2);
         strcat(stringquor1, COMMA);
  
         sprintf(stringQuor, "%d", quor[2]);
         strncat(stringquor1, stringQuor, 2);
         strcat(stringquor1, COMMA);         

         /* fuege an die Ausgabezeile2 das Quorengewicht 4 */
         sprintf(stringQuor, "%d", quor[3]);
         strncat(stringquor2, stringQuor, 2);
                              
         /* 
          * zeichne die Ausgabezeile2 in Abhaengigkeit der
          * Zoomstufe
          */
         switch(zoomOption)
         {
	 case ZOOM_100 :
              XDrawString(display, drawingWindow, 
                         drawkit.writeGCs[ZOOM_100], 
                         x_pos + 30, y_pos + 30, 
                         stringquor2, strlen(stringquor2));
              break;
         
	 case ZOOM_75 :
              XDrawString(display, drawingWindow,
                         drawkit.writeGCs[ZOOM_75],
                         x_pos + 22, y_pos + 30,
                         stringquor2, strlen(stringquor2));
               break;

	 case ZOOM_50 :
              XDrawString(display, drawingWindow,
                         drawkit.writeGCs[ZOOM_50],
                         x_pos + 17, y_pos + 22,
                         stringquor2, strlen(stringquor2));
              break;
	 }
         break;


         case 3 :

         /* fuege an die Ausgabezeile1 die Quorengewichte 2 und 3 */         
         strcat(stringquor1, COMMA);
   
         sprintf(stringQuor, "%d", quor[1]);
         strncat(stringquor1, stringQuor, 2);
         strcat(stringquor1, COMMA);
         
         sprintf(stringQuor, "%d", quor[2]);
         strncat(stringquor1, stringQuor, 2);
         break;


         case 2 :
         
         /* fuege an die Ausgabezeile1 das Quorengewicht 2 */
         strcat(stringquor1, COMMA);

         sprintf(stringQuor, "%d", quor[1]);
         strncat(stringquor1, stringQuor, 2);
         break;
  } /* end switch */
  
  /* zeichne die Ausgabezeile1 in Abhaengigkeit der Zoomstufe */
  switch(zoomOption)
  {
        case ZOOM_100 :
             XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_100], 
                        x_pos + 30, y_pos + 20, 
                        stringquor1, strlen(stringquor1));
             break;
      
	case ZOOM_75 :
             XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                        x_pos + 22, y_pos + 20,
                        stringquor1, strlen(stringquor1));
             break;

        case ZOOM_50 :
             XDrawString(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                        x_pos + 17, y_pos + 15,
                        stringquor1, strlen(stringquor1));
             break;
  }

} 



void draw75Lines(node)
     GRAPHIC_NODE *node;
/*
 * AU: Zeichnen der Verbindungen und ihrer Belegung bei Zoomstufe 75% 
 * ST: Aufruf drawArcWeight()
 */
{
 GRAPHIC_ARC *arc_ptr;      /* Zeiger auf eine Beziehung */ 
 double      a, b, alpha;   /* Koordinaten der Pfeilspitze und Winkel */
 int         numb_segments; /* Anzahl der Koordinatenpaare */
 XSegment    point[3];      /* Feld der Koordinatenpaare */

  /* Zeichne alle Verbindungen des Knoten "node" */
  for(arc_ptr = node->first_arc; arc_ptr != NULL; arc_ptr = arc_ptr->next_arc) {
  
     if (node->top_y > arc_ptr->arc_to_object->top_y) {

        /* Belegung des ersten Koordinatenpaares */
        point[0].x1 = (short) node->top_x - 6; 
        point[0].y1 = (short) node->top_y + 7;
        point[0].x2 = (short) arc_ptr->arc_to_object->bottom_x  - 6;
        point[0].y2 = (short) arc_ptr->arc_to_object->bottom_y - 7;
        numb_segments = 1;

        /* Zeichnen der Kantenbelegung */
        drawArcWeight(arc_ptr, node->top_x - 6, node->top_y + 7,
	              arc_ptr->arc_to_object->bottom_x - 6,
		      arc_ptr->arc_to_object->bottom_y - 7);

        if (node->top_y > arc_ptr->arc_to_object->bottom_y) {
	   /* der Ausgangsknoten ist vollstaendig unter dem Zielknoten */  
           a = (double)(node->top_x - arc_ptr->arc_to_object->bottom_x - 12);
           b = (double)(node->top_y - arc_ptr->arc_to_object->bottom_y);
           alpha = atan(a/b);

           /* Belegung des zweiten und dritten Koordinatenpaares */
           point[1].x1 = (short) (arc_ptr->arc_to_object->bottom_x +
		                  (sin(alpha + 0.5) * 10)  - 6);
	   point[1].y1 = (short) (arc_ptr->arc_to_object->bottom_y +
                                  (cos(alpha + 0.5) * 10) - 8);
           point[1].x2 = (short) arc_ptr->arc_to_object->bottom_x  - 6;
           point[1].y2 = (short) arc_ptr->arc_to_object->bottom_y - 8;

           point[2].x1 = (short) arc_ptr->arc_to_object->bottom_x - 6;
           point[2].y1 = (short) arc_ptr->arc_to_object->bottom_y - 8;
           point[2].x2 = (short) (arc_ptr->arc_to_object->bottom_x +
                                  (sin(alpha - 0.5) * 10)  - 6);
           point[2].y2 = (short) (arc_ptr->arc_to_object->bottom_y +
                                  (cos(alpha - 0.5) * 10) - 8);
           numb_segments = 3;
           }
        }
     else {
          /* Belegung des ersten Koordinatenpaares */          
          point[0].x1 = (short) node->bottom_x - 4;
          point[0].y1 = (short) node->bottom_y - 10;
          point[0].x2 = (short) arc_ptr->arc_to_object->top_x - 3;
          point[0].y2 = (short) arc_ptr->arc_to_object->top_y + 7;
          numb_segments = 1;

          /* Zeichnen der Kantenbelegung */
	  drawArcWeight(arc_ptr, node->bottom_x - 4, node->bottom_y - 10,
                        arc_ptr->arc_to_object->top_x - 3,
                        arc_ptr->arc_to_object->top_y + 7);

           if (node->bottom_y < arc_ptr->arc_to_object->top_y) {
              /* der Ausgangsknoten ist vollstaendig ueber dem Zielknoten */
              a = (double)(node->bottom_x - arc_ptr->arc_to_object->top_x - 9);
              b = (double)(node->bottom_y - arc_ptr->arc_to_object->top_y - 3);
              alpha = atan(a/b);

              /* Belegung des zweiten und dritten Koordinatenpaares */
              point[1].x1 = (short) (arc_ptr->arc_to_object->top_x -
                                     (sin(alpha + 0.5) * 10) - 3);
              point[1].y1 = (short) (arc_ptr->arc_to_object->top_y -
                                     (cos(alpha + 0.5) * 10) + 7);
              point[1].x2 = (short) arc_ptr->arc_to_object->top_x - 3;
              point[1].y2 = (short) arc_ptr->arc_to_object->top_y + 7;

              point[2].x1 = (short) arc_ptr->arc_to_object->top_x - 3;
              point[2].y1 = (short) arc_ptr->arc_to_object->top_y + 7;
              point[2].x2 = (short) (arc_ptr->arc_to_object->top_x -
	                             (sin(alpha - 0.5) * 10) - 3);
              point[2].y2 = (short) (arc_ptr->arc_to_object->top_y -
                                     (cos(alpha - 0.5) * 10) + 7);
              numb_segments = 3;
              }
           }
      /* Zeichnen der Beziehung */
      XDrawSegments(display, drawingWindow, drawkit.writeGCs[ZOOM_75],
                    point, numb_segments);
      }
}



void draw50Lines(node)
     GRAPHIC_NODE *node;
/*
 * AU: Zeichnen der Verbindungen und ihrer Belegung bei Zoomstufe 50% 
 * ST: Aufruf drawArcWeight()
 */
{
 GRAPHIC_ARC *arc_ptr;      /* Zeiger auf eine Beziehung */  
 double      a, b, alpha;   /* Koordinaten der Pfeilspitze und Winkel */
 int         numb_segments; /* Anzahl der Koordinatenpaare */
 XSegment    point[3];      /* Feld der Koordinatenpaare */

 /* Zeichne alle Verbindungen des Knoten "node" */
 for(arc_ptr = node->first_arc; arc_ptr != NULL; arc_ptr = arc_ptr->next_arc) {

    if (node->top_y > arc_ptr->arc_to_object->top_y) {

       /* Belegung des ersten Koordinatenpaares */       
       point[0].x1 = (short) node->top_x - 10;
       point[0].y1 = (short) node->top_y + 8;
       point[0].x2 = (short) arc_ptr->arc_to_object->bottom_x - 11;
       point[0].y2 = (short) arc_ptr->arc_to_object->bottom_y - 16;
       numb_segments = 1;

       /* Zeichnen der Kantenbelegung */
       drawArcWeight(arc_ptr, node->top_x - 10, node->top_y +8,
                     arc_ptr->arc_to_object->bottom_x - 11,
                     arc_ptr->arc_to_object->bottom_y - 16);

       if (node->top_y > arc_ptr->arc_to_object->bottom_y) {
          /* der Ausgangsknoten ist vollstaendig unter dem Zielknoten */
          a = (double)(node->top_x - arc_ptr->arc_to_object->bottom_x);
          b = (double)(node->top_y - arc_ptr->arc_to_object->bottom_y);
          alpha = atan(a/b);

          /* Belegung des zweiten und dritten Koordinatenpaares */          
          point[1].x1 = (short) (arc_ptr->arc_to_object->bottom_x +
                                 (sin(alpha + 0.5) * 10) - 11);
          point[1].y1 = (short) (arc_ptr->arc_to_object->bottom_y +
                                 (cos(alpha + 0.5) * 10) - 16);
          point[1].x2 = (short) arc_ptr->arc_to_object->bottom_x - 11;
          point[1].y2 = (short) arc_ptr->arc_to_object->bottom_y - 16;

          point[2].x1 = (short) arc_ptr->arc_to_object->bottom_x - 11;
          point[2].y1 = (short) arc_ptr->arc_to_object->bottom_y - 16;
          point[2].x2 = (short) (arc_ptr->arc_to_object->bottom_x +
                                 (sin(alpha - 0.5) * 10) - 11);
          point[2].y2 = (short) (arc_ptr->arc_to_object->bottom_y +
                                 (cos(alpha - 0.5) * 10) - 16);
          numb_segments = 3;
          }
       }
    else {
         /* Belegung des ersten Koordinatenpaares */
         point[0].x1 = (short) node->bottom_x - 9; /* von */
         point[0].y1 = (short) node->bottom_y - 17;
         point[0].x2 = (short) arc_ptr->arc_to_object->top_x - 10; /* nach */
         point[0].y2 = (short) arc_ptr->arc_to_object->top_y + 7;
         numb_segments = 1;

         /* Zeichnen der Kantenbelegung */
         drawArcWeight(arc_ptr, node->bottom_x - 9, node->bottom_y - 17,
                       arc_ptr->arc_to_object->top_x - 10,
                       arc_ptr->arc_to_object->top_y + 7);

         if (node->bottom_y < arc_ptr->arc_to_object->top_y) {
            /* der Ausgangsknoten ist vollstaendig ueber dem Zielknoten */
            a = (double)(node->bottom_x - arc_ptr->arc_to_object->top_x);
            b = (double)(node->bottom_y - arc_ptr->arc_to_object->top_y);
            alpha = atan(a/b);

            /* Belegung des zweiten und dritten Koordinatenpaares */
            point[1].x1 = (short) (arc_ptr->arc_to_object->top_x -
                                   (sin(alpha + 0.5) * 10) - 10);
            point[1].y1 = (short) (arc_ptr->arc_to_object->top_y -
                                   (cos(alpha + 0.5) * 10) + 7);
            point[1].x2 = (short) arc_ptr->arc_to_object->top_x - 10;
            point[1].y2 = (short) arc_ptr->arc_to_object->top_y + 7;

            point[2].x1 = (short) arc_ptr->arc_to_object->top_x - 10;
            point[2].y1 = (short) arc_ptr->arc_to_object->top_y + 7;
            point[2].x2 = (short) (arc_ptr->arc_to_object->top_x -
                                   (sin(alpha - 0.5) * 10) - 10);
            point[2].y2 = (short) (arc_ptr->arc_to_object->top_y -
                                   (cos(alpha - 0.5) * 10) + 7);
            numb_segments = 3;
            }
         }
    /* Zeichnen der Beziehung */
    XDrawSegments(display, drawingWindow, drawkit.writeGCs[ZOOM_50],
                  point, numb_segments);  
    }     
}



 
void draw100Lines(node)
     GRAPHIC_NODE *node;
/*
 * AU: Zeichnen der Verbindungen und ihrer Belegung bei normaler Anzeige 
 * ST: Aufruf drawArcWeight()
 */
{
 GRAPHIC_ARC *arc_ptr;       /* Zeiger auf eine Beziehung */    
 double      a, b, alpha;    /* Koordinaten der Pfeilspitze und Winkel */
 int         numb_segments;  /* Anzahl der Koordinatenpaare */
 XSegment    point[3];       /* Feld der Koordinatenpaare */

 /* Zeichne alle Verbindungen des Knoten "node" */                         
 for(arc_ptr = node->first_arc; arc_ptr != NULL; arc_ptr = arc_ptr->next_arc) { 

    if (node->top_y > arc_ptr->arc_to_object->top_y) {

       /* Belegung des ersten Koordinatenpaares */
       point[0].x1 = (short) node->top_x;
       point[0].y1 = (short) node->top_y;
       point[0].x2 = (short) arc_ptr->arc_to_object->bottom_x;
       point[0].y2 = (short) arc_ptr->arc_to_object->bottom_y - 2;
       numb_segments = 1;

       /* Zeichnen der Kantenbelegung */
       drawArcWeight(arc_ptr, node->top_x, node->top_y,
                     arc_ptr->arc_to_object->bottom_x,
                     arc_ptr->arc_to_object->bottom_y - 2);
      
       /* Belegung des zweiten und dritten Koordinatenpaares */  
       if (node->top_y > arc_ptr->arc_to_object->bottom_y) {
          /* der Ausgangsknoten ist vollstaendig unter dem Zielknoten */
          a = (double)(node->top_x - arc_ptr->arc_to_object->bottom_x);
          b = (double)(node->top_y - arc_ptr->arc_to_object->bottom_y);
          alpha = atan(a/b); 
                        
          point[1].x1 = (short) (arc_ptr->arc_to_object->bottom_x + 
                                 (sin(alpha + 0.5) * 10));
          point[1].y1 = (short) (arc_ptr->arc_to_object->bottom_y + 
                                 (cos(alpha + 0.5) * 10) - 2);
          point[1].x2 = (short) arc_ptr->arc_to_object->bottom_x;
          point[1].y2 = (short) arc_ptr->arc_to_object->bottom_y - 2;

          point[2].x1 = (short) arc_ptr->arc_to_object->bottom_x;
          point[2].y1 = (short) arc_ptr->arc_to_object->bottom_y - 2;
          point[2].x2 = (short) (arc_ptr->arc_to_object->bottom_x + 
                                 (sin(alpha - 0.5) * 10));
          point[2].y2 = (short) (arc_ptr->arc_to_object->bottom_y + 
                                 (cos(alpha - 0.5) * 10) - 2);
          numb_segments = 3;
          }
       } 
    else {
         /* Belegung des ersten Koordinatenpaares */        
         point[0].x1 = (short) node->bottom_x;
         point[0].y1 = (short) node->bottom_y - 2;
         point[0].x2 = (short) arc_ptr->arc_to_object->top_x;
         point[0].y2 = (short) arc_ptr->arc_to_object->top_y;
         numb_segments = 1;  

         /* Zeichnen der Kantenbelegung */
         drawArcWeight(arc_ptr, node->bottom_x, node->bottom_y - 2,
                       arc_ptr->arc_to_object->top_x,
                       arc_ptr->arc_to_object->top_y);

         if (node->bottom_y < arc_ptr->arc_to_object->top_y) {
            /* der Ausgangsknoten ist vollstaendig ueber dem Zielknoten */
            a = (double)(node->bottom_x - arc_ptr->arc_to_object->top_x);
            b = (double)(node->bottom_y - arc_ptr->arc_to_object->top_y);
            alpha = atan(a/b);

            /* Belegung des zweiten und dritten Koordinatenpaares */            
            point[1].x1 = (short) (arc_ptr->arc_to_object->top_x - 
                                   (sin(alpha + 0.5) * 10));
            point[1].y1 = (short) (arc_ptr->arc_to_object->top_y - 
                                   (cos(alpha + 0.5) * 10));
            point[1].x2 = (short) arc_ptr->arc_to_object->top_x; 
            point[1].y2 = (short) arc_ptr->arc_to_object->top_y;

            point[2].x1 = (short) arc_ptr->arc_to_object->top_x;
            point[2].y1 = (short) arc_ptr->arc_to_object->top_y;
            point[2].x2 = (short) (arc_ptr->arc_to_object->top_x - 
                                   (sin(alpha - 0.5) * 10));
            point[2].y2 = (short) (arc_ptr->arc_to_object->top_y - 
                                   (cos(alpha - 0.5) * 10));
            numb_segments = 3; 
            }
         }
    /* Zeichnen der Beziehung */
    XDrawSegments(display, drawingWindow, drawkit.writeGCs[ZOOM_100], 
                  point, numb_segments);
    }
}

 

void drawArcWeight(arc, from_x, from_y, to_x, to_y)
     GRAPHIC_ARC *arc;
     int from_x, from_y, to_x, to_y;
/*
 * AU: Zeichnen der Kantengewichtungen
 */
{
  int  middle_x, middle_y, distance_x, distance_y, i; /* Hilfsvariablen */
  char weight_string[12];    /* Ausgabezeile */
  char text[4];              /* Hilfsvariable */
  Bool weighted_arc = False; /* markiert die Existenz einer Belegung */

  /* Aufbau der Ausgabezeile */
  strcpy(weight_string, SPACE);  

  for (i = 0; i < data.numb_quor; i++) {
      if (arc->arc_value[i] != 0) {
         weighted_arc = True;
         sprintf(text, "%d",arc->arc_value[i]);
         strcat(weight_string, text); 
         }
      strcat(weight_string, COMMA); 
     }
  weight_string[strlen(weight_string) - 1] = EOS;


  if (weighted_arc) {
     /* Errechnung der Ausgabekoordinaten */
     distance_x = from_x - to_x;
     distance_y = from_y - to_y;

     middle_x = from_x - (distance_x / 2);
     middle_y = from_y - (distance_y / 2);

     if (distance_x > 0) middle_x = middle_x + 8;
     else                middle_x = middle_x - 8;

     if (distance_y > 0) middle_y = middle_y - 8;
     else                middle_y = middle_y + 8;

     /* Zeichnen der Ausgabe in Abhaengigkeit von der Zoomstufe */
     switch (zoomOption)
     {
            case ZOOM_100 :  
                 XDrawString(display, drawingWindow, 
                             drawkit.writeGCs[ZOOM_100], middle_x, middle_y,
                             weight_string, strlen(weight_string));
                 break;
            
            case ZOOM_75 :
                 XDrawString(display, drawingWindow,
                             drawkit.writeGCs[ZOOM_75], middle_x, middle_y,
                             weight_string, strlen(weight_string));
                 break;

            case ZOOM_50 :
                 XDrawString(display, drawingWindow,
                             drawkit.writeGCs[ZOOM_50], middle_x, middle_y,
                             weight_string, strlen(weight_string));
                 break;

            default :
                 XBell(display, 0);
                 wprint(unknown_event, "draw.c : drawArcWeight");
                 break;
     }
     }
  /* weight_string/test sind lokale arrays: XtFree(weight_string); XtFree(text); */

}


void redrawArea(GraphicObjectList)
     GRAPHIC_NODE *GraphicObjectList;
/*
 * AU: Zeichnen der Rechner und ihrer Kanten
 * ST: Aufruf draw100Lines(), draw75Lines(), draw50Lines(),
 *     drawStringsTerminal()
 */
{  
  static GRAPHIC_NODE *object; 

  /* Zeichnen der Beziehungen zwischen den Rechnern */  
  for(object = GraphicObjectList; object != NULL; object = object->next) {
     switch(zoomOption)
     {
           case ZOOM_100 :
                draw100Lines(object);
                break;

	   case ZOOM_75 :
                draw75Lines(object);
                break;

	   case ZOOM_50 :
                draw50Lines(object);
                break;
    
	   default :
                XBell(display, 0);
                wprint(unknown_event, "draw.c : redrawArea");
                break;
     }
     }

  /* Zeichnen der Rechnersymbole */
  for(object = GraphicObjectList; object != NULL; object = object->next) 
      drawStringsTerminal(object->pos_x, object->pos_y,
                          object->name, object->ident,
                          object->vote, object->quor,
                          object->version);
}


void clearArea()
/*
 * AU: Loeschen der Arbeitsflaeche
 */
{
  XClearWindow(display, drawingWindow); 
}


void delDependingExposeEvents()
/*
 * AU: Unterdruecken ueberzaehliger Events zum Wiederherstellen
 *     der Zeichnung im Arbeitsbereich 
 */
{
  int    event_type = ExposureMask;
  XEvent evt;
  
  while (0 != XCheckWindowEvent(display,drawingWindow,event_type,&evt)) 
        XtAppNextEvent(app_context, &evt);
}

/***************************************************************************/
/*                    Ende der Funktionen                                  */
/***************************************************************************/



