/*
Name:   draw.h
Zweck:  1. Aufnahme der Vorabdefinitionen von draw.c
        2. Deklarationen der Bitmap-Dateien
Autor:  Andreas Seifert
Datum:
Update:
*/

#ifndef _DRAW_H_
#define _DRAW_H_


/********************************************************/
/*  draw.h - header fuer draw.c                         */
/********************************************************/

#include <stdio.h>
#include <math.h>
#include <Xm/Xm.h>
#include <X11/cursorfont.h>


#include "struct.h"
#include "globals.h"


/* Definition des grossen Rechnersymbols */
#define term100_width 35
#define term100_height 35
static char term100_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xff, 0xff, 0x1f, 0x00, 0x10, 0x00,
   0x00, 0x60, 0x00, 0x10, 0xff, 0xff, 0xa3, 0x00, 0x90, 0x00, 0x00, 0x24,
   0x01, 0x50, 0x00, 0x00, 0x28, 0x02, 0x50, 0xfe, 0x01, 0x28, 0x02, 0x50,
   0x00, 0x00, 0x28, 0x02, 0x50, 0x1e, 0x00, 0x28, 0x02, 0x50, 0x00, 0x00,
   0x28, 0x02, 0x50, 0xfe, 0x1f, 0x28, 0x02, 0x50, 0x00, 0x00, 0x28, 0x02,
   0x50, 0x1e, 0x00, 0x28, 0x02, 0x50, 0x00, 0x00, 0x28, 0x02, 0x50, 0xfe,
   0x00, 0x28, 0x02, 0x50, 0x00, 0x00, 0x28, 0x02, 0x50, 0x06, 0x00, 0x28,
   0x02, 0x50, 0x00, 0x00, 0x28, 0x01, 0x50, 0x00, 0x00, 0x28, 0x01, 0x50,
   0x00, 0x00, 0xa8, 0x00, 0x90, 0x00, 0x00, 0xa4, 0x00, 0x10, 0xff, 0xff,
   0x63, 0x00, 0x10, 0x00, 0x00, 0x60, 0x00, 0xe0, 0xff, 0xff, 0x1f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xff, 0xff, 0x7f, 0x00, 0x10, 0x00,
   0x00, 0x60, 0x00, 0x50, 0x55, 0x55, 0x65, 0x00, 0x08, 0x00, 0x00, 0x48,
   0x00, 0xa8, 0xaa, 0xaa, 0x48, 0x00, 0x04, 0x00, 0x00, 0x24, 0x00, 0xb4,
   0xff, 0xbf, 0x15, 0x00, 0x02, 0x00, 0x00, 0x0a, 0x00, 0xfe, 0xff, 0xff,
   0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   


/* Definition des mittleren Rechnersymbols */
#define term75_width 25
#define term75_height 25
static char term75_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0xe0, 0xff, 0x0f, 0x00, 0x10, 0x00, 0x30, 0x00,
   0x90, 0xff, 0x53, 0x00, 0x50, 0x00, 0x94, 0x00, 0x50, 0x1f, 0x94, 0x00,
   0x50, 0x00, 0x94, 0x00, 0x50, 0x03, 0x94, 0x00, 0x50, 0x00, 0x94, 0x00,
   0x50, 0x07, 0x54, 0x00, 0x50, 0x00, 0x54, 0x00, 0x90, 0xff, 0x33, 0x00,
   0x10, 0x00, 0x30, 0x00, 0xe0, 0xff, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00,
   0xe0, 0xff, 0x3f, 0x00, 0x10, 0x00, 0x30, 0x00, 0x50, 0x55, 0x31, 0x00,
   0x08, 0x00, 0x24, 0x00, 0xa8, 0xaa, 0x24, 0x00, 0x04, 0x00, 0x12, 0x00,
   0xb4, 0xdf, 0x0a, 0x00, 0x02, 0x00, 0x05, 0x00, 0xfe, 0xff, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00};
			   

/* Definition des kleinen Rechnersymbols */
#define term50_width 18
#define term50_height 18
static char term50_bits[] = {
   0x00, 0x00, 0x00, 0xf0, 0x7f, 0x00, 0x08, 0x80, 0x01, 0xe8, 0xbf, 0x02,
   0x28, 0xa0, 0x02, 0xa8, 0xa3, 0x02, 0x28, 0xa0, 0x02, 0x28, 0xa0, 0x02,
   0xe8, 0xbf, 0x01, 0x08, 0x80, 0x00, 0xf0, 0x7f, 0x00, 0x00, 0x00, 0x00,
   0xf0, 0xff, 0x01, 0x08, 0x80, 0x01, 0xb4, 0xad, 0x00, 0x02, 0x40, 0x00,
   0xfe, 0x3f, 0x00, 0x00, 0x00, 0x00};

	       
	       

/* Vorabdefinitionen */
void        drawRubberRectangle(int, int);
void        drawRubberArc(int, int);
void        drawStringsTerminal(int, int, char text_name[40], char *, int, int a[], int);
void        draw100Lines(GRAPHIC_NODE *);
void        draw75Lines(GRAPHIC_NODE *);
void        draw50Lines(GRAPHIC_NODE *);
void        drawArcWeight(GRAPHIC_ARC *, int, int, int, int);
void        redrawArea(GRAPHIC_NODE *);
void        clearArea();
void        delDependingExposeEvents();
void        clean_Workbench();
void        setCursor(Widget, unsigned int);
void        initGraphik();
void        createGC();
void        printFontPath();
void        moveCursor(int, int);
Bool        loadZoomFonts();
Bool        createPixmap();
Bool        Intersect(int, int, int, int);

#endif /* _DRAW_H_ */


