/*
Name:   file.c
Zweck:  Einlesen, Pruefen der Eingabedateien und Abspeichern
        der Ergebnisse
Autor:  Andreas Seifert
Datum:
Update:
*/

#include "file.h"
#include "motif.h"
#include "graph.h"
#include "main.h"

/*
 * AU: Dateien einzulesen und zurueckschreiben
 *     Dieses Programm ermvglicht den Zugriff auf die Dateien:
 *     - Netztopologie-Beschreibungsdatei (Endung ".top")
 *     - Votingstruktur-Beschreibungsdatei (Endung ".vot")
 *     - Grafikdaten-Beschreibungsdatei (Endung ".drw")
 * ST: readFileSuffix()            Dateiendungen erkennen
 *     openFileIO()                Datei zur Ein- Ausgabe oeffnen
 *     closeFile()                 Datei schliesen
 *     loadObjectsFile()           Datei oeffnen und einlesen
 *     loadNetObjects()            Netztopologie lesen
 *     loadNetNode()               Knoten des Netzes lesen
 *     loadNetArc()                Verbindungen des Netzes lesen
 *     loadDrawObjects()           Grafikdatendatei lesen
 *     extractDrawAreaSize()       Groesse Arbeitsflaeche lesen
 *     loadDrawNode()              Rechnerkoordinaten lesen
 *     loadGraphicObjects()        Voting-Struktur lesen
 *     loadGraphicOperation()      Operationsbeschreibung lesen
 *     loadGraphicNode()           Rechner der Struktur lesen
 *     extractNodeVersion()        Versionsnummer ermitteln
 *     loadGraphicRepresentative() Repraesentanten lesen
 *     loadGraphicVertical()       Kooperationsrelationen lesen
 *     loadGraphicHorizontal()     Zugriffsreihenfolge lesen
 *     saveGraphicObjects()        Abspeichern Struktur und Grafikdaten
 *     saveGraphicFile()           Abspeichern Voting-Struktur
 *     saveGraphicNet()            Abspeichern Topologie
 *     saveGraphicOperation()      Abspeichern Operationsbeschreibung
 *     saveGraphicNode()           Abspeichern Voting-Struktur-Rechner
 *     saveGraphicRepresentative() Abspeichern Repraesentanten
 *     saveGraphicVertical()       Abspeichern Kooperationsrelation
 *     saveGraphicHorizontal()     Abspeichern Zugriffsreihenfolge
 *     saveDrawFile()              Abspeichern Grafikdaten
 *     saveDrawDimension()         Abspeichern Groesse Arbeitsbereich
 *     saveDrawNode()              Abspeichern Rechnerkoordinaten
 *     file_exists()               Existenz einer bestimmten Datei pruefen
 */



/****************************************************************************/
/****************                             *******************************/
/****************    Beginn der Funktionen    *******************************/
/****************                             *******************************/
/****************************************************************************/




TYPE_OF_FILE readFileSuffix(name)
             char *name; /* Dateiname */
/*
 * AU: Zeichenkette auf Dateiendungen untersuchen und den erkannten Typ
 *     zurueckliefern
 * NB: enthdlt die Zeichenkette keine der erwarteten Endungen, wird ein
 *     Default-Wert zurueckgeliefert
 */
{
 char *mark_string; /* Zeiger auf den Beginn der gesuchten Teilzeichenkette */

 mark_string = (char *)strstr(name, ".top");
 if (mark_string != NULL) { /* Netztopologiedatei */
    data.dateityp = top_file; 
    return top_file;
    }
 else { 
      mark_string = (char *)strstr(name, ".vot");
      if (mark_string != NULL) { /* Voting-Struktur-Datei */
         data.dateityp = vot_file;
         return vot_file;
         }
      else { /* ungueltiger Dateityp */
           data.dateityp = no_file;
           return no_file;
           }
      }
}


FILE *openFileIO(filename, mode, typeOfFile, file_length)
     char         filename[255]; /* Dateiname (ohne Endung) */
     char         *mode;         /* Lese- oder Schreibemodus */
     TYPE_OF_FILE typeOfFile;    /* Dateityp */
     int          *file_length;  /* Dateildnge */
/*
 * AU: Oeffnen der Datei, spezifiziert durch den Dateinamen filename,
 *     den beabsichtigten Lesemodus mode ("r" oder "w"), und den 
 *     Dateityp typeOfFile, Ermittlung der Dateilaenge und Rueckgabe 
 *     des Dateideskriptors
 * NB: im Fehlerfall wird NULL zurueckgeliefert
 */
{
 FILE *file;          /* Dateideskriptor */
 char openName[259];  /* vollstdndiger Dateiname */
 struct stat statbuf; /* Dateiinformationen */

 strncpy(openName, filename, 255);

 switch(typeOfFile)  
 {
       case top_file:
            strcat(openName, ".top");
            break;

       case drw_file:
            strcat(openName, ".drw");
            break;

       case vot_file:
            strcat(openName, ".vot");
            break;

       default:
            XBell(display, 0);
            wprint(unknown_event, "file.c : openFileIO");                      
            break;
 }
 
 /* Datei oeffnen */
 file = fopen(openName, mode);
 

 if (file == NULL) { /* Fehlerfall, Fehlercharakterisierung durch perrno */
    fprintf(stderr, errorMessage, "file.c", "openFileIO", "1");
    fprintf(stderr, "Oeffnen der Datei %s fehlerhaft\n",
                    openName);
    perror("Fehlernummer in errno(==1 fehlendes Zugriffsrecht)");
    }
 else { /* auslesen der Dateilaenge */
      if (stat(openName, &statbuf) == 0) *file_length = statbuf.st_size;
      else                               *file_length = 100000;
      }

 return file;
}


int closeFile(file)
    FILE *file; /* Dateideskriptor */
/* 
 * AU: schliesen der Datei mit dem Dateideskriptor file, Rueckgabe
 *     des Ergebniswertes 0
 * NB: im Fehlerfall wird eine Zahl ungleich 0 zurueckgegeben
 */
{
 int returnvalue = 0; /* Rueckgabewert */

 returnvalue  = fclose(file);

 if (returnvalue == EOF) { /* Fehlerfall, Charakterisierung durch perrno */
    fprintf(stderr, errorMessage, "file.c", "closeFile", "1");
    fprintf(stderr,"Schliessen der Datei fehlerhaft\n");
    fprintf(stderr,"siehe Errno-Fehlercodes in xman, SystemCalls, intro\n");
    perror("Fehlernummer in errno");
    }

 return returnvalue;
}


Bool loadObjectsFile(filename, typeOfFile)
     char         filename[255]; /* Dateiname */
     TYPE_OF_FILE typeOfFile;    /* Dateityp */ 
/*
 * AU: Datei einlesen und entsprechend der Dateiendung das Einlesen von
 *     Netztopologie-, Votingstruktur- oder Grafikdatendateien veranlassen
 *     das erfolgreiche Einlesen wird mit dem Rueckgabewert True dokumentiert
 * NB: das erfolglose Einlesen hat den Rueckgabewert False zur Folge        
 * ST: Aufruf openFileIO um den Filedeskriptor zu erhalten
 *     Aufruf loadNetObjectsFile fuer Netztopologiedateien
 *     Aufruf loadGraphicObjects fuer Votingstrukturdateien
 *     Aufruf loadDrawObjects fuer Grafikdatendateien
 *     Aufruf closeFile zum Schliesen der Datei
 */
{
  FILE        *file = NULL; /* Dateideskriptor */   
  char        buff[256];    /* Zwischenspeicher fuer eine Dateizeile */
  char        *line;        /* Zeiger auf ein Zeichen in buff */
  int         file_length;  /* Dateilaenge fuer Speicheranforderung */
  int         line_no = 0;  /* Zeilennummer zur Lokalisierung von Dateifehlern */
  int         node_no = 0;  /* Knotenanzahl fuer Plausibilitaetspruefung */
   
  /* Datei zum Lesen oeffnen, Dateilaenge und -deskriptor entnehmen */     
  if ((file = openFileIO(filename, MODE_READ_ONLY, typeOfFile, &file_length))
       == NULL) return False;
                           
  /* bei Votingstrukturdateien die Quorenzahl initialisieren */
  if (typeOfFile == vot_file) data.numb_quor = 0;
       
  /* File einlesen und Liste der graphischen Objekte aufbauen */
  while (fgets(buff, 256, file) != NULL) { /* liest eine Zeile der Datei file
                                            * nach buff ein und fuegt ein 
                                            * Zeilenendezeichen an */
         line = buff;
         line_no++;
                  
         /* Leerzeilen und Tabulatoren ueberlesen */
         while ((*line == BLANK) || (*line == TAB)) line++;
                                  
         if ((*line != LF) && (*line != SLASH)) { /* Line Feed und Kommentare
                                                   * ueberlesen */  
            switch(typeOfFile)
            {
                  case top_file: /* Netztopologiedatei */
                       if (!loadNetObjects(line, &node_no)) {
                          fprintf(stderr, errorMessage, "file.c", 
                                          "loadObjectsFile", "1");
                          fprintf(stderr,
                                  "Fehler Topologie-Datei: %s.top Zeile %d\n",
                                  filename, line_no);
                          return False; 
                          }
                       break;

                  case vot_file: /* Votingstrukturdatei */
                       if (!loadGraphicObjects(line, file, file_length)) {
                          fprintf(stderr, errorMessage, "file.c",
                                          "loadObjectsFile", "2");
                          fprintf(stderr,
                                  "Fehler Strategie-Datei: %s.vot Zeile %d\n",
                                  filename, line_no);
                          return False;
                          }
                       break;
                                  
                  case drw_file: /* Grafikdatendatei */
                       if (!loadDrawObjects(line)) {
                          fprintf(stderr, errorMessage, "file.c",
                                          "Fehler in loadObjectsFile", "3");
                          fprintf(stderr,
                                  "Fehler Koordinaten-Datei: %s.drw Zeile %d\n",
                                  filename, line_no);
                          return False;
                          }
                       break;

                  default: /* unerwartetes Ereignis */ 
                       XBell(display, 0);
                       wprint(unknown_event, "file.c : loadObjectsFile");
                       return False;
                       break;
            }   
            }
         }

  /* Datei schliesen */
  if (closeFile(file) != 0) return False;
    
  return True;
}


Bool loadNetObjects(input, node_no)
     char *input;   /* Zeiger auf den Beginn der Zeilenmarkierung */ 
     int  *node_no; /* Knotenzahl aus Plausibilitaetsgruenden */
/*
 * AU: eine Zeile der Datei untersuchen und durch Vergleich mit der 
 *     Zeilenmarkierung je nach identifiziertem Objekt eine Prozedur zum 
 *     Lesen dieses Objekts aufrufen. Beim erfolgreiche Einlesen eines 
 *     Objekts, beim Erkennen einer einleitenden Kopfzeile ("define") 
 *     und bei Zeilenanfang "R" (aus Kompatibiltaetsgruenden) wird True 
 *     zurueckgeliefert
 * NB: Fehler beim Lesen eines Objekts oder ein nicht erkanntes Objekt
 *     erzeugt Rueckgabe False
 * ST: Aufruf loadNetNode fuer Knoten
 *     Aufruf loadNetArc fuer Verbindungen     
 */
{
 if (strncmp(input, "NODE DESCRIPTION", 16) == 0) {
    return loadNetNode(input + 16, node_no);  
 } else if (strncmp(input, "ARC DESCRIPTION", 15) == 0) {
    return loadNetArc(input + 15, node_no);
 } else if (strncmp(input, "node description", 16) == 0) {
    return loadNetNode(input + 16, node_no);
 } else if (strncmp(input, "arc description", 15) == 0) {
    return loadNetArc(input + 15, node_no);
 } else if (strncmp(input, "define", 6) == 0) {
    return True;
 } else if (strncmp(input, "R", 1) == 0) {
    return True;
 } 
 return False;
}  



Bool loadNetNode(input, node_no)
     char *input;   /* Zeiger auf die Werte des Objekts */
     int  *node_no; /* Knotenanzahl aus Plausibilitaetsgruenden */
/*
 * AU: Einlesen der Objektdaten fuer einen Knoten, Einfuegen des Knotens 
 *     in die Liste der Knoten und Erhoehung der Anzahl gelesener Knoten
 *     Rueckgabe im Normalfall True
 * NB: Fehler beim Einlesevorgang oder beim Einfuegen des Knotens erzeugt
 *     False als Rueckgabe
 * ST: Aufruf addNetNode (graph.c)
 */
{
 int    match;                /* Kontrolle der Anzahl eingelesener Werte */ 
 float  reliability;          /* Knotenausfallwahrscheinlichkeit */
 char   ident[40];            /* eindeutige Rechnerbezeichnung */
 char   name[40];             /* frei vergebener Rechnername */
 char   dummy[40];            /* Kommentar zur besseren Lesbarkeit */
 NODE   *current_node = NULL; /* Zeiger auf Knoten in der Liste */
 
 match = sscanf(input, "%s %s %s %f",
                       ident,
                       dummy,
                       name,
                       &reliability);

 if (match != 4) return False; /* Pruefung Anzahl eingelesener Werte */
        
 /* Einfuegen des gelesenen Knotens in die Knotenliste */
 current_node = (NODE *)addNetNode(ident, name, reliability);

 if (current_node == NULL) return False;
            
 /* Anzahl gelesener Knoten erhoehen */
 *node_no = *node_no + 1;

 return True;
}



Bool loadNetArc(input, node_no)
     char *input;   /* Zeiger auf die Werte des Objekts */
     int  *node_no; /* Knotenanzahl aus Plausibilitaetsgruenden */
/*
 * AU: Einlesen der Objektdaten fuer Verbindungen(Kanten) zwischen dem Knoten 
 *     "sourcenode" und den verbundenen Knoten "currnode", Einfuegen der 
 *     Verbindung in die Liste der mit "sourcenode" verbundenen Knoten, 
 *     Pruefung auf Uebereinstimmung zwischen der Anzahl existierender Knoten und 
 *     der Anzahl Knoten, zwischen denen hier eine Verbindung hergestellt 
 *     werden kann.
 *     Die Identifikation und Existenz einer Verbindung ist an die Position in
 *     der Zeichenkette und an die Wertebelegung geknuepft. Die Zeichenkette 
 *     besitzt in der ersten Position den Ausgangsknoten und danach durch 
 *     Leerzeichen getrennt, die Wertebelegungen fuer alle Knoten in der 
 *     Reihenfolge ihres Einlesens. 
 *     Dabei indiziert eine Belegung:
 *     - "0" es existiert keine Verbindung
 *     - "1" keine Kante mit gleichem Ausgangs- und Endknoten
 *     - sonst, Ausfallswahrscheinlichkeit der Kante
 *     Die Zeichenkette wird bei jedem Auftreten eines Leerzeichens in 
 *     Teilstrings (Token) zerlegt
 *     Rueckgabewert im Normalfall True
 * NB: Fehler im Aufbau der Zeichenkette, nicht vorhandene Knoten, 
 *     Abweichungen zwischen den im Netz vorhandenen Knoten und den fuer eine 
 *     Verbindung vorgesehenen Knoten erzeugen den Rueckgabewert False
 * ST: Aufruf correspondingNetNode_exist (graph.c)
 *     Aufruf insertNetEdge (graph.c)   
 */ 
{
 float  reliability;                       /* Kantenausfallwahrscheinlichkeit */
 char   ident[40];                         /* eindeutiger Ausgangsrechnername */
 char   *word;                             /* Zeiger auf Zeichenkettenelemente */
 NODE   *sourceNode;                       /* Zeiger auf Ausgangsrechner */
 NODE   *currNode = ptrNetObjectListFront; /* Zeiger auf verbundenen Rechner */
 int    numbNodes = 0;                     /* Anzahl fuer Verbindungen 
                                            * bereitstehender Rechner */ 

 /* Bezeichnung des Ausgangsknoten auslesen */
 word = (char *)strtok(input, SPACETAB);
 
 if (word != NULL) {
    strcpy(ident, word);

    /* Pruefung, ob der Ausgangsknoten existiert */
    sourceNode = (NODE *)correspondingNetNode(word);
    if (sourceNode == NULL) return False;

    word = (char *)strtok(NULL, SPACETAB);
    }
 else return False;

 /* Wertebelegungen zu den anderen Knoten auslesen */
 while (word != NULL) {
        /* Umwandlung Zeichenkette in eine Gleitpunktzahl*/
        reliability = atof(word); 

        /* Verbindungen besitzen einen Wert zwischen 0 und 1 */
        if ((currNode != NULL) && (reliability < 1) && (reliability > 0)) {
            insertNetEdge(sourceNode, currNode, reliability);
           }
        if (currNode == NULL) { 
           wprint(mismatchArcNode);
           return False;
           }
        currNode = currNode->nextnode;
        numbNodes++;
        word = (char *)strtok(NULL, SPACETAB);
       }

 /* Pruefung, ob die in loadNetNode ermittelte Knotenanzahl mit der hier 
  * ermittelten Anzahl uebereinstimmt */
 if (numbNodes != *node_no) {
    fprintf(stderr, errorMessage, "file.c", "loadNetArc", "1");
    fprintf(stderr, "Anzahl Kanten widerspruechlich\n"); 
    return False;
    }

 return True;
}
 
 

Bool loadDrawObjects(input)
     char *input; /* Zeiger auf den Beginn der Zeilenmarkierung*/
/* 
 * AU: eine Zeile der Datei untersuchen und durch Vergleich mit der
 *     Zeilenmarkierung entweder das Einlesen der Koordinaten eines 
 *     Votingstrukturrechners oder die Entnahme der benoetigten Groesse 
 *     der Zeichenfldche veranlassen
 *     Rueckgabewert True
 * NB: eine nicht vorhergesehe Zeilenmarkierung oder Fehler beim Einlesen 
 *     von Werten fuehren zum Rueckgabewert False
 * ST: Aufruf loadDrawNode fuer Knotenkoordinaten
 *     Aufruf loadNetArc fuer Groesse der Zeichenflaeche
 */
{
  if (strncmp(input, "KNOTEN", 6) == 0) {
     return loadDrawNode(input + 6);
  } else if (strncmp(input, "DRAWING_AREA_SIZE", 17) == 0) {
     return extractDrawAreaSize(input + 17);
  }
  return False;
}
 

Bool extractDrawAreaSize(input)
     char *input; /* Zeiger auf den Beginn der Informationen */
/*
 * AU: Einlesen der Dimensionen der Zeichenflaeche, Rueckgabe im
 *     Normalfall True
 * NB: Fehler beim Lesen der Daten erzeugt Rueckgabewert False
 */
{        
 int  match; /* Kontrolle der Anzahl eingelesener Werte */
        
 match = sscanf(input, "%d %d",
                       &data.draw_xsize, &data.draw_ysize);
 
 if (match != 2) return False; /* Pruefung der Anzahl eingelesener Werte */
 
 return True;   
}


Bool loadDrawNode(input)
     char *input; /* Zeiger auf den Beginn der Informatioen */
/*
 * AU: Einlesen der Koordinaten eines Votingstrukturrechners
 *     Die Adresse des Votingstrukturrechners wird ermittelt und
 *     seine Koordinaten belegt
 *     Rueckgabewert False
 * NB: Koordinaten fuer einen nicht existierenden Rechner fuehren
 *     zum Rueckgabewert False
 * ST: Aufruf correspondingGraphicNode_exist (graph.c)
 *     Aufruf change_GraphicObject (graphic.c)
 */    
{
 int          match;              /* Kontrolle der Anzahl eingelesener Werte */
 int          pos_x, pos_y;       /* X-, Y-Koordinate */
 char         ident[40];          /* eindeutige Votingstrukturrechnerbezeichnung */
 GRAPHIC_NODE *nodeadress = NULL; /* Zeiger auf Votingstrukturrechner */
 
 match = sscanf(input, "%s %d %d",
                       ident, 
                       &pos_x,   
                       &pos_y);  

 if (match != 3) return False; /* Pruefung der Anzahl eingelesener Werte */
  
 /* Durchsuchen der Liste existierender Votingstrukturrechner */    
 nodeadress = (GRAPHIC_NODE *)correspondingGraphicNode(ident);
                       
 if (nodeadress != NULL)  { /* Votingstrukturrechner existiert */
    change_GraphicObject(pos_x, pos_y, nodeadress); /* Koordinaten belegen */
    return True;
    }

 /* Fehlerfall */
 fprintf(stderr, errorMessage, __FILE__, __FUNCTION__, "1");
 fprintf(stderr, 
         "Zeicheninformationen vorhanden Knoten %s fehlt in Datei *.vot\n",
         ident);

 return False;
}
     



Bool loadGraphicObjects(input, file, file_length)
     char *input;      /* Zeiger auf den Beginn der Zeilenmarkierung */   
     FILE *file;       /* Dateideskriptor */
     int  file_length; /* Dateilaenge */
/*
 * AU: eine Zeile der Datei untersuchen und durch Vergleich mit der
 *     Zeilenmarkierung je nach identifiziertem Objekt eine Prozedur zum
 *     Lesen des Objekts aufrufen. Beim Erfolgreichen Lesen eines Objekts, 
 *     beim Erkennen zu ueberlesender Zeilenmarkierungen (aus 
 *     Kompatibilitaetsgruenden) wird True zurueckgeliefert
 *     Als zugelassene Objekte gelten:
 *     - Operationsbeschreibung
 *     - Votingstrukturrechnerbeschreibung
 *     - Beschreibung eines virtuellen Votingstrukturrechner
 *     - Beschreibung der Kooperationsrelationen
 *     - Beschreibung der Belegung der Kooperationsrelationen
 * NB: Fehler beim Lesen eines Objekts oder eine nicht zugelassene
 *     Zeilenmarkierung fuehrt zum Rueckgabewert False
 * ST: Aufruf loadGraphicOperation
 *     Aufruf loadGraphicNode
 *     Aufruf loadGraphicRepresentative
 *     Aufruf loadGraphicVertical
 *     Aufruf loadGraphicHorizontal
 */
{
 if (strncmp(input, "define netname", 14) == 0) {
    return True;
 } else if (strncmp(input, "NODE DESCRIPTION", 16) == 0) {
    return True;
 } else if (strncmp(input, "node description", 16) == 0) {
    return True;
 } else if (strncmp(input, "define operation", 16) == 0) {
    return True;
 } else if (strncmp(input, "define node", 11) == 0) {
    return True;
 } else if (strncmp(input, "define representative", 21) == 0) {
    return True;
 } else if (strncmp(input, "define vertical", 15) == 0) {
    return True;
 } else if (strncmp(input, "OP DESCRIPTION", 14) == 0) {
    return loadGraphicOperation(input + 14);
 } else if (strncmp(input, "op description", 14) == 0) {
    return loadGraphicOperation(input + 14);
 } else if (strncmp(input, "OBJECT DESCRIPTION", 18) == 0) {
    return loadGraphicNode(input + 18);
 } else if (strncmp(input, "object description", 18) == 0) {
    return loadGraphicNode(input + 18);
 } else if (strncmp(input, "VIRTUAL DESCRIPTION", 19) == 0) {
    return loadGraphicRepresentative(input + 19);
 } else if (strncmp(input, "virtual description", 19) == 0) {
    return loadGraphicRepresentative(input + 19);  
 } else if (strncmp(input, "TREE DESCRIPTION", 16) == 0) {
    return loadGraphicVertical(input + 16);
 } else if (strncmp(input, "tree description", 16) == 0) {
    return loadGraphicVertical(input + 16);
 } else if (strncmp(input, "define horizontal", 17) == 0) {  
    return loadGraphicHorizontal(input + 17, file, file_length);
 } else if (strncmp(input, "end", 3) == 0) {
    return True;
 }
 return False;
}


   
Bool loadGraphicOperation(input)
     char *input; /* Zeiger auf den Beginn der Information */
/*
 * AU: eine Zeile der Datei untersuchen, die Bezeichnung der Operation
 *     entnehmen und die Zahl der Operationen erhoehen
 *     normaler Rueckgabewert True
 * NB: die Anzahl der Zeilenmarkierungen wurde zu Beginn des Einlesevorgangs
 *     auf 0 gesetzt (Prozedur loadObjectsFile) und nun beim Einlesen jeder 
 *     neuen Operation angepasst
 *     Fehler beim Einlesevorgang erzeugt False als Rueckgabe
 */
{
  int match; /* Kontrolle der Anzahl eingelesener Werte */

  match = sscanf(input, "%s",
                        data.operation[data.numb_quor]);  
                       
 if (match != 1) return False; /* Pruefung der Anzahl eingelesener Werte */

 data.numb_quor++; /* Operationszaehler anpassen */ 

 return True;
}


Bool loadGraphicNode(input)
     char *input; /* Zeiger auf den Beginn der Informationen */
/*
 * AU: Einlesen der Daten eines Votingstrukturrechner in Abhaengigkeit
 *     von der festgestellten Quorenanzahl
 *     Rueckgabe im Normalfall True
 * NB: Quorenanzahl wurde in loadGraphicOperation korrekt festgestellt
 *     Fehler beim Lesen der Daten, ungueltige Quorenanzahl(<1 oder >5),
 *     Widersprueche in der Bezeichnung der Rechnerart, Votingstrukturrechner,
 *     die nicht als virtuelle Rechnerbezeichnet sind, allerdings auch keinen 
 *     korrespondierenden Rechner in der Netztopologie haben, fuehren zum
 *     Rueckgabewert False
 * ST: Aufruf extractNodeVersion
 *     Aufruf correspondingNetNode_exist (graph.c)
 *     Aufruf add_GraphicObject (graph.c)
 *     Aufruf add_GraphicVoteQuor (graph.c)
 *     Aufruf markListobject (main.c) 
 */  
{
 int  match;                             /* Kontrolle Werteanzahl */
 int  vote = 0;                          /* Stimmenanzahl */
 int  version = 0;                       /* Rechnerart <>0 == virtuell */
 int  quor1, quor2, quor3, quor4, quor5; /* Quoren */
 char ident[40];                         /* eindeutige Rechnerbezeichnung */  
 char dummy[40];                         /* Kommentar zur besseren Lesbarkeit */
 char nodetype[40];                      /* Zeichenkette Rechnerart */
 NODE *netNode = NULL;                   /* Zeiger auf den Rechners im Netz */

 switch(data.numb_quor) /* entsprechend der Quorenanzahl die Daten eins 
                         * Votingstrukturrechner einlesen */
 {
       case 1 : /* Quorenanzahl == 1 */
            match = sscanf(input, "%s %s %s %d %d",
                                  ident,
                                  dummy,
                                  nodetype,
                                  &vote,
                                  &quor1);
 
            if (match != 5) return False; /* Pruefung der Werteanzahl */
            break;

       case 2 : /* Quorenanzahl == 2 */
            match = sscanf(input, "%s %s %s %d %d %d",
                                  ident,
                                  dummy,
                                  nodetype,  
                                  &vote,  
                                  &quor1,
                                  &quor2);
            
            if (match != 6) return False; /* Pruefung der Werteanzahl */
            break;
  
       case 3 : /* Quorenanzahl == 3 */
            match = sscanf(input, "%s %s %s %d %d %d %d",
                                  ident,
                                  dummy,
                                  nodetype,  
                                  &vote,  
                                  &quor1,
                                  &quor2, 
                                  &quor3);
            
            if (match != 7) return False; /* Pruefung der Werteanzahl */
            break;
  
       case 4 : /* Quorenanzahl == 4 */
            match = sscanf(input, "%s %s %s %d %d %d %d %d",
                                  ident,
                                  dummy,
                                  nodetype,  
                                  &vote,  
                                  &quor1,  
                                  &quor2,   
                                  &quor3,
                                  &quor4);
            
            if (match != 8) return False; /* Pruefung der Werteanzahl */
            break;
  
       case 5 : /* Quorenanzahl == 5 */
            match = sscanf(input, "%s %s %s %d %d %d %d %d %d",
                                  ident,
                                  dummy,
                                  nodetype,  
                                  &vote,  
                                  &quor1, 
                                  &quor2, 
                                  &quor3,
                                  &quor4,
                                  &quor5);
            
            if (match != 9) return False; /* Pruefung der Werteanzahl */
            break;
  
       default : /* Fehlerfall, unerwartete Quorenanzahl */
            XBell(display, 0);
            wprint(unknown_event, "file.c : loadGraphicNode");
            return False;
            break;
 }

 /* Feststellung, ob es sich um einen virtuellen Rechner handelt, indiziert durch
  * die Zeichenkette "virtual" und eine Versionsnummer groesser 0 */
 if (strncmp(nodetype, "virtual", 7) == 0) { 
    version = extractNodeVersion(ident); /* vergebene Versionsnummer entnehmen */

    if (version == 0) { /* Widerspruch zwischen Rechnertyp und Versionsnummer */
       fprintf(stderr, "%s:%s:%s: ", "file.c", "loadGraphicNode", "1");
       fprintf(stderr, "Virtuelle Rechner mit Versionsnummer == 0\n");
       return False;
       }
    } 
 else version = 0; /* Belegung der Versionsnummer fuer nicht virtuelle Rechner */

 netNode = (NODE *)correspondingNetNode(ident); /* Suche des Knoten in der
                                                 * Liste der Netztopologie */ 
 
 if (netNode != NULL) { /* Vermutung fuer einen nicht virtuellen Rechner */ 

    /* einfuegen des Votingstrukturrechners in die Liste der Votingstruktur */
    add_GraphicObject(netNode->name, ident, version); 

    /* Belegung der Stimmen- und Quorenanzahl */
    add_GraphicVoteQuor(ptrGraphicObjectList,                     
                        vote, quor1, quor2, quor3, quor4, quor5);  

    /* der zum Votingstrukturrechner korrespondierende Netz-Rechner wird in
     * der Rechnerliste der Benutzeroberflaeche als ausgewaehlt gekennzeichnet */ 
    if(!markListobject(ptrGraphicObjectList)) {
      fprintf(stderr, errorMessage, "file.c", "loadGraphicNode", "2");
      fprintf(stderr, "Listenelement zum Knoten %s nicht gefunden\n",
                      ident);
      return False;
      }
    return True;
    }
 else { /* Vermutung fuer einen virtuellen Rechner */
      if (version != 0) { /* Pruefung, ob die Versionsnummer <>0 fuer einen 
                           * virtuellen Rechner eingehalten wurde */

         /* einfuegen des virtuellen Votingstrukturrechners in die Liste der 
          * Votingstrukturrechner */
         add_GraphicObject("virtual", ident, version); 

         /* Belegung der Stimmen- und Quorenanzahl */
         add_GraphicVoteQuor(ptrGraphicObjectList,                   
                            vote, quor1, quor2, quor3, quor4, quor5); 
                               
         return True; 
         }
      else { /* Fehlerfall */
           fprintf(stderr, errorMessage, "file.c", "loadGraphicNode", "3");
           fprintf(stderr, "Listenelement zum Knoten %s nicht gefunden\n",
                           ident); 
           return False;
           }
      }
}


int extractNodeVersion(ident)
    char ident[]; /* Zeiger auf die eindeutige Bezeichnung des virtuellen Rechners */
/*
 * AU: aus der Rechnerbezeichnung fuer einen virtuellen Votingstrukturrechner
 *     wird die durch "[" und "]" eingeschlossene Versionsnummer ermittelt
 */
{
  char cpy[41];
  char *firstptrchr, *lastptrchr;
  int  returnvalue;

  strcpy(cpy, ident);
  firstptrchr = (char *)strchr(cpy, '[');
  firstptrchr++;
  lastptrchr = (char *)strchr(cpy, ']');
  *lastptrchr = EOS;
  returnvalue = atoi(firstptrchr);

  return returnvalue;    
}



Bool loadGraphicRepresentative(input)
     char *input; /* Zeiger auf den Beginn der Informationen */
/*
 * AU: fuer jeden virtuellen Votingstrukturrechner wird die durch Leerzeichen 
 *     unterbrochene Folge seiner Repraesentanten eingelesen, zerlegt und die 
 *     Repraesentantenliste aufgebaut. Das Ende der Repraesentantenliste wird 
 *     durch ";" signalisiert
 *     Rueckgabewert im Normalfall oder beim Hinweis "empty" True
 * NB: erste Information bezeichnet den virtuellen Rechner, fuer den die
 *     Repraesentanten angegeben werden
 *     zweites Element ist eine Zeichenfolge ohne Leerzeichen zur besseren 
 *     Kommentierung der Zeile
 *     die restlichen Informationen bezeichnen jeweils einen der Repraesentanten
 *     Im Falle eines nicht bekannten virtuellen Votingstrukturrechners lautet
 *     der Rueckgabewert False
 *     eine nicht bewilligte Speicheranforderung hat die Beendigung des  
 *     Programms zur Folge
 * ST: Aufruf correspondingGraphicNode_exist (graph.c)
 *     Aufruf addRepresentatives (graph.c)
 *     Makro demand (macro.h)
 */
{
 char          word[40];           /* Aufnahme der Elemente der Zeichenkette */  
 char          *ptrchr;            /* Zeiger auf Elemente der Zeichenkette */
 int           position = 1;       /* Identifikation der beiden ersten Elemente */
 GRAPHIC_IDENT *identTemp = NULL;  /* Zeiger auf Element der Repraesentantenliste */
 GRAPHIC_IDENT *identFront = NULL; /* Zeiger auf Beginn der Repraesentantenliste */
 GRAPHIC_IDENT *identTail = NULL;  /* Zeiger auf Ende der Repraesentantenliste */
 GRAPHIC_NODE  *nodeadress = NULL; /* Zeiger auf den virtuellen Votingstruktur-
                                    * rechner in der Votingstrukturliste */

 ptrchr = (char *)strtok(input, SPACETAB); /* erstes Element erkennen */ 

 /* die Markierung "empty" bezeichnet eine leere Repraesentantenliste */
 if ((char *)strstr(ptrchr, "empty") != NULL) return True;
 
 /* Zerlegen der Zeichenkette beim Auftauchen von Leerzeichen in Tokens */
 while (ptrchr != NULL) {
       strncpy(word, ptrchr, 40); /* Kopie fuer die Verarbeitung */
      
       switch (position)
       {
              case 1 : /* Bezeichnung des virtuellen Votingstrukturrechners*/

                   /* Existenz des Rechners ueberpruefen */
                   nodeadress = (GRAPHIC_NODE *)
                                correspondingGraphicNode(word);

                   if (nodeadress == NULL) { /* Votingstrukturrechner existiert
                                              * nicht in der Votingstrukturliste */
                      fprintf(stderr, errorMessage, "file.c", 
                                      "loadGraphicRepresentative", "1");
                      fprintf(stderr, 
                              "Knoten %s kein Rechnerstrukturmitglied\n",
                              word);
                      return False;
                      } 
                   position++;
                   break;

              case 2 : /* Kommentar ueberspringen */
                   position++;
                   break;

              case 3 : /* Elemente an Position >2 sind Repraesentanten */
                   if ((char *)strchr(word, ';') == NULL) { /* das Ende der Folge
                                                             * der Repraesentanten
                                                             * wurde noch nicht
                                                             * erreicht
                                                             */
                    
                      /* neues Element der Repraesentantenliste erzeugen */
                      identTemp = (GRAPHIC_IDENT *)
                                  XtMalloc(sizeof(GRAPHIC_IDENT));

                      demand(identTemp,alloc loadGraphicRepresentative failed);
                      strncpy(identTemp->identItem, word, 4);

                      /* neues Element in die Repraesentantenliste einfuegen */
                      if (identFront == NULL) identFront = identTemp;
                      else identTail->nextListIdent = identTemp;

                      identTail = identTemp;
                      identTemp->nextListIdent = NULL;
                      }
                   break;     
       }
       /* Beginn des ndchsten Tokens ermitteln, sonst Rueckgabe NULL */
       ptrchr = (char *)strtok(NULL, SPACETAB);
       }
 /* Zeiger auf Repraesentantenliste an den Votingstrukturrechner uebergeben */
 addRepresentatives(nodeadress, identFront);
  
 return True;
} 



Bool loadGraphicVertical(input)
     char *input; /* Zeiger auf den Beginn der Informationen */
/*
 * AU: fuer jeden Votingstrukturrechner werden die mit ihm in einer 
 *     Kooperationsrelationen stehenden Votingstrukturrechner eingelesen
 *     Rueckgabewert im Normalfall oder beim Hinweis "empty" True
 * NB: die erste Information bezeichnet den Votingstrukturrechner von
 *     dem die Kooperationsrelation ausgeht
 *     zweites Element ist eine Zeichenfolge ohne Leerzeichen zur besseren
 *     Kommentierung der Zeile
 *     die restlichen Informationen kennzeichnen die durch Leerzeichen 
 *     unterbrochene Folge der Votingstrukturrechner
 *     das Ende ist durch ";" signalisiert
 *     Sind Votingstrukturrechner der Kooperationsrelation nicht bekannt,
 *     so lautet der Rueckgabewert False
 * ST: Aufruf correspondingGraphicNode_exist (graph.c) 
 *     Aufruf insertArc (graph.c)
 */
{
 char          word[40];     /* Aufnahme der Elemente der Zeichnekette */
 char          *ptrchr;      /* Zeiger auf Elemente der Zeichenkette */
 int           position = 1; /* Identifikation der ersten beiden Elemente */
 GRAPHIC_NODE  *startNode;   /* Zeiger auf Ausgangsknoten */
 GRAPHIC_NODE  *targetNode;  /* Zeiger auf Zielknoten */
                      
 ptrchr = (char *)strtok(input, SPACETAB); /* erstes Element erkennen */
 
 /* die Markierung "empty" bezeichnet das Fehlen einer Kooperationsrelation */
 if ((char *)strstr(ptrchr, "empty") != NULL) return True;

 /* Zerlegen der Zeichenkette beim Auftauchen von Leerzeichen in Tokens */
 while (ptrchr != NULL) {
       strncpy(word, ptrchr, 40); /* Kopie fuer die Verarbeitung */
               
       switch (position) 
       {
              case 1 : /* Bezeichnung den initiierenden Votingstrukturrechner */

                   /* Existenz des Rechners ueberpruefen */
                   startNode = (GRAPHIC_NODE *)
                               correspondingGraphicNode(word);

                   if (startNode == NULL) {
                      fprintf(stderr, errorMessage, "file.c",
                                      "loadGraphicVertical", "1");
                      fprintf(stderr,
                              "Knoten %s kein Rechnerstrukturmitglied\n",
                              word);
                      return False;
                      }
                   position++;
                   break;
 
              case 2 : /* Kommentar ueberspringen */
                   position++;
                   break;

              case 3 : /* Elemente an Positionen > 2 sind die Votingstrukturrechner
                        * die zur Kooperation aufgefordert werden */
                   if ((char *)strchr(word, ';') == NULL) { /* das Ende der Folge
                                                             * wurde noch nicht
                                                             * erreicht
                                                             */

                      /* Existenz des Rechners ueberpruefen */
                      targetNode = (GRAPHIC_NODE *)
                                   correspondingGraphicNode(word);

                      if (targetNode == NULL) {
                         fprintf(stderr, errorMessage, "file.c", 
                                         "loadGraphicVertical", "2"); 
                         fprintf(stderr,
                                "Knoten %s kein Rechnerstrukturmitglied\n",
                                word);
                         return False;
                         }
                      /* Kooperationsrelation einfuegen */
                      insertArc(startNode, targetNode);
                      }
                   break;
       }
       /* Beginn des naechsten Tokens ermitteln, sonst Rueckgabe NULL */
       ptrchr = (char *)strtok(NULL, SPACETAB);
       }           
 return True;
}



Bool loadGraphicHorizontal(input, file, file_length)
     char *input;      /* Zeiger auf den Beginn der Informationen */
     FILE *file;       /* Dateideskriptor */
     int  file_length; /* Dateilaenge */
/*
 * AU: fuer jede Kooperationsrelation zwischen Votingstrukturrechnern
 *     und jeden Dienst wird die Belegung  eingelesen 
 *     Rueckgabewert im Normalfall oder beim Hinweis "empty" True
 * NB: die Beschreibung der Kooperationsrelationen umfasst den gesamten
 *     Rest der Datei
 *     Beschreibungsbeginn mit der Bezeichnung der Operation
 *     die Beschreibung fuer jede einzelne Operation ist in "(" und ")"
 *     eingeschlossen
 *     die Beschreibung der Dienste ist in "(" und ")" eingeschlossen, d.h.
 *     eine komplette Operationsbeschreibung beginnt nach dem Namen der 
 *     Operation mit "((" und endet mit dem ersten Auftauchen des Zeichens "))"
 *     ist die Beschreibung der Kooperationsrelation nicht eindeutig, tritt ein 
 *     Fehler beim Lesen der Informationen auf, ist einer der an der
 *     Kooperationsrelation beteiligten Votingstrukturrechner nicht bekannt, 
 *     oder soll eine Kooperationsrelation fuer eine nicht existierende
 *     Beziehung hergestellt werden, lautet der Rueckgabewert False
 *     eine nicht bewilligte Speicheranforderung hat die Beendigung des 
 *     zur Folge
 * ST: Aufruf correspondingGraphicNode_exist (graph.c)
 *     Aufruf setGraphicArcWeight (graph.c)
 *     Makro demand (macro.h)
 */
{
 int           i = 0;                  /* Laufvariable fuer jede Kantenbelegung */
 int           match;                  /* Kontrolle der Anzahl eingelesener Werte */
 int           arc_weight;             /* Belegung eines einzelnen Dienstes innerhalb
                                        * der Kooperationsrelation */
 char          *opptr;                 /* Zeiger auf den Beginn der Beschreibung einer
                                        * Kooperationsrelation */
 char          *endptr;                /* Zeiger auf das Ende einer einzelnen Belegung */
 unsigned char *file_string = NULL;    /* Zeiger auf den Beginn aller 
                                        * Kooperationsrelationen */
 char          startIdent[40];         /* eindeutige Bezeichnung des Ausgangsknoten */
 char          targetIdent[40];        /* eindeutige Bezeichnung des Zielknoten */
 char          dummy1[40], dummy2[40]; /* Aufnahme von Kommentarzeichen */
 GRAPHIC_NODE  *startNode;             /* Zeiger auf Ausgangsknoten */
 GRAPHIC_NODE  *targetNode;            /* Zeiger auf Zielknoten */

 /* die Markierung "empty" bezeichnet das Fehlen einer Kooperationsrelation */
 if ((char *)strstr(input, "empty") != NULL) return True;

 /* Speicherplatzanforderung mind. in Dateigroesse */
 file_string = (unsigned char *) XtMalloc((unsigned)++file_length);

 demand(file_string, memory allocation in loadGraphicHorizontal failed);

 /* Einlesen der Kooperationsrelationenbeschreibung */
 fread(file_string, sizeof(char), file_length, file);

 /* Ende der Beschreibung */
 file_string[file_length] = '\0';

 /* fuer jeden Dienst */
 while (i < data.numb_quor) {
       /* Name der Operation erkennen */
       opptr = (char *)strstr(file_string, data.operation[i]);

       if (opptr != NULL) {
          /* Beginn der Operationsbeschreibung erkennen */
          opptr = (char *)strstr(opptr, "((");

          if (opptr != NULL) opptr++;
          else { /* Fehlerfall */
               fprintf(stderr, errorMessage, "file.c",
                               "loadGraphicHorizontal", "1");
               fprintf(stderr, " '((' erwartet\n");
               return False;
               }

          do { /* fuer jeden einzelnen Dienst eine Beschreibung auslesen */ 
             opptr = (char *)strstr(opptr, "(");
             opptr++;
             endptr = (char *)strstr(opptr, ")");
             *endptr = '\0';
             match = sscanf(opptr, "%s %s %s %s %d",
                                   startIdent,
                                   dummy1,
                                   targetIdent,
                                   dummy2,
                                   &arc_weight);
             
             if (match != 5) return False; /* Pruefung der Werteanzahl */

             /* Ausgangsknoten ermitteln */
             startNode = (GRAPHIC_NODE *)
                               correspondingGraphicNode(startIdent);
             /* Zielknoten ermitteln */
             targetNode = (GRAPHIC_NODE *)
                                   correspondingGraphicNode(targetIdent);

             /* Existenz der Votingstrukturrechner ueberpruefen */
             if ((startNode == NULL) || (targetNode == NULL)) {
                fprintf(stderr, errorMessage, "file.c",
                                "loadGraphicHorizontal", "2");
                fprintf(stderr,
                        "%s oder %s keine Rechnerstrukturmitglieder\n",
                        startIdent, targetIdent);
                return False;
                }

             /* Belegung der Kooperationsrelation fuer Dienst Nr. i veranlassen */
	     /* !!! setGraphicArcWeight hat rueckgabe void !!! */
             if (!setGraphicArcWeight(startNode, targetNode, i, arc_weight)) {
                fprintf(stderr, errorMessage, "file.c", 
                                "loadGraphicHorizontal", "3");
                fprintf(stderr, "Belegung fuer nicht existierende Kante");
                return False;
                }

             *endptr = ')';
             endptr++;
             opptr = endptr;             
             }
          while (strncmp(endptr, ")", 1) != 0);
          }
       i++;
       }
 XtFree(file_string);

 return True; 
}


Bool saveGraphicObjects(filename, quor_numb, area_sizex, area_sizey)
     char filename[255];                     /* Dateiname (ohne Endung) */
     int  quor_numb;                         /* Quorenanzahl */
     int  area_sizex, area_sizey;            /* Groesse der Zeichenflaeche */
/*
 * AU: Abspeichern der Votingstrukturdatei und der dazugehoerigen
 *     Grafikdatendatei
 *     erfolgreiches Abspeichern erzeugt Rueckgabewert True
 * NB: Fehler beim Apspeichern erzeugen False als Rueckgabe
 * ST: Aufruf saveDrawFile
 *     Aufruf saveGraphicFile
 */
{
  /* Abspeichern Grafikdatendatei */
  if (!saveDrawFile(filename, area_sizex, area_sizey)) return False;

  /* Abspeichern Votingstrukturdatei */
  if (!saveGraphicFile(filename, quor_numb)) return False;

  return True; 
}


Bool saveGraphicFile(filename ,quor_numb)
     char filename[255]; /* Dateiname (ohne Endung) */
     int  quor_numb;     /* Quorenanzahl */
/*
 * AU: Abspeichern der einzelnen Komponenten einer Votingstrukturdatei
 *     Rueckgabewert True
 * NB: Fehler beim Oeffnen oder Schliessen der Datei erzeugt 
 *     Rueckgabe False
 * ST: Aufruf openFileIO
 *     Aufruf saveGraphicNet
 *     Aufruf saveGraphicOperation
 *     Aufruf saveGraphicNode
 *     Aufruf saveGraphicRepresentative
 *     Aufruf saveGraphicVertival
 *     Aufruf saveGraphicHorizontal
 *     Aufruf closeFile
 */         
{
  FILE *graphicFile = NULL; /* Dateideskriptor */
  int  file_length;         /* Dateilaenge (nicht verwendet) */


  /* Datei oeffnen */
  if ((graphicFile = 
         openFileIO(filename, MODE_WRITE_ONLY, vot_file, &file_length)) 
                    == NULL) return False;

  /* Abspeichern der Netzwerktopologie */
  saveGraphicNet(graphicFile, ptrNetObjectListFront);
          
  /* Abspeichern der Operationen */
  saveGraphicOperation(graphicFile, quor_numb);        

  /* Abspeichern der Votingstrukturrechner*/
  saveGraphicNode(graphicFile, ptrGraphicObjectList, quor_numb);
         
  /* Abspeichern der virtuellen Votingstrukturrechner */
  saveGraphicRepresantive(graphicFile, ptrGraphicObjectList);
                     
  /* Abspeichern der Kooperationsrelationen */
  saveGraphicVertical(graphicFile, ptrGraphicObjectList);

  /* Abspeichern der Belegungen */
  saveGraphicHorizontal(graphicFile, ptrGraphicObjectList, quor_numb);

  /* Datei schliessen */
  if (closeFile(graphicFile) != 0) return False;

  return True;
}


void saveGraphicNet(graphicFile, ptrObjectList)
     FILE *graphicFile;   /* Dateideskriptor */
     NODE *ptrObjectList; /* Zeiger auf den Beginn der Votingstruktur-
                           * rechnerliste */
/*
 * AU: Abspeichern der Netztopologierechner in der Reihenfolge
 *     - eindeutigen Bezeichnungen
 *     - frei vergebener Rechnername
 *     - Knotenausfallwahrscheinlichkeit
 */
{
  NODE *zwischennode; /* Laufvariable */

  fprintf(graphicFile, "define netname\n");

  /* Durchlauf der Votingstrukturrechnerliste */
  for (zwischennode= ptrObjectList; zwischennode != NULL;
       zwischennode = zwischennode->nextnode) 
      fprintf(graphicFile, "NODE DESCRIPTION %s :: %s %f\n",
                           zwischennode->ident,
                           zwischennode->name,
                           zwischennode->reliability);
      
  fprintf(graphicFile, "\n");
}

void saveGraphicOperation(graphicFile, quor_numb)
     FILE *graphicFile; /* Dateideskriptor */
     int  quor_numb;    /* Quorenanzahl */
/*
 * AU: Abspeichern der Operationsbeschreibungen
 */
{
  char          operation[15]; /* Operationsbeschreibung */
  char          operNumb[3];   /* Operationsnummer */
  int           i;             /* Laufvariable */

  fprintf(graphicFile, "define operation");
  
  if (quor_numb > 0) { 
     fprintf(graphicFile, "\n");
     for (i = 0; i < quor_numb; i++) {
         strcpy(operation, "operation");
         sprintf(operNumb, "%d", i+1);  
         strncat(operation, operNumb, 2);
         strcpy(data.operation[i], operation);
         fprintf(graphicFile, "OP DESCRIPTION %s\n", 
                              operation);
         }
     }
  else fprintf(graphicFile, " empty\n");

  fprintf(graphicFile, "\n");
}


void saveGraphicNode(graphicFile, ptrObjectList, quor_numb)
     FILE         *graphicFile;   /* Dateideskriptor */
     GRAPHIC_NODE *ptrObjectList; /* Zeiger auf den Beginn der Votingstruktur-
                                   * rechnerliste */
     int          quor_numb;      /* Quorenanzahl */
/*
 * AU: Abspeichern der Votingstrukturrechner bestehend aus den Informationen
 *     - eindeutige Bezeichnung
 *     - Kennung "real" oder "virtual"
 *     _ Quorenbelegung
 */
{
  GRAPHIC_NODE *objectnd; /* Laufvariable */
  int          i;         /* Laufvariable */
  
  fprintf(graphicFile, "define node");
                                        
  if (ptrObjectList != NULL) {
     fprintf(graphicFile, "\n");
     /* Durchlauf der Votingstrukturrechnerliste */
     for (objectnd = ptrObjectList; objectnd != NULL;
                                    objectnd = objectnd->next) {
         fprintf(graphicFile, "OBJECT DESCRIPTION %s :: ", 
                              objectnd->ident);
         
         /* Unterscheidung virtuellen und sonstigen Votingstrukturrechnern */
         if (objectnd->version == 0) fprintf(graphicFile, "real   ");
         else                        fprintf(graphicFile, "virtual");
  
         fprintf(graphicFile, " %d ", objectnd->vote);
              
         /* Quorenbelegung */
         for (i = 0; i < quor_numb; i++)
             fprintf(graphicFile, "%d ", 
                                  objectnd->quor[i]);
         fprintf(graphicFile, "\n");
         }
     }
  else fprintf(graphicFile, " empty\n");
  
  fprintf(graphicFile, "\n");
} 


void saveGraphicRepresantive(graphicFile, ptrObjectList)
     FILE         *graphicFile;   /* Dateideskriptor */
     GRAPHIC_NODE *ptrObjectList; /* Zeiger auf den Beginn der Votingstruktur-
                                   * rechnerliste */
/*
 * AU: Abspeichern der Repraesentanten der virtuellen Votingstrukturrechner
 *     mit der Reihenfolge der Informationen
 *     - eindeutige Bezeichnung des virtuellen Votingstrukturrechners
 *     - eindeutige Bezeichnungen seiner Repraesentanten
 */
{
  Bool          first_print = True; /* Beginn oder laufende Ausgabe */ 
  GRAPHIC_NODE  *objectnd;          /* Laufvariable */
  GRAPHIC_IDENT *zwischenident;     /* Laufvariable */

  fprintf(graphicFile, "define representative");
                                        
  /* Durchlauf der Votingstrukturrechnerliste */
  for (objectnd = ptrObjectList; objectnd != NULL;
                                 objectnd = objectnd->next) {
      if (objectnd->version != 0) {
         if (first_print) {
            fprintf(graphicFile, "\n");
            first_print = False;
            }
         fprintf(graphicFile, "VIRTUAL DESCRIPTION %s is_represented_by ",
                              objectnd->ident); 
         /* Durchlauf der Liste der Repraesentanten */
         for (zwischenident = objectnd->firstIdent; zwischenident != NULL;
              zwischenident = zwischenident->nextListIdent) {
             fprintf(graphicFile, "%s ", 
                                  zwischenident->identItem);
             }
         fprintf(graphicFile, ";\n"); 
         }
      }  
  if (first_print) fprintf(graphicFile, " empty\n");
     
  fprintf(graphicFile, "\n");
}



void saveGraphicVertical(graphicFile, ptrObjectList)
     FILE         *graphicFile;   /* Dateideskriptor */
     GRAPHIC_NODE *ptrObjectList; /* Zeiger auf den Beginn der Votingstruktur-
                                   * rechnerliste */
/*
 * AU: Abspeichern der Kooperationsrelationen zwischen den Elementen einer
 *     Votingstruktur
 */
{
  Bool         first_print = True; /* Beginn oder laufende Ausgabe */
  GRAPHIC_NODE *objectnd;          /* Laufvariable */
  GRAPHIC_ARC  *objectarc;         /* Laufvariable */

  fprintf(graphicFile, "define vertical");
  
  /* Durchlauf der Votingstrukturrechnerliste */
  for (objectnd = ptrObjectList; objectnd != NULL;
                                 objectnd = objectnd->next) {
      if (objectnd->first_arc != NULL) {
         if (first_print) {
            fprintf(graphicFile, "\n");
            first_print = False;
            }
         fprintf(graphicFile, "TREE DESCRIPTION %s depends_on ",
                              objectnd->ident);

         /* Durchlauf der Liste der mit "objectnd" in einer Kooperations-
          * relation stehenden Votingstrukturrechner */
         for (objectarc = objectnd->first_arc; objectarc != NULL;
              objectarc = objectarc->next_arc) {
             fprintf(graphicFile, "%s ", 
                                  objectarc->arc_to_object->ident);
             }
         fprintf(graphicFile, ";\n");
         }
      }
  if (first_print) fprintf(graphicFile, " empty\n");
         
  fprintf(graphicFile, "\n");
}               

void saveGraphicHorizontal(graphicFile, ptrObjectList, quor_numb)
     FILE         *graphicFile;   /* Dateideskriptor */
     GRAPHIC_NODE *ptrObjectList; /* Zeiger auf den Beginn der Votingstruktur-
                                   * rechnerliste */
     int          quor_numb;      /* Quorenanzahl */
{
  GRAPHIC_NODE  *objectnd;            /* Laufvariable */
  GRAPHIC_ARC   *objectarc;           /* Laufvariable */
  int           i;                    /* Laufvariable */
  Bool          first_print;          /* Beginn oder laufende Ausgabe */
  Bool          quorum_found = False;
  Bool          print_empty = True;

  fprintf(graphicFile, "define horizontal");

  /* Durchlauf aller Quoren */
  for (i = 0; i < quor_numb; i++) {
      objectnd = ptrObjectList; /* Beginn am Listenanfang */
   
      /* Liste der Votingstrukturrechner durchsuchen */
      while ((objectnd != NULL) && (!quorum_found)) { 
            objectarc = objectnd->first_arc;
         
            /* Liste der mit "objectnd" in Kooperationsrelation stehenden
             * Rechner nach einer Belegung <> 0 durchsuchen */
            while ((objectarc != NULL) && (!quorum_found)) {
                  if (objectarc->arc_value[i] != 0) quorum_found = True;
  
                  if (!quorum_found) objectarc = objectarc->next_arc;
                  }
            if (!quorum_found) objectnd = objectnd->next;
            }
                
      /* Belegung gefunden */
      if (quorum_found) {
         fprintf(graphicFile, "\n");
         fprintf(graphicFile, "%s", 
                              data.operation[i]);

         if (print_empty) print_empty = False;

         first_print = True; 
         while (objectnd != NULL) {
               objectarc = objectnd->first_arc;
               while (objectarc != NULL) {
                     if (objectarc->arc_value[i] != 0) {
                        if (first_print) {
                           fprintf(graphicFile, " (");
                           first_print = False;
                           }
                        else fprintf(graphicFile, "\n\t    ");

                        fprintf(graphicFile, "(%s priors %s with %d)",
                                             objectnd->ident,
                                             objectarc->arc_to_object->ident,
                                             objectarc->arc_value[i]);
                        }
                     objectarc = objectarc->next_arc;
                     }
               objectnd = objectnd->next;
               }
         fprintf(graphicFile, ")");
         }
      quorum_found = False;
      }
  if (print_empty) fprintf(graphicFile, " empty\n");

  fprintf(graphicFile, "\n");
  fprintf(graphicFile, "end");
}     



Bool saveDrawFile(filename, area_sizex, area_sizey)
     char filename[255];          /* Dateiname (ohne Endung) */
     int  area_sizex, area_sizey; /* Groesse der Zeichenflaeche */
/*
 * AU: Abspeichern der einzelnen Komponenten einer Grafikdatendatei
 *     Rueckgabewert True
 * NB: Fehler beim Vffnen oder Schliessen der Datei erzeugt 
 *     Rueckgabe False
 * ST: Aufruf openFileIO
 *     Aufruf saveDrawDimension
 *     Aufruf saveDrawNode
 *     Aufruf closeFile
 */
{
  FILE *drawFile = NULL; /* Dateideskriptor */
  int  file_length;      /* Dateilaenge (nicht verwendet) */

  /* Datei oeffnen */
  if ((drawFile = 
         openFileIO(filename, MODE_WRITE_ONLY, drw_file, &file_length))
                    == NULL) return False;

  /* Abspeichern der Zeichenflaechedimension */
  saveDrawDimension(drawFile, area_sizex, area_sizey);

  /* Abspeichern Knotenkoordinaten */
  saveDrawNode(drawFile);
  
  /* Datei schliessen */
  if (closeFile(drawFile) != 0) return False;
       
  return True;
}


void saveDrawDimension(drawFile, sizex, sizey)
     FILE *drawFile; /* Dateideskriptor */
     int   sizex, sizey;
/*
 * AU: Abspeichern der Koordinaten der Zeichenflaeche
 */
{
  fprintf(drawFile, "DRAWING_AREA_SIZE %d %d\n", sizex, sizey);
}


void saveDrawNode(drawFile)
     FILE *drawFile;      /* Dateideskriptor */
/*
 * AU: Abspeichern der Koordinaten jedes einzelnen Votingstrukturrechners
 */
{
  GRAPHIC_NODE *objectnd; /* Laufvariable */

  /* Durchlauf der Votingstrukturrechnerliste */
  for (objectnd = ptrGraphicObjectList; objectnd != NULL;
                                        objectnd = objectnd->next) {
      fprintf(drawFile, "KNOTEN %s %d %d\n",
                        objectnd->ident,
                        objectnd->pos_x,
                        objectnd->pos_y);
      }
}  



Bool file_exists(filename)
     char *filename; /* Dateiname (ohne Endung) */
/*
 * AU: Pruefung, ob eine gleichnamige Votingstrukturrechnerdatei
 *     existiert
 *     Rueckgabe True bei Existenz einer solchen Datei
 * NB: sonstiger Rueckgabewert False
 */
{
 char checkname[260]; /* Dateiname */
 struct stat statbuf; /* Struktur mit Dateiinformationen aus sys/stat.h */

 strncpy(checkname, filename, 255);
 strcat(checkname, ".vot");
 if(stat(checkname, &statbuf) == 0) return True;

 return False;  
}



/***************************************************************************/
/*                    Ende der Funktionen                                  */
/***************************************************************************/


































