/*
Name:   graph.c
Zweck:  Funktionen zur Manipulation der internen Datenstrukturen der
        Netztopologie und der Voting-Struktur (VS)
Autor:  Andreas Seifert
Datum:
Update:
*/

/*
 * AU: Datenstruktur aufbauen, entfernen untersuchen 
 *     und mit Werten belegen
 * ST: createGraph()                Netztopologie initialisiern
 *     correspondingNetNode()       sucht nach Rechner im Netz
 *     correspondingGrahicNode()    sucht nach Rechner in Voting-Struktur
 *     addNetNode()                 Erzeugen eines Netzrechners
 *     insertNetEdge()              Einfuegen einer Verbindung im Netz
 *     clear_Graph()                Aufloesen der Netztopologie
 *     clear_GraphicObjects()       Aufloesen der Voting-Struktur
 *     remove_arcs()                Entferne alle Verbindungen zu einem
 *                                  Knoten der Voting-Struktur
 *     removeIdent()                Entferne Liste der Repraesentanten
 *     deleteArc()                  Entferne eine Verbindung in der VS
 *     insertArc()                  Einfuegen einer Verbindung in der VS
 *     arc_already_exists()         Verbindung in der VS existiert?
 *     setGraphicArcWeight()        Belegung Zugriffsrelation
 *     change_GraphicObject()       Belegung der Koordinaten eines 
 *                                  Rechners der Voting-Struktur
 *     node_hit()                   Pruefung ob Koordinaten in einem
 *                                  Rechnersymbol liegen
 *     add_GraphicObject()          Erzeugen eines Rechners der VS
 *     addRepresentatives()         Repraesentantenliste anfuegen
 *     add_GraphicVoteQuor()        Belegung Stimme und Quoren in VS
 *     computeConnections()         Ausgangs- und Endkoordinaten einer
 *                                  Verbindung am Rechnersymbol ermitteln
 *     validPosition()              Pruefung ob Position im Arbeitsbereich 
 *                                  bereits belegt
 *     mark_version()               Versionsnummer fuer virtuellen Rechner
 *                                  bestimmen
 *     listcmp()                    Vergleich zweier Repraesentantenlisten
 *     clean_SuccessfulSons()       Liste efolgreicher Nachfolgerknoten 
 *                                  loeschen
 *     clean_VQM()                  Quorenlist loeschen
 *     deleteVQMSet()               Virt. Quorenmenge loeschen
 *     clean_ListOfReferences()     Referenzliste loeschen
 *     *findRoot()                  Wurzelknoten ermitteln
 *     *searchRoot()                Topologische Sortierung
 *     *checkVQM()                  Pruefe Voting-Str. wg. Zyklenfreiheit
 *     sortArc()                    Kanten nach Stimmen sortieren
 *     exchangeArcFromTo()          Positionentausch zweier Kanten  
 *     addSolutionToVQMSet()        Virtuelle Gruppe hinzufuegen
 *     existVQMSet()                Pruefe Existenz einer virt. Gruppe
 *     listcmpVQMSet()              Vergleich zweiere virt. Gruppen
 *     insertElementIntoVQMSet()    Rechner der virt. Gruppe hinzufuegen
 *     insertItemIntoVQMSet()       Element sortiert hinzufuegen
 *     notMemberOfSet()             Elementeigenschaft pruefen
 *     *createVQMSet()              Kopf einer virt. Gruppe erzeugen
 *     *createVQMSetElement()       Element einer virt. Gruppen erzeugen
 *     *createHeadOfNodeSet()       Kopf f. d. virt. Gruppen einer OP erz.
 *     addNodeSetElement()          Nachfolger dem Vaterknoten hinzufuegen
 *     removeObjectSetFromRefList() Entferne virtuelle Gruppen
 *     removeObjectFromRefList()    Entferne Referenzlistenelement
 *     addObjectSetToRefList()      Menge der Referenzliste hinzufuegen
 *     *addObjectToRefList()        Element der Referenzliste hinzufuegen
 *     *getRefListElement()         Element der Referenzliste ermitteln
 */


#include "graph.h"
#include "motif.h" /* fuer wprint() */
#include "draw.h"



/**************************************************************************/
/*                     Beginn der Funktionen                              */
/**************************************************************************/


NODE *create_Graph()
/*
 * AU: Initialisiere die Netztopologie
 * NB: Rueckgabewert NULL
 */
{
 NODE *init_graph;

 init_graph = NULL;

 return init_graph;
}



NODE *correspondingNetNode(identifier)
     char identifier[42]; /* eindeutiger Rechnername */
/*
 * AU: Suche nach einem Rechner der Netztopologie
 * NB: Erfolglose Suche ergibt Rueckgabewert NULL, sonst
 *     Zeiger auf Rechner der Netztopologie
 */
{
 NODE *ptr_search;

 ptr_search = ptrNetObjectListFront; /* Zeiger auf den Beginn der Liste */
  
 while (ptr_search != NULL) {
       if ((strncmp(identifier, ptr_search->ident, 40)) == 0) 
          return ptr_search; /* Rechner gefunden */
       else ptr_search = ptr_search->nextnode; /* weitersuchen */
       }
 return NULL; /* Fehlerfall */
}



GRAPHIC_NODE *correspondingGraphicNode(identifier)
             char identifier[42]; /* eindeutiger Rechnername */
/*
 * AU: Suche nach einem Rechner der Voting-Struktur
 * NB: Erfolglose Suche ergibt Rueckgabewert NULL, sonst
 *     Zeiger auf REchner der Voting-Struktur
 */
{
 GRAPHIC_NODE *ptr_search; 

 ptr_search = ptrGraphicObjectList; /* Zeiger auf den Beginn der Liste */
  
 while (ptr_search != NULL) {
        if ((strncmp(identifier, ptr_search->ident, 40)) == 0) 
           return ptr_search; /* Rechner gefunden */
        else ptr_search = ptr_search->next; /* weitersuchen */
       }
 return NULL; /* Fehlerfall */
} 



NODE *addNetNode(ident, name, reliability)
     char   ident[40];   /* eindeutiger Rechnername */
     char   name[40];    /* selbst vergebener Name */ 
     double reliability; /* Aufallwahrscheinlichkeit */
/*
 * AU: Erzeugen eines Rechners der Netztopologie und
 *     Einfuegen in die Liste der Rechner
 * NB: Rueckgabewert Zeiger auf den neueingefuegten Rechner
 * ST: demand() Meldung und anschl. Programmabbruch bei 
 *              erfolgloser Speicherplatzanforderung
 */
{
 NODE  *temp_node;
 
 temp_node = (NODE *)XtMalloc(sizeof(NODE));

 demand(temp_node, memory allocation in addNetNode failed);

 strncpy(temp_node->ident, ident, 40);

 strncpy(temp_node->name, name, 40);

 temp_node->reliability = reliability;

 temp_node->arcptr = NULL;

 temp_node->nextnode = NULL;

 if (ptrNetObjectListFront == NULL)
    ptrNetObjectListFront = ptrNetObjectListTail = temp_node;
 else {
      ptrNetObjectListTail->nextnode = temp_node;
      ptrNetObjectListTail = temp_node;
      }
 return temp_node;
} 



void insertNetEdge(sourceNode, currNode, reliability)
     NODE   *sourceNode;  /* Ausgangsrechner der Verbindung */
     NODE   *currNode;    /* Endrechner der Verbindung */
     double reliability;  /* Ausfallwahrscheinlichkeit */
/*
 * AU: Einfuegen einer Verbindung zwischen Rechnern der Netztopologie
 * ST: demand() Meldung und anschl. Programmabbruch bei 
 *              erfolgloser Speicherplatzanforderung
 */
{
  ARC *temp;

  temp = sourceNode->arcptr;
  
  sourceNode->arcptr = (ARC *)XtMalloc(sizeof(ARC));

  demand(sourceNode->arcptr, memory allocation in insertNetEdge failed);

  sourceNode->arcptr->ndptr = currNode;

  sourceNode->arcptr->fatherndptr = sourceNode;

  sourceNode->arcptr->reliability = reliability;

  sourceNode->arcptr->arcType = other_object; 

  sourceNode->arcptr->nextarc = temp;
}




void clear_Graph()
/*
 * AU: Aufloesen der Netztopologie
 */
{
 NODE  *zwischennode = ptrNetObjectListFront; /* Beginn der Rechner */
 NODE  *delnode;     /* Zeiger auf zu loeschenden Rechner */
 ARC   *zwischenarc; /* Zeiger auf eine Verbindung */
 ARC   *delarc;      /* Zeiger auf zu loeschende Verbindung */

 while (zwischennode != NULL) {
        zwischenarc = zwischennode->arcptr;

        /* Aufloesung Verbindung */
        while (zwischenarc != NULL) {
               delarc = zwischenarc;
               zwischenarc = zwischenarc->nextarc;
               XtFree((char *)delarc);
               }  

        /* Aufloesung Rechner */
        delnode = zwischennode;
        zwischennode = zwischennode->nextnode;
        XtFree((char *)delnode);
        }
 ptrNetObjectListFront = zwischennode;
}



void  clear_GraphicObjects()
/*
 * AU: Aufloesen der Voting-Struktur
 */
{
 GRAPHIC_NODE        *zwischennode; /* Beginn der Rechner der Struktur */ 
 GRAPHIC_NODE        *delnode;     /* Zeiger auf zu loeschenden Rechner */
 GRAPHIC_ARC         *zwischenarc;  /* Zeiger auf eine Verbindung */
 GRAPHIC_ARC         *delarc;        /* Zeiger auf zu loeschende Verbindung */ 
 GRAPHIC_IDENT       *zwischenident; /* Zeiger auf einen Repraesentanten */
 GRAPHIC_IDENT       *delident;      /* Zeiger zu loeschender Repraesentant */

 zwischennode = ptrGraphicObjectList; /* Zeiger auf den Beginn der Liste */

 while (zwischennode != NULL) {

        /* Aufloesen Verbindungen */
        zwischenarc = zwischennode->first_arc;
        while (zwischenarc != NULL) {
               delarc = zwischenarc;
               zwischenarc = zwischenarc->next_arc;
               XtFree((char *)delarc);
               }

        /* Augloesung Repraesentantenliste */
        zwischenident = zwischennode->firstIdent;
        while (zwischenident != NULL) {
               delident = zwischenident;
               zwischenident = zwischenident->nextListIdent;
               XtFree((char *)delident);
               }

        /* Aufloesung Rechner */
        delnode = zwischennode;
        zwischennode = zwischennode->next;
        XtFree((char *)delnode);
       }
 ptrGraphicObjectList = NULL;
} 


void clean_SuccessfulSons()
/*
 * AU: Knoten der Voting-Struktur initialisieren und die Reste
 *     vorheriger Berechnungen entfernen
 */   
{
 GRAPHIC_NODE        *zwischennode;       /* Rechner der Voting-Struktur */
 GRAPHIC_SET         *zwischenset;        /* Zeiger auf virtuelle Gruppe */
 GRAPHIC_SET         *delset;             /* Zeiger auf virtuelle Gruppe */
 GRAPHIC_SET_ELEMENT *zwischensetelement; /* Zeiger auf Gruppenelement */
 GRAPHIC_SET_ELEMENT *delsetelement;      /* Zeiger auf Gruppenelement */
 int                 i;
        
 zwischennode = ptrGraphicObjectList; /* Zeiger auf den Beginn der Liste */
	      
 while (zwischennode != NULL) {
       for (i = 0; i < 5; i++) { /* fuer jede Operation */
           zwischenset = zwischennode->firstNodeSetHead[i];
				  
	   while (zwischenset != NULL) {
	         zwischensetelement = zwischenset->firstNodeSetElement;
							       
		 while (zwischensetelement != NULL) {
		       delsetelement = zwischensetelement;
		       zwischensetelement =
		           zwischensetelement->nextNodeSetElement;
		       XtFree((char *)delsetelement);
                       }
  
                 delset = zwischenset;
		 zwischenset = zwischenset->nextNodeSetHead;
		 XtFree((char *)delset);
		 }
	   zwischennode->firstNodeSetHead[i] = NULL;
	   }
       zwischennode = zwischennode->next;
       }
}



void clean_VQM()
/*
 * AU: Virtuelle Quorenmengen vorheriger Berechnungen
 *     beseitigen
 * SU: deleteVQMSet()
 */
{
 VQM_SET *zwischenset;
 VQM_SET *delset;
 int     i;
          
 for (i = 0; i < 5; i++) { /* fuer jede Operation */
     zwischenset = ptrVQMSet[i];
		        
     while (zwischenset != NULL) {
           delset = zwischenset;
	   zwischenset = zwischenset->nextVQMSetHead;
	   deleteVQMSet(delset);
	   }

     ptrVQMSet[i] = NULL;
     }
}



void deleteVQMSet(ptrVQMSetHead)
     VQM_SET *ptrVQMSetHead; /* Zeiger auf zu loeschende virtuelle Gruppe */
/*
 * AU: Loeschen der spezifizierten Menge
 */
{
 VQM_SET_ELEMENT *zwischensetelement;
 VQM_SET_ELEMENT *delsetelement;
       
 zwischensetelement = ptrVQMSetHead->firstVQMSetElement;
	
 while (zwischensetelement != NULL) {
       delsetelement = zwischensetelement;
       zwischensetelement = zwischensetelement->nextVQMSetElement;
       XtFree((char *)delsetelement);
       }
 XtFree((char *)ptrVQMSetHead);
}



void clean_ListOfReferences()   
/*
 * AU: Loeschen der Referenzliste
 */
{
 REF_SET_ELEMENT *ptrCurrRef;
 REF_SET_ELEMENT *ptrDelRef;
        
 ptrCurrRef = ptrListOfReference; /* Zeiger auf den Beginn der Referenzliste */
	        
 while (ptrCurrRef != NULL) {
       ptrDelRef = ptrCurrRef;  
       ptrCurrRef= ptrCurrRef->nextRefSetElement;
       XtFree((char *)ptrDelRef);
       }
					      
 ptrListOfReference = ptrCurrRefList = NULL;
}




void remove_arcs(ptrDelNode)
     GRAPHIC_NODE *ptrDelNode; /* Rechner der Voting-Struktur */
/*
 * AU: Entferne alle Verbindungen von und zu einem 
 *     Rechner der Voting-Struktur
 */
{
 GRAPHIC_NODE *currnode;
 GRAPHIC_ARC  *currarc, *beforearc;
     
 for (currnode = ptrGraphicObjectList; currnode != NULL;
                              currnode = currnode->next) {
     currarc = currnode->first_arc;
     if (currarc != NULL) {
        if (strncmp(currarc->arc_to_object->ident,
            ptrDelNode->ident, 40) == 0) {

             /* ptrDelNode ist die erste Verbindung zu
              * einem anderen Rechner */
             currnode->first_arc = currarc->next_arc;
             XtFree((char *)currarc);
           }
        else { /* Suche nach einer Verbindung im Rest 
                * der Liste */
             while (currarc->next_arc != NULL) {
                   beforearc = currarc;
                   currarc = currarc->next_arc;
                   if (strncmp(currarc->arc_to_object->ident,
                       ptrDelNode->ident, 40) == 0) {
                      beforearc->next_arc = currarc->next_arc;
                      XtFree((char *)currarc);
                      break;
                      }
                   }
             }
        }
     }

  /* Entferne alle Verbindungen mit Ausgangspunkt ptrDelNode */
  currarc = ptrDelNode->first_arc;
  while (currarc != NULL) {
     beforearc = currarc;
     currarc = currarc->next_arc;
     XtFree((char *)beforearc);
     }
}          




void remove_ident(ptrDelNode)
     GRAPHIC_NODE *ptrDelNode; /* Rechner der Voting-Struktur */
/*
 * AU: Entferne alle Repraesentanten eines Rechners
 *     der Voting-Struktur
 */
{
 GRAPHIC_IDENT *zwischenident, *delident;
                       
 zwischenident = ptrDelNode->firstIdent;
 while (zwischenident != NULL) {
       delident = zwischenident;
       zwischenident = zwischenident->nextListIdent;
       XtFree((char *)delident);
       }
}



Bool deleteArc(first_marked_node, next_marked_node)
     GRAPHIC_NODE *first_marked_node;
     GRAPHIC_NODE *next_marked_node;
/*
 * AU: Entferne eine Verbindung zwischen Rechnern
 *     der Voting-Struktur
 * NB: vergebliche Suche nach der Verbindung ergibt Rueckgabewert
 *     False, sonst True
 */
{
 GRAPHIC_ARC  *currarc, *beforearc;
     
 /* entferne die Kante, die zwischen den beiden Knoten besteht
  * es darf nicht angenommen werden, dass der Benutzer die Knoten
  * korrekt in Pfeilrichtung markiert hat
  */

 /* Annahme: der als zweites markierte Rechner ist Ausgangspunkt der
  * Verbindung */       
 currarc = first_marked_node->first_arc;
 if (currarc != NULL) {
    if (strncmp(currarc->arc_to_object->ident,
                next_marked_node->ident, 40) == 0) {
       
       /* zu loeschende Verbindung ist am Beginn der Liste */
       first_marked_node->first_arc = currarc->next_arc;
       XtFree((char *)currarc);
       return True;
       }
    else { /* zu loeschende Verbindung ist im Rest der Liste */
         while (currarc->next_arc != NULL) {
               beforearc = currarc;
               currarc = currarc->next_arc;
               if (strncmp(currarc->arc_to_object->ident,
                           next_marked_node->ident, 40) == 0) {
                  beforearc->next_arc = currarc->next_arc;
                  XtFree((char *)currarc);
                  return True;
                  }
               }
         }   
    }

 /* Annahme: der als zweites markierte Rechner ist Ausgangspunkt der
  * Verbindung */
 currarc = next_marked_node->first_arc;
 if (currarc != NULL) {
    if (strncmp(currarc->arc_to_object->ident,
                first_marked_node->ident, 40) == 0) {

       /* zu loeschende Verbindung ist am Beginn der Liste */
       next_marked_node->first_arc = currarc->next_arc;
       XtFree((char *)currarc);   
       return True;
       }
    else { /* zu loeschende Verbindung ist im Rest der Liste */
         while (currarc->next_arc != NULL) {
               beforearc = currarc;
               currarc = currarc->next_arc;
               if (strncmp(currarc->arc_to_object->ident,
                           first_marked_node->ident, 40) == 0) {
                  beforearc->next_arc = currarc->next_arc;
                  XtFree((char *)currarc);
                  return True;
                  }
               }
         }
    }
 /* Fehlerfall */        
 return False;
}




Bool insertArc(start, end)
     GRAPHIC_NODE *start, *end; /* Start- und Endknoten */
/*
 * AU: Einfuegen einer Verbindung zwischen Rechnern der 
 *     Voting-Struktur
 * NB: Rueckgabe im Fehlerfall False, sonst True
 * ST: demand() Meldung und anschl. Programmabbruch bei 
 *              erfolgloser Speicherplatzanforderung
 */
{
 GRAPHIC_ARC *new_arc;
 int         i;
  
 new_arc = arc_already_exists(start, end);

 if (new_arc == NULL) {
      
    /* Erzeuge eine neue Kante und initialisiere sie */
    new_arc = (GRAPHIC_ARC *)XtMalloc(sizeof(GRAPHIC_ARC));

    demand(new_arc, memory allocation in insertArc failed);
  
    new_arc->next_arc = start->first_arc;
            
    new_arc->arc_to_object = end;
    
    start->first_arc = new_arc;
     
    for (i = 0; i < 5; i++) new_arc->arc_value[i] = 0;

    return True;
    }
    
 return False;
}



GRAPHIC_ARC *arc_already_exists(a_node, other_node)
            GRAPHIC_NODE *a_node, *other_node; /* Rechner der Voting-Struktur */
/*
 * AU: Pruefung, ob eine Verbindung zwischen Rechnern der 
 *     Voting-Struktur besteht
 * NB: Rueckgabe im Fehlerfall NULL, sonst Zeiger auf die Verbindung
 */
{
 GRAPHIC_ARC  *currarc;
 
 /* Annahme: a_node ist Ausgangspunkt der Verbindung */
 currarc = a_node->first_arc; 
 while (currarc != NULL) {
       if ((strncmp(currarc->arc_to_object->ident,
                   other_node->ident, 40) == 0) &&
                   (currarc->arc_to_object->version == other_node->version)) {
          return currarc;
          }
       currarc = currarc->next_arc;
       }
 
 /* Annahme: other_node ist Ausgangspunkt der Verbindung */
 currarc = other_node->first_arc;
 while (currarc != NULL) {
       if ((strncmp(currarc->arc_to_object->ident,
                   a_node->ident, 40) == 0) &&
                   (currarc->arc_to_object->version == a_node->version)) { 
          return currarc;
          }
       currarc = currarc->next_arc;
       }

 /* Fehlerfall */
 return NULL;
}




int setGraphicArcWeight(startNode, targetNode, service, value)
    GRAPHIC_NODE *startNode, *targetNode; /* Rechner der Voting-Struktur */
    int service, value;                   /* Nummer des Dienstes, Belegung */
/*
 * AU: Belegung eine Verbindung mit einer Zugriffsreihenfolge
 */ 
{
 GRAPHIC_ARC  *currarc;
 
 /* Annahme: startNode ist Ausgangspunkt der Verbindung */
 currarc = startNode->first_arc;
 while (currarc != NULL) {
       if (currarc->arc_to_object == targetNode) {
          currarc->arc_value[service] = value;
          break;
          }
       currarc = currarc->next_arc;
       } 

 /* Annahme: targetNode ist Ausgangspunkt der Verbindung */ 
 if (currarc == NULL) {
    currarc = targetNode->first_arc;
    while (currarc != NULL) {
          if (currarc->arc_to_object == startNode) {
             currarc->arc_value[service] = value;
             break;
             }
          currarc = currarc->next_arc;           
          }
    }

 if(currarc == NULL)
   return 0;
 return 1;
 
}




void change_GraphicObject(x_koord, y_koord, nodeadress)
     int x_koord, y_koord;     /* Neue Koordinaten */
     GRAPHIC_NODE *nodeadress; /* Zeiger auf Rechner */
/*
 * AU: Belegung der Koordinaten eines Rechners der Voting-Struktur
 * ST: compute_Connections()
 */
{
  nodeadress->pos_x = x_koord;
  nodeadress->pos_y = y_koord;
  compute_Connections(x_koord, y_koord, nodeadress);
}




GRAPHIC_NODE *node_hit(x_koord, y_koord)
             int x_koord, y_koord; /* Mauszeigerkoordinaten */
/*
 * AU: Pruefung, ob Koordinaten in einem Rechnersymbol der Voting-Struktur
 *     liegen
 * NB: Rueckgabe NULL bei erfolgloser Suche, sonst Zeiger auf Rechner
 */ 
{
 GRAPHIC_NODE *object;
         
 for(object = ptrGraphicObjectList; object != NULL; object = object->next) {

    switch (zoomOption)
    {
           case ZOOM_100 : /* normale Darstellung */
                if (x_koord >= object->pos_x - 8 && 
                    x_koord <= object->pos_x + 90 &&
                    y_koord >= object->pos_y - 17 && 
                    y_koord <= object->pos_y + 50) return object;
                break;

           case ZOOM_75 : /* 75 % Darstellung */
                if (x_koord >= object->pos_x - 6 &&
                    x_koord <= object->pos_x + 75 &&
                    y_koord >= object->pos_y - 11 &&
                    y_koord <= object->pos_y + 44) return object; 
                break;

           case ZOOM_50 : /* 50 % Darstellung */
                if (x_koord >= object->pos_x - 10 &&
                    x_koord <= object->pos_x + 62 &&
                    y_koord >= object->pos_y - 9  &&
                    y_koord <= object->pos_y + 36) return object;
                break;

           default :
                XBell(display, 0);
                wprint(unknown_event, "graph.c : node_hit");
                break;
    }
    }

 /* Fehlerfall */
 return NULL;
}



void add_GraphicObject(name, ident, version)
     char name[40];  /* selbst vergebener Name */
     char ident[40]; /* eindeutiger Rechnername */
     int version;    /* Versionsnummer */
/*
 * AU: Erzeugen eines Rechners der Voting Voting-Struktur und
 *     Einfuegen in die Liste der Rechner
 * ST: demand() Meldung und anschl. Programmabbruch bei 
 *              erfolgloser Speicherplatzanforderung
 */ 
{
   int          i;
   GRAPHIC_NODE *new_ptr;

   new_ptr = (GRAPHIC_NODE *)XtMalloc(sizeof(GRAPHIC_NODE));

   demand(new_ptr, memory allocation in add_GraphicObject failed);
   
   new_ptr->next = ptrGraphicObjectList;

   if (ptrGraphicObjectList != NULL) ptrGraphicObjectList->previous = new_ptr; 
 
   ptrGraphicObjectList = new_ptr;

   strncpy(ptrGraphicObjectList->ident, ident, 40);
   
   strncpy(ptrGraphicObjectList->name, name, 40);
 
   ptrGraphicObjectList->version = version;

   ptrGraphicObjectList->vote = 0;

   ptrGraphicObjectList->graphObjStatus = non_visited;

   ptrGraphicObjectList->previous = NULL;

   for (i = 0; i < 5; i++) {
       ptrGraphicObjectList->quor[i] = 0;
       ptrGraphicObjectList->firstNodeSetHead[i] = NULL; 
       }

   ptrGraphicObjectList->first_arc = NULL;

   ptrGraphicObjectList->firstIdent = NULL; 
}




void addRepresentatives(graphicObject, ptrIdent)
     GRAPHIC_NODE  *graphicObject; /* Rechner der Voting-Struktur */
     GRAPHIC_IDENT *ptrIdent;      /* Repraesentantenliste */
/*
 * AU: Anfuegen einer Repraesentantenliste an einen Rechner der 
 *     Voting-Struktur
 */
{
     graphicObject->firstIdent = ptrIdent;
}


GRAPHIC_IDENT *createIdent(ident)
              char   ident[5];   /* eindeutiger Rechnername */
/*
 * AU: Erzeugen eines Rechners der Netztopologie und   
 * NB: Rueckgabewert Zeiger auf den neueingefuegten Rechner
 * ST: demand() Meldung und anschl. Programmabbruch bei 
 *              erfolgloser Speicherplatzanforderung
 */
{
  GRAPHIC_IDENT  *temp_ident;
		       
  temp_ident = (GRAPHIC_IDENT *)XtMalloc(sizeof(GRAPHIC_IDENT));
			 
  demand(temp_ident, memory allocation in createIdent failed);

  strncpy(temp_ident->identItem, ident, 5);

  temp_ident->nextListIdent = NULL;

  return temp_ident;
}




void add_GraphicVoteQuor(nodeptr, vote, quor1, quor2, quor3, quor4, quor5)
     GRAPHIC_NODE *nodeptr; /* Rechner der Voting-Struktur */
     int vote, quor1, quor2, quor3, quor4, quor5; /* Stimme und Quoren */
/*
 * AU: Belegung der Stimme und der Quoren eines Rechners der 
 *     Voting-Struktur
 */
{
 nodeptr->vote = vote;

 nodeptr->quor[0] = quor1;
 nodeptr->quor[1] = quor2;
 nodeptr->quor[2] = quor3;
 nodeptr->quor[3] = quor4;
 nodeptr->quor[4] = quor5;
}



void compute_Connections(x, y, node)
     int x, y;           /* Rechnerkoordinaten */
     GRAPHIC_NODE *node; /* Rechner der Voting-Struktur */
/*
 * AU: Ausgangs- und Endkoordinaten einer Verbindung am Rechnersymbol
 *     ermitteln 
 */ 
{
 node->top_x = node->bottom_x = x + 40;
 node->top_y = y - 17;   
 node->bottom_y = y + 53;
}




Bool validPosition(koord_x, koord_y, ptrOwnObject)
     int koord_x, koord_y;       /* Rechnerkoordinaten */
     GRAPHIC_NODE *ptrOwnObject; /* zu positionierender Rechner */
/*
 * AU: Pruefung, ob die Koordinaten eines Rechners der Voting-Struktur
 *     nicht bereits belegt sind
 * NB: Rueckgabe im Fehlerfall False, sonst True
 * ST: Intersect()
 */
{
 GRAPHIC_NODE *object;
  
 for(object = ptrGraphicObjectList; object != NULL; object = object->next) {
    if (Intersect(koord_x, koord_y, object->pos_x, object->pos_y) &&
        (object != ptrOwnObject)) 
        return False;
    }

 /* Fehlerfall */
 return True;
}



GRAPHIC_NODE *findRoot(graphic_objects)
             GRAPHIC_NODE *graphic_objects;
/*
 * AU: Wurzel der Voting-Struktur ermitteln
 */
{
 GRAPHIC_NODE *object;
 GRAPHIC_NODE *estimatedRoot;
 GRAPHIC_ARC  *arc;
 Bool         root_found = False;
		 
 estimatedRoot = NULL;

 /*
  * Durchlaufe die Liste der Knoten und markiere alle Knoten als nicht besucht
  */
 for (object = graphic_objects; object != NULL; object = object->next)
     object->graphObjTyp = other_object; 

 /* Markiere alle Knoten die Ziel einer Kante sind */
 for (object = graphic_objects; object != NULL; object = object->next) {
	arc = object->first_arc;
	while (arc != NULL) {
	      arc->arc_to_object->graphObjTyp = visited_object;
              arc = arc->next_arc;
	      }
     }

 /* der erste Knoten der nicht Ziel von Kanten war ist vermutlich
  * eine Wurzel, doch danach Suche nach weiteren "Wurzeln" */
 for (object = graphic_objects; object != NULL; object = object->next) {
     if (object->graphObjTyp == other_object) {
	if (root_found) return NULL; /* falls bereits eine Wurzel gefunden
                                      * wurde ergibt das Auftauchen eines
                                      * weiteren Knotens ohne Vorgaenger
                                      * einen Fehler */
        else { /* erstmaliges Auftauchen einer vermutlichen Wurzel */
	     root_found = True;
	     estimatedRoot = object;
             }
        }
     }

 if (!root_found) return NULL;

 return estimatedRoot; /* Rueckgabe des Wurzelknotens */
}




GRAPHIC_NODE *searchRoot(graphic_objects)
             GRAPHIC_NODE *graphic_objects;
/*
 * AU: Finde einen Knoten in der Voting-Struktur der noch nicht
 *     als Wurzel entfernt wurde und markiere ihn als entfernt,
 *     Anwendung des topsort
 */
{
 GRAPHIC_NODE *object;
 GRAPHIC_ARC  *arc;
	        
 /*
  * Durchlaufe die Liste der Knoten und markiere alle Knoten als nicht besucht
  */
 for (object = graphic_objects; object != NULL; object = object->next) {
     if (object->graphObjTyp != topsorted_object) /* falls noch nicht entfernt */ 
        object->graphObjTyp = other_object;
     }       
				 
 for (object = graphic_objects; object != NULL; object = object->next) {
	if (object->graphObjTyp != topsorted_object) {
	   arc = object->first_arc;
	   while (arc != NULL) {
		 arc->arc_to_object->graphObjTyp = visited_object; /* Nachfolger */
		 arc = arc->next_arc;
                 }
           }
     }
		
 for (object = graphic_objects; object != NULL; object = object->next) {
     if (object->graphObjTyp == other_object) return object; /* Wurzelrechner */
     }

 return NULL;
}




GRAPHIC_NODE *checkVQM(graphic_objects)
             GRAPHIC_NODE *graphic_objects;
/*
 * AU: Pruefe die Voting-Struktur auf Zyklenfreiheit  
 * SU: findRoot();
 *     searchRoot();
 */
{
 GRAPHIC_NODE *object;
 GRAPHIC_NODE *root;
	                  
 root = (GRAPHIC_NODE *)findRoot(graphic_objects);

 if (root == NULL) {
    wprint(multipleRoot);
    return NULL;
    }					             
						      
 object = root;
						              
 while (object != NULL) {
       /* Wurzel wird mittels Markierung aus dem Graphen entfernt */
       object->graphObjTyp = topsorted_object;
       /* finde die naechste Wurzel */
       object = (GRAPHIC_NODE *)searchRoot(graphic_objects);
       }

 /* konnte immer ein Knoten als eindeutige Wurzel
  * gefunden werden oder sind mehrere Knoten uebriggeblieben
  */
 for (object = graphic_objects; object != NULL; object = object->next) {
     if (object->graphObjTyp != topsorted_object) {
        XBell(display, 0);
	wprint(cyclicVQM, object->ident);
	return NULL;
        }
     }
 return root; /* Rueckgabe Wurzelknoten */
}



  
int mark_version(identFront)
    GRAPHIC_IDENT *identFront; /* Repraesentantenliste */
/*
 * AU: Bestimmung der Versionsnummer fuer einen virtuellen Rechner
 *     der Voting-Struktur
 * NB: Rueckgabewert ist die Versionsnummer
 * ST: listcmp()
 */ 
{
  static int version_nr;

  GRAPHIC_NODE *zwischenObject = ptrGraphicObjectList; /* Beginn der Liste */

  version_nr = 0;
  
  while (zwischenObject != NULL) {
        if (listcmp(zwischenObject->firstIdent, identFront) == 0) 
           version_nr = Maximum(version_nr, zwischenObject->version);
         
        zwischenObject = zwischenObject->next;
        }
  return version_nr + 1; /* Rueckgabe der neuen Versionsnummer */
}


int listcmp(firstIdent, frontIdent)
    GRAPHIC_IDENT *firstIdent, *frontIdent; /* zu vergleichende Listen
                                             * der Repraesentanten */
/*
 * AU: Vergleich der Repraesentantenlisten zweier virtueller Rechner
 *     der Voting-Struktur
 * NB: Rueckgabewert bei Uebereinstimmung 0, sonst -1
 */
{
  GRAPHIC_IDENT *list1, *list2;

  list1 = frontIdent;
  list2 = firstIdent;

  while ((list1 != NULL) && (list2 != NULL)) {
        if (strncmp(list1->identItem, list2->identItem, 5) != 0) return -1;
        else {
             list1 = list1->nextListIdent;
             list2 = list2->nextListIdent;
             }            
        }
  
  if ((list1 == NULL) && (list2 == NULL)) return 0;
  else                                    return -1;
}       



void sortArc(startArc, service)
     GRAPHIC_ARC *startArc;
     int         service;
/*
 * AU: Sortiere die Nachfolgerknoten nach der Stimme
 * SU: exchangeArcFromTo()
 */
{
  GRAPHIC_ARC *ptoArc;
  Bool        switched = True;

  if (startArc != NULL) {
     while (switched) {
	   switched = False;
           for (ptoArc = startArc; ptoArc->next_arc != NULL;
		ptoArc = ptoArc->next_arc) {
	       if (ptoArc->arc_to_object->quor[service] <
		   ptoArc->next_arc->arc_to_object->quor[service]) {
		   switched = True;
		   exchangeArcFromTo(ptoArc, ptoArc->next_arc);
	           }
	       }
	   }
     }
}




void exchangeArcFromTo(fromArc, toArc)
     GRAPHIC_ARC *fromArc, *toArc;
/*
 * AU: Tausch der Positionen zweier Nachfolgerknoten
 */
{
  int i;
  int holdArcValue;
  GRAPHIC_NODE *holdNode;
	   
  /* fprintf(stderr, "change %s kleiner als %s\n",
	           fromArc->arc_to_object->ident,
	toArc->arc_to_object->ident); */
						 
  holdNode = fromArc->arc_to_object;
  fromArc->arc_to_object = toArc->arc_to_object;
  toArc->arc_to_object = holdNode;
						       
  for (i = 0; i < data.numb_quor; i++) {
      holdArcValue = fromArc->arc_value[i];
      fromArc->arc_value[i] = toArc->arc_value[i];
      toArc->arc_value[i] = holdArcValue;
      }
}





void addSolutionToVQMSet(service)
     int service;
/*
 * AU: Fuege eine neue virtuelle Gruppe hinzu, aber entferne
 *     sie wieder falls sie bereits vorhanden ist
 * SU: insertElementIntoVQMSet();
 *     createVQMSet();
 *     existVQMSet();
 *     deleteVQMSet();
 */
{
  VQM_SET         *ptrVQMSetHead;
  REF_SET_ELEMENT *ptrCurrList;
	 
  ptrVQMSetHead = (VQM_SET *)createVQMSet();
	   
  for (ptrCurrList = ptrListOfReference; ptrCurrList != NULL;
       ptrCurrList = ptrCurrList->nextRefSetElement) {
      insertElementIntoVQMSet(ptrVQMSetHead, ptrCurrList->ptrRefItem);
      }
				
  if (existVQMSet(ptrVQMSetHead, service)) deleteVQMSet(ptrVQMSetHead);
  else {
        ptrVQMSetHead->nextVQMSetHead = ptrVQMSet[service];
	ptrVQMSet[service] = ptrVQMSetHead;
        }
}




Bool existVQMSet(ptrVQMSetHead, service)
     VQM_SET *ptrVQMSetHead;
     int     service;
/*
 * AU: Pruefe die Existenz einer virtuellen Gruppe
 * SU: listcmpVQMSet();
 */    
{
  VQM_SET *ptrCurrVQMSet;
	    
  for (ptrCurrVQMSet = ptrVQMSet[service]; ptrCurrVQMSet != NULL;
       ptrCurrVQMSet = ptrCurrVQMSet->nextVQMSetHead) {
      if (listcmpVQMSet(ptrCurrVQMSet->firstVQMSetElement,
		        ptrVQMSetHead->firstVQMSetElement)) return True;
      }
  return False;
}



Bool listcmpVQMSet(firstElementSet1, firstElementSet2)
     VQM_SET_ELEMENT *firstElementSet1, *firstElementSet2;
/*
 * AU: Vergleich zweier virtuellen Gruppen
 */
{
  VQM_SET_ELEMENT *set1, *set2;
       
  set1 = firstElementSet1;
  set2 = firstElementSet2;
	   
  while ((set1 != NULL) && (set2 != NULL)) {
	if (strncmp(set1->identVQMSet, set2->identVQMSet, 5) != 0) return False;
        else {
             set1 = set1->nextVQMSetElement;
	     set2 = set2->nextVQMSetElement;
	     }
        }
									    
  if ((set1 == NULL) && (set2 == NULL)) return True;
  else                                  return False;
}


void insertElementIntoVQMSet(ptrVQMSetHead, ptrGraphicObject)
     VQM_SET      *ptrVQMSetHead;
     GRAPHIC_NODE *ptrGraphicObject;
/*
 * AU: Fuege einen Rechner der virtuellen Gruppen hinzu
 * SU: insertItemIntoVQMSet();
 */
{
  GRAPHIC_IDENT *ptrIdent;
	      
  if (ptrGraphicObject->version != 0) { /* virtueller Rechner */
     for (ptrIdent = ptrGraphicObject->firstIdent; ptrIdent != NULL;
	  ptrIdent = ptrIdent->nextListIdent) {
	/*  insertItemIntoVQMSet(ptrVQMSetHead, ptrIdent->identItem); */
	 }
     }
  else { /* physikalischer Rechner */
       insertItemIntoVQMSet(ptrVQMSetHead, ptrGraphicObject->ident);
       }
}



void insertItemIntoVQMSet(ptrVQMSetHead, ident)
     VQM_SET *ptrVQMSetHead;
     char    ident[5];
/*
 * AU: Sortiere ein Element der virtuellen Gruppe hinzu
 * SU: notMemberOfSet();
 *     createVQMSetElement();
 */
{
  VQM_SET_ELEMENT *ptrCurrSetElement;
  VQM_SET_ELEMENT *ptrOldSetElement;
  VQM_SET_ELEMENT *ptrNewSetElement;
		                      
  if (notMemberOfSet(ptrVQMSetHead, ident)) {
     ptrNewSetElement = (VQM_SET_ELEMENT *)createVQMSetElement();
     strcpy(ptrNewSetElement->identVQMSet, ident);
						             
							           
     ptrCurrSetElement = ptrVQMSetHead->firstVQMSetElement;
									     
     if (ptrCurrSetElement == NULL) { /* Liste ist leer */
        ptrVQMSetHead->firstVQMSetElement = ptrNewSetElement;
        }
     else {
	  if (strncmp(ptrCurrSetElement->identVQMSet, ident, 5) > 0) {
             ptrVQMSetHead->firstVQMSetElement = ptrNewSetElement;
             }
          else {
	       ptrOldSetElement = ptrCurrSetElement;
	       ptrCurrSetElement = ptrCurrSetElement->nextVQMSetElement;
	       while (ptrCurrSetElement != NULL) {
		     if (strncmp(ptrCurrSetElement->identVQMSet,
		                 ident, 5) > 0) {
			break;
		        }
		     ptrOldSetElement = ptrCurrSetElement;
		     ptrCurrSetElement =
		        ptrCurrSetElement->nextVQMSetElement;
		     }
	       ptrOldSetElement->nextVQMSetElement = ptrNewSetElement;
	       }
	  ptrNewSetElement->nextVQMSetElement = ptrCurrSetElement;
          }
      }
}




Bool notMemberOfSet(ptrVQMSetHead, ident)
     VQM_SET *ptrVQMSetHead;
     char    ident[5];
/*
 * AU: Pruefe ob Element bereits in der virtuellen Gruppe enthalten
 */
{
  VQM_SET_ELEMENT *ptrElement;
	     
  for (ptrElement = ptrVQMSetHead->firstVQMSetElement; ptrElement != NULL;
       ptrElement = ptrElement->nextVQMSetElement) {
      if (strncmp(ptrElement->identVQMSet, ident, 5) == 0) return False;
      }
  return True;           
}



VQM_SET *createVQMSet()   
/*        
 * AU: Erzeuge das Kopfelement einer neuen virtuellen Gruppe
 */           
{
  VQM_SET *ptrVQMSetHead;
             
  ptrVQMSetHead = (VQM_SET *)XtMalloc(sizeof(VQM_SET));
	       
  demand(ptrVQMSetHead, memory allocation in createHeadOfVQMSet failed);
		             
  ptrVQMSetHead->firstVQMSetElement = NULL;
			         
  return ptrVQMSetHead;
}



VQM_SET_ELEMENT *createVQMSetElement()
/*         
 * AU: Erzeuge ein Element einer neuen virtuellen Gruppe
 */
{ 
  VQM_SET_ELEMENT *ptrVQMSetElement;
            
  ptrVQMSetElement =
	      (VQM_SET_ELEMENT *)XtMalloc(sizeof(VQM_SET_ELEMENT));
			            
  demand(ptrVQMSetElement, memory allocation in addVQMSetElement failed);
				           
  ptrVQMSetElement->nextVQMSetElement = NULL;
					                  
  return ptrVQMSetElement;
}






GRAPHIC_SET *createHeadOfNodeSet(ptrGraphicObject, service)
            GRAPHIC_NODE *ptrGraphicObject;
	    int          service;
/* 
 * AU: Erzeuge das Kopfelement der virtuellen Gruppen einer Operation
 */
{
  GRAPHIC_SET *ptrSetHead;
			   
  ptrSetHead = (GRAPHIC_SET *)XtMalloc(sizeof(GRAPHIC_SET));
			      
  demand(ptrSetHead, memory allocation in createHeadOfNodeSet failed);
				       
  ptrSetHead->nextNodeSetHead = NULL;
  ptrSetHead->firstNodeSetElement = NULL;
					     
  if (ptrGraphicObject->firstNodeSetHead[service] != NULL) {
     ptrSetHead->nextNodeSetHead = ptrGraphicObject->firstNodeSetHead[service];
     }

  ptrGraphicObject->firstNodeSetHead[service] = ptrSetHead;

  return ptrSetHead;
}





void addNodeSetElement(ptrGraphicObjectSet, ptrGraphicObject)
     GRAPHIC_SET  *ptrGraphicObjectSet;
     GRAPHIC_NODE *ptrGraphicObject;
/*
 * AU: fuege eine Kombination aus Nachfolgern dem Vaterknoten hinzu   
 * SU: demand()
 */
{
  GRAPHIC_SET_ELEMENT *ptrSetElement;
	           
  ptrSetElement =
	      (GRAPHIC_SET_ELEMENT *)XtMalloc(sizeof(GRAPHIC_SET_ELEMENT));
				            
  demand(ptrSetElement, memory allocation in addSetElement failed);
					             
  ptrSetElement->nextNodeSetElement = NULL;
  ptrSetElement->ptrSetItem = NULL;
							      
  if (ptrGraphicObjectSet->firstNodeSetElement != NULL)
     ptrSetElement->nextNodeSetElement =
		    ptrGraphicObjectSet->firstNodeSetElement;
		
  ptrGraphicObjectSet->firstNodeSetElement = ptrSetElement;

  ptrSetElement->ptrSetItem = ptrGraphicObject;
}




void removeObjectSetFromRefList(memberOfSet)
     GRAPHIC_SET     *memberOfSet;
/*
 * AU: Entferne nicht erfolgreich gebildete virtuelle Gruppen
 * SU: removeObjectFromRefList();
 */
{
  GRAPHIC_SET_ELEMENT *graphicObjSetElement;

  for (graphicObjSetElement = memberOfSet->firstNodeSetElement;
       graphicObjSetElement != NULL;
       graphicObjSetElement = graphicObjSetElement->nextNodeSetElement) {

      if (ptrCurrRefList != NULL)
         ptrCurrRefList = ptrCurrRefList->beforeRefSetElement;

      if (ptrCurrRefList != NULL) {
	 removeObjectFromRefList(ptrCurrRefList->nextRefSetElement->ptrRefItem);
	 }
      }
}




void removeObjectFromRefList(ptrRemoveObject)
     GRAPHIC_NODE    *ptrRemoveObject;
/* 
 * AU: Entferne ein einzelnes Element aus der Referenzliste
 */
{
  REF_SET_ELEMENT *delItem;

  if(ptrListOfReference->ptrRefItem == ptrRemoveObject) {
    delItem = ptrListOfReference;
    ptrListOfReference = ptrListOfReference->nextRefSetElement;
    ptrListOfReference->beforeRefSetElement = NULL;
    }       
  else {
       for (delItem = ptrListOfReference->nextRefSetElement;
            delItem != NULL;
	    delItem = delItem->nextRefSetElement) {
	   if (delItem->ptrRefItem == ptrRemoveObject) {
	      delItem->beforeRefSetElement->nextRefSetElement =
                       delItem->nextRefSetElement;
	      if (delItem->nextRefSetElement != NULL) {
		 delItem->nextRefSetElement->beforeRefSetElement =
		          delItem->beforeRefSetElement;
                 }
              }
           }
       } 		
  XtFree((char *)delItem);
}





void addObjectSetToRefList(ptrCurrListElement, graphicObjectSet)
     REF_SET_ELEMENT *ptrCurrListElement;
     GRAPHIC_SET     *graphicObjectSet;
{ 
  GRAPHIC_SET_ELEMENT *ptrNextSetElement;
  REF_SET_ELEMENT     *ptrListElement;
	        
  ptrListElement = ptrCurrListElement;
		                
  for (ptrNextSetElement = graphicObjectSet->firstNodeSetElement;
       ptrNextSetElement != NULL;
       ptrNextSetElement = ptrNextSetElement->nextNodeSetElement) {
      addObjectToRefList(ptrListElement, ptrNextSetElement->ptrSetItem);
      ptrListElement = ptrListElement->nextRefSetElement;
      }
}



REF_SET_ELEMENT *addObjectToRefList(ptrCurrListElement, ptrGraphicObject)
                REF_SET_ELEMENT *ptrCurrListElement;
		GRAPHIC_NODE    *ptrGraphicObject;
/*
 * AU: Fuege eine Menge von Nachfolgern der Referenzliste hinzu
 * SU: addObjectToRefList()
 */
{
  REF_SET_ELEMENT *ptrToListElement;
				      
  ptrToListElement = (REF_SET_ELEMENT*)XtMalloc(sizeof(REF_SET_ELEMENT));
					    
  demand(ptrToListElement, memory allocation addReferenceListElement failed);
					        
  ptrToListElement->ptrRefItem = ptrGraphicObject;
						              
  if (ptrCurrListElement == NULL) {
     ptrToListElement->nextRefSetElement = NULL;

     ptrToListElement->beforeRefSetElement = NULL;

     ptrListOfReference = ptrToListElement;
     }
  else {
       ptrToListElement->nextRefSetElement =
			 ptrCurrListElement->nextRefSetElement;
			
       ptrToListElement->beforeRefSetElement = ptrCurrListElement;

       ptrCurrListElement->nextRefSetElement = ptrToListElement;
       }
  return ptrToListElement;
}



REF_SET_ELEMENT *getRefListElement(ptrGraphicObject)
                GRAPHIC_NODE    *ptrGraphicObject;
/*
 * AU: Ermittle das Element der Referenzliste zu einem Rechner der
 *     Voting-Struktur
 */
{ 
  REF_SET_ELEMENT *ptrToListElement;
		                
  for (ptrToListElement = ptrListOfReference;
       ptrToListElement != NULL;
       ptrToListElement = ptrToListElement->nextRefSetElement) {
      if (ptrGraphicObject == ptrToListElement->ptrRefItem)
	 return ptrToListElement;
      }
  return NULL;           
}

/***************************************************************************/
/*                    Ende der Funktionen                                  */
/***************************************************************************/

