/*
Name :  motif.c
Zweck:  Benutzeroberflaeche aufbauen und Initialisierung
Autor:  Andreas Seifert
Datum:
Update:
*/
/*-------------------------------------------------------------------------*/


/****************************************************************************/

#include "motif.h"
#include "draw.h"
#include "main.h"

/****************************************************************************/

/*
 * AU: Aufbau der Oberflaeche
 * ST: main()                     Aufbau der Anwendung, Registrierung der 
 *                                Benutzeraktionen
 *     CreateWidgetHierarchie()   Aufbau der Widget-Hierarchie
 *     CreatePopupShells()        Erzeugen der Dialogfenster
 *     MemCB()                    Registrierung der Widget-IDs
 *     FetchDialogWidgets()       Einlesen der Dialogfenster
 *     XtError()                  Fehlermeldungen
 *     XError()                   Fehlermeldungen
 *     wprint()                   Ausgabe von Meldungen im Nachrichtenbereich
 */

/****************************************************************************/
/************************                           *************************/
/************************ Beginn des Hauptprogramms *************************/
/************************                           *************************/
/****************************************************************************/

int
main (argc, argv)
  int    argc;
  char   *argv[];
/*
 * AU: Aufbau der Anwendung
 * ST: Aufruf CreateWidgetHierarchie()
 *            CreatePopupShells()
 *            FetchDialogWidgets()
 *            initGraphik()
 *            activateZoom()
 *            FetchWorkingDirectory()
 *            printStatus()                     
 */
{
   MrmHierarchy  hierarchie;

   /*
    * Fallback-Ressourcen
    */
   static String fallback_resources[] = {
   "Xvsed.title :                            Voting-Struktur-Editor Fallback-Ressourcen",
   "*datei.labelString :                     Datei",
   "*new_tree.labelString :                  Neu_Voting-Struktur",
   "*new_tree.borderWidth :                  0",
   "*oeffnen_net.labelString :               Oeffne_Netz",
   "*oeffnen_net.borderWidth :               0",
   "*oeffnen_tree.labelString :              Oeffne_Voting-Struktur",
   "*oeffnen_tree.borderWidth :              0",
   "*save_yn.labelString :                   Sichere_Voting-Struktur",
   "*save_yn.borderWidth :                   0",
   "*beenden.labelString :                   Beenden",
   "*beenden.borderWidth :                   0",
   "*hilfe.labelString :                     Hilfe",
   "*run.labelString :                       Analyse",
   "*checkVQM.labelString :                  Pruefe_Voting-Struktur",
   "*checkVQM.borderWidth :                  0",
   "*computeVQM.labelString :                Berechne_Voting-Struktur",
   "*computeVQM.borderWidth :                0",
   "*computeReli.labelString :               Berechne_Wahrscheinlichkeit",
   "*computeReli.borderWidth :               0",
   "*main.width :                            750",  
   "*main.height :                           650",
   "*menuLeiste.borderWidth :                1",
   "*helpmessageDialog_popup.title :         Nachricht",  
   "*helpmessageDialog.messageString :       Eine Datei auswaehlen",
   "*helpmenueDialog_popup.title :           Nachricht",
   "*helpmenueDialog.messageString :         Hilfe nicht installiert",
   "*nodehelpmessageDialog_popup.title :     Nachricht",
   "*nodehelpmessageDialog.messageString :   Ungueltiger Name! Neuer Versuch mit OK, Ende mit Abbruch",
   "*nodehelpmessageDialog.cancelLabelString:Abbruch",
   "*popupFileNamePromptShell.title :        Eingabe", 
   "*message_label.labelString :             Eingabe Dateinamen ohne Suffix",
   "*message_label.borderWidth :             0",
   "*scale_message.labelString :             Quorenanzahl ggf. aendern",
   "*scale_message.borderWidth :             0",
   "*file_prompt.borderWidth :               1",
   "*promptButtons.borderWidth :             0",
   "*ok_promptPB.labelString :               OK",
   "*cancel_promptPB.labelString :           Abbruch",  
   "*openfsdDialog_popup.title :             Dateiauswahl",
   "*openfsdDialog.filterLabelString :       Datei-Filter:",
   "*openfsdDialog.dirListLabelString :      Verzeichnisse:",
   "*openfsdDialog.fileListLabelString :     Dateien:",
   "*openfsdDialog.selectionLabelString :    Auswahl:",
   "*openfsdDialog.applyLabelString :        Filtern",
   "*openfsdDialog.cancelLabelString :       Abbruch",
   "*openfsdDialog.helpLabelString :         Hilfe",
   "*fileNameDialog_popup.title :            Dateisicherung",
   "*fileNameDialog.selectionLabelString :   Datei abspeichern ?",
   "*fileNameDialog.okLabelString :          Abspeichern",
   "*fileNameDialog.cancelLabelString:       Abbruch",
   "*fileNameDialog.helpLabelString:         Hilfe",
   "*editBox.width :                         244",
   "*editBox.height :                        500",
   "*editBox.borderWidth :                   0",
   "*editBox.mappedWhenManaged :             false",
   "*nodeLabel.labelString :                 Rechner:",
   "*arcLabel.labelString :                  Beziehung:",
   "*zoomLabel.labelString :                 Zoom:",  
   "*nodeLabel.borderWidth :                 0", 
   "*arcLabel.borderWidth :                  0",
   "*zoomLabel.borderWidth :                 0",
   "*nodeInsert.labelString :                Einfuegen",
   "*nodeInsert.borderWidth :                1",
   "*nodeDelete.labelString :                Entfernen",
   "*nodeDelete.borderWidth :                1",
   "*nodeMove.labelString :                  Verschieben",
   "*nodeMove.borderWidth :                  1",
   "*nodeInfo.labelString :                  Veraendern",
   "*nodeInfo.borderWidth :                  1",  
   "*arcInsert.labelString :                 Einfuegen",
   "*arcInsert.borderWidth :                 1",
   "*arcDelete.labelString :                 Entfernen",
   "*arcDelete.borderWidth :                 1",
   "*arcInfo.labelString :                   Veraendern",
   "*arcInfo.borderWidth :                   1",
   "*endFunction.labelString :               Funktion beenden",
   "*endFunction.borderWidth :               1",
   "*endFunction.sensitive :                 false",   
   "*percent50.labelString :                 50%",
   "*percent50.borderWidth :                 1",
   "*percent50.sensitive :                   false",
   "*percent75.labelString :                 75%",
   "*percent75.borderWidth :                 1",
   "*percent75.sensitive :                   false",
   "*percent100.labelString :                100%",
   "*percent100.borderWidth :                1",
   "*percent100.sensitive :                  false",
   "*scrolledNodelist.width :                240",
   "*scrolledNodelist.height :               200",
   "*scrolledNodelist.borderWidth :          0",  
   "*scrolledNodelist.mappedWhenManaged :    false",
   "*nodelist.borderWidth :                  0",
   "*nodelist.listMarginHeight :             10",
   "*nodelist.listMarginWidth :              47", 
   "*nodelist.listSpacing :                  7",  
   "*scrolledGraphik.width :                 500",
   "*scrolledGraphik.height :                500",
   "*scrolledGraphik.borderWidth :           1",
   "*graphik.width :                         1500",
   "*graphik.height :                        1500",
   "*graphik.borderWidth :                   0",  
   "*message.width :                         1500",
   "*message.height :                        1500", 
   "*message.borderWidth :                   0",
   "*nodeNameDialog_popup.title :            Virtueller Rechner",
   "*nodeNameDialog.selectionLabelString :   Rechnername (10 Zeichen) z.B. R1-2,5,6",
   "*nodeNameDialog.textString :             R",   
   "*nodeNameDialog.cancelLabelString :      Abbruch",
   "*nodeNameDialog.helpLabelString :        Text loeschen",
   "*popupNodeModifyShell.title :            Knotenbelegung editieren",
   "*popupNodeModifyShell.height :           215",
   "*popupNodeModifyShell.width :            510",
   "*popupNodeModifyShell.minWidth :         500",
   "*popupNodeModifyShell.minHeight :        200",
   "*popupNodeModifyShell.maxWidth :         520",
   "*popupNodeModifyShell.maxHeight :        220",
   "*node_part_one.borderWidth :             0",    
   "*node_part_two.borderWidth :             0",
   "*node_part_three.borderWidth :           0",  
   "*node_part_four.borderWidth :            0",
   "*node_part_five.borderWidth :            0",   
   "*node_part_six.borderWidth :             0",
   "*node_Votes.borderWidth :                0",
   "*node_quorum_one.borderWidth :           0",
   "*node_quorum_two.borderWidth :           0",   
   "*node_quorum_three.borderWidth :         0",
   "*node_quorum_four.borderWidth :          0",  
   "*node_quorum_five.borderWidth :          0",
   "*nodeLabel_one.labelString :             Quorum Nr.1",
   "*nodeLabel_two.labelString :             Quorum Nr.2",
   "*nodeLabel_three.labelString :           Quorum Nr.3",
   "*nodeLabel_four.labelString :            Quorum Nr.4",
   "*nodeLabel_five.labelString :            Quorum Nr.5",
   "*nodeLabel_six.labelString :             Stimmen",
   "*nodeLabel_one.borderWidth :             0",
   "*nodeLabel_two.borderWidth :             0",  
   "*nodeLabel_three.borderWidth :           0",  
   "*nodeLabel_four.borderWidth :            0",  
   "*nodeLabel_five.borderWidth :            0",
   "*nodeLabel_six.borderWidth :             0",
   "*nodeModify.borderWidth :                0", 
   "*nodeButtons.borderWidth :               0",  
   "*ok_node_modifyPB.labelString :          Werte uebernehmen",
   "*cancel_node_modifyPB.labelString :      Anzeige beenden",
   "*popupArcModifyShell.title :             Kantenbelegung editieren",
   "*popupArcModifyShell.height :            215",
   "*popupArcModifyShell.width :             450", 
   "*arc_part_one.borderWidth :              0",   
   "*arc_part_two.borderWidth :              0",  
   "*arc_part_three.borderWidth :            0",   
   "*arc_part_four.borderWidth :             0",    
   "*arc_part_five.borderWidth :             0",
   "*arcScale_one.borderWidth :              0",
   "*arcScale_two.borderWidth :              0",
   "*arcScale_three.borderWidth :            0",   
   "*arcScale_four.borderWidth :             0",
   "*arcScale_five.borderWidth :             0",
   "*arcLabel_one.labelString :              Prioritaet 1",
   "*arcLabel_two.labelString :              Prioritaet 2",
   "*arcLabel_three.labelString :            Prioritaet 3",
   "*arcLabel_four.labelString :             Prioritaet 4",
   "*arcLabel_five.labelString :             Prioritaet 5",
   "*arcLabel_one.borderWidth :              0",  
   "*arcLabel_two.borderWidth :              0",  
   "*arcLabel_three.borderWidth :            0",    
   "*arcLabel_four.borderWidth :             0",
   "*arcLabel_five.borderWidth :             0",  
   "*arcModify.borderWidth :                 0",
   "*arcButtons.borderWidth :                0",   
   "*ok_arc_modifyPB.labelString :           Werte uebernehmen",
   "*ok_arc_modifyPB.borderWidth :           0",
   "*cancel_arc_modifyPB.labelString :       Anzeige beenden",
   "*cancel_arc_modifyPB.borderWidth :       0",   
   "*top_label.labelString :                 Netztopologiedatei:",
   "*top_label.borderWidth :                 0",
   "*vot_label.labelString :                 Votingstrukturdatei:",
   "*vot_label.borderWidth :                 0",
   "*res1_label.labelString :                Verfuegbarkeit Dienst 1:",
   "*res1_label.borderWidth :                0",
   "*res2_label.labelString :                Verfuegbarkeit Dienst 2:",
   "*res2_label.borderWidth :                0",
   "*res3_label.labelString :                Verfuegbarkeit Dienst 3:",
   "*res3_label.borderWidth :                0",
   "*res4_label.labelString :                Verfuegbarkeit Dienst 4:",
   "*res4_label.borderWidth :                0",
   "*res5_label.labelString :                Verfuegbarkeit Dienst 5:",
   "*res5_label.borderWidth :                0",
   "*accept_resultPB.labelString :           Anzeige beenden",
   "*accept_resultPB.borderWidth :           0",
   "*containTop.borderWidth :                0",
   "*containVot.borderWidth :                0",
   "*containRes1.borderWidth :               0",
   "*containRes2.borderWidth :               0",
   "*containRes3.borderWidth :               0",
   "*containRes4.borderWidth :               0",
   "*containRes5.borderWidth :               0",
   "*top_file.borderWidth :                  0",
   "*vot_file.borderWidth :                  0",
   "*res1_label.labelString :                Verfuegbarkeit Dienst 1:",
   "*res1_label.borderWidth :                0",
   "*res2_label.labelString :                Verfuegbarkeit Dienst 2:",
   "*res2_label.borderWidth :                0",
   "*res3_label.labelString :                Verfuegbarkeit Dienst 3:",
   "*res3_label.borderWidth :                0",
   "*res4_label.labelString :                Verfuegbarkeit Dienst 4:",
   "*res4_label.borderWidth :                0",
   "*res5_label.labelString :                Verfuegbarkeit Dienst 5:",
   "*res5_label.borderWidth :                0",
   "*accept_resultPB.labelString :           Anzeige beenden",
   "*accept_resultPB.borderWidth :           0",
   "*containTop.borderWidth :                0",
   "*containVot.borderWidth :                0",
   "*containRes1.borderWidth :               0",
   "*containRes2.borderWidth :               0",
   "*containRes3.borderWidth :               0",
   "*containRes4.borderWidth :               0",
   "*containRes5.borderWidth :               0",
   "*top_file.borderWidth :                  0",
   "*vot_file.borderWidth :                  0",
   "*result1.borderWidth :                   0",
   "*result2.borderWidth :                   0",
   "*result3.borderWidth :                   0",
   "*result4.borderWidth :                   0",
   "*result5.borderWidth :                   0",
   "*scrollTop.borderWidth :                 0",
   "*scrollVot.borderWidth :                 0",
   "*popupPresentResultShell.title :         Verfuegbarkeit der Dienste",
   NULL
   };

   /*
    * Kommandozeilenparameter
    */
   static XrmOptionDescRec optionen[] = {
     {"-label",      ".title",          XrmoptionSepArg, NULL},  
     {"-width",      ".main.width",     XrmoptionSepArg, NULL},
     {"-height",     ".main.height",    XrmoptionSepArg, NULL},
     {"-widthDraw",  "*graphik.width",  XrmoptionSepArg, NULL},
     {"-heightDraw", "*graphik.height", XrmoptionSepArg, NULL}
   };

   /*
    * Toolkit initialisieren, Shell-Widget erzeugen
    */
   Widgetliste[APPL_SHELL] = 
    XtAppInitialize(&app_context,            /* Applikationskontext */
                    APPL_KLASSE,             /* Klassenname der Applikation */
                    optionen,                /* besonderen Optionen */
                    XtNumber(optionen),      /* Eintraege Optionsliste */
                    &argc,                   /* Parameteranzahl Kommandozeile */
                    argv,                    /* Kommandozeilenparameter */
                    fallback_resources,      /* Fallback-Ressourcen */
                    (Arg*)NULL,              /* Ressourcen-Argumentliste */
                    0);                      /* Anzahl Argumente */


   /*
    * Hinweis auf den Gebrauch der Kommandozeilenparameter 
    */
   if (argc != 1) printUsage();

   /*
    * Installation der Fehlerroutinen
    */
   /* Xt(App)Error() erwartet als 2. argument "_Xconst _XtString" == "char *" statt
    * String, daher die casts */
   XtAppSetErrorHandler(app_context, (XtErrorHandler)XtAppError); 
   XtAppSetWarningHandler(app_context, (XtErrorHandler)XtError);  
   XSetErrorHandler(XError);
   
   /*
    * Widgets unter der Applikations-Shell erzeugen und realisieren
    */
   CreateWidgetHierarchie(Widgetliste[APPL_SHELL], &hierarchie);
   
   /*
    * PopupShells erzeugen
    */ 
   CreatePopupShells(Widgetliste[APPL_SHELL], &hierarchie);

   /*
    * Widget-Baum realisieren
    */
   XtRealizeWidget(Widgetliste[APPL_SHELL]);
     
   /*
    * Widget-ID`s der UIL-Datei entnehmen
    */
   FetchDialogWidgets(Widgetliste[APPL_SHELL], &app_context, &hierarchie);
     
   /*
    * initialisiere die Graphik fuer die Drawing Area
    */
   initGraphik();
                    
   /*
    * aktiviere die Zoom-Option
    */
   activateZoom();
                    
   /*
    * Pfadname des aktuellen Arbeitsverzeichnisses entnehmen
    */
   FetchWorkingDirectory();

   /*
    * Statusmeldungen
    */
   printStatus();     

   /*
    * Events lesen und an die entsprechenden Prozeduren zur Verarbeitung
    * weiterleiten
    */
   XtAppMainLoop(app_context);
   
   return 0;
   
}

    
/*************************************************************************/
/**********************                         **************************/
/********************** Ende des Hauptprogramms **************************/
/**********************                         **************************/
/*************************************************************************/
   


void CreateWidgetHierarchie(parent, hierarchie)   
     Widget        parent;
     MrmHierarchy  *hierarchie;
/*
 * AU: Widget-Baum unter der Applikations-Shell
 *     anhand UIL/UID-Beschreibung erzeugen
 */
{
   Widget       main = NULL;   /* main aus UIL */
   MrmType      widgetType;    /* Klasse von main */
   Cardinal     status;
   struct stat  statbuf;       /* Struktur mit Dateiinformationen */     
   int          i;             /* Laufvariable */

   static char *dateienUID[] = {  /* Namen der UID-Dateien */
           /* der Arbeitsfeld */
           "work.uid",
           /* Dialogfenster */
           "popup.uid"
   };
      
   static MrmRegisterArg callbacks[] = { /* Zuordnung UIL-C-Callbacks */
           { "MenuCB",           (caddr_t)MenuCB },
           { "MemCB",            (caddr_t)MemCB },
           { "EditCB",           (caddr_t)EditCB },  
           { "ZoomCB",           (caddr_t)ZoomCB },
           { "RunCB",            (caddr_t)RunCB },
           { "exposeCB",         (caddr_t)exposeCB },
           { "listCB",           (caddr_t)listCB },
           { "scaleCB",          (caddr_t)scaleCB },
           { "modifyCB",         (caddr_t)modifyCB },
           { "file_SelectCB",    (caddr_t)file_SelectCB },
           { "fileNameDialogCB", (caddr_t)fileNameDialogCB },
           { "showResultsCB",    (caddr_t)showResultsCB }, 
   };
    
   /* 
    * MRM initialisieren und die Anwendung auf das Einlesen der 
    * Widgethierarchie vorbereiten
    */
   MrmInitialize();

   /* Existenz der UID-Dateien pruefen */
   for (i = 0; i < XtNumber(dateienUID); i++) {
       if (stat(dateienUID[i], &statbuf) != 0)
          fprintf(stderr, missingUID, dateienUID[i]);
       }
    
   /* 
    * MRM-Hierarchie mit UID-Dateien oeffnen
    */
   status = MrmOpenHierarchy(XtNumber(dateienUID), dateienUID,
                             (MrmOsOpenParamPtr*)NULL, hierarchie);
           
   if (status != MrmSUCCESS) {
       fprintf (stderr, "UID-Datein koennen nicht gelesen werden\n");
       exit(BAD);
      }    
           
   /*
    * Callbacks fuer UIL registrieren
    */
   status = MrmRegisterNames(callbacks, XtNumber(callbacks));

   if (status != MrmSUCCESS) {
       fprintf (stderr, "Callbacks koennen nicht registriert werden\n");
       exit(BAD);
      }
     
   /*
    * Widget-Hierarchie einlesen und aufbauen. Das oberste
    * Widget dieser Hierarchie wird 'main' zugewiesen
    */
   status = MrmFetchWidget(*hierarchie, "main", parent, &main, &widgetType);
   
   if (status != MrmSUCCESS) {
       fprintf (stderr, "Widget-Hierarchie kann nicht erzeugt werden\n");
       exit(BAD);
      }
       
   XtManageChild(main);
}    

      
   
void CreatePopupShells(parent, hierarchie)
     Widget       parent;
     MrmHierarchy *hierarchie;
/*
 * AU: Shells der Dialogfenster erzeugen
 */  
{   
   Widget   shell;
   MrmType  widgetType;
   Cardinal status;

   /* Dialogfenster zur Modifikation der Stimmen und Quoren der Knoten */    
   Widgetliste[NODE_MODIFY_SHELL] =
     XtVaCreatePopupShell("popupNodeModifyShell", vendorShellWidgetClass,
                          parent,
                          NULL);

   shell = NULL;
   
   status = MrmFetchWidget(*hierarchie, "nodeModifyCont",
                          Widgetliste[NODE_MODIFY_SHELL], &shell, &widgetType);
     
   if (status != MrmSUCCESS) {   
       fprintf(stderr, "nodeModify nicht geladen\n");
       exit(BAD);
      }

   XtManageChild(shell);

   /* Dialogfenster zur Modifikation der Zugriffsrelationen */   
   Widgetliste[ARC_MODIFY_SHELL] =
     XtVaCreatePopupShell("popupArcModifyShell", vendorShellWidgetClass,
                          parent,
                          NULL);
                          
   shell = NULL;
   
   status = MrmFetchWidget(*hierarchie, "arcModifyCont",
                          Widgetliste[ARC_MODIFY_SHELL], &shell, &widgetType);
     
   if (status != MrmSUCCESS) {   
       fprintf(stderr, "arcModify nicht geladen\n");
       exit(BAD);
      }
     
   XtManageChild(shell);

   /* Dialogfenster zur Eingabe Quorenanzahl und Dateiname */
   Widgetliste[FILE_NAME_SHELL] =
     XtVaCreatePopupShell("popupFileNamePromptShell", vendorShellWidgetClass,
                          parent,
                          NULL);
                          
   shell = NULL;

   status = MrmFetchWidget(*hierarchie, "fileNamePromptCont",
                          Widgetliste[FILE_NAME_SHELL], &shell, &widgetType);
       
   if (status != MrmSUCCESS) {
       fprintf(stderr, "FileNamePromptCont nicht geladen\n");
       exit(BAD);
      }                   

   XtManageChild(shell);

   /* Dialogfenster zur Praesentation von Ergebnissen */                    
   Widgetliste[PRES_RES_SHELL] =
     XtVaCreatePopupShell("popupPresentResultShell", vendorShellWidgetClass,
                          parent,
                          NULL);
                       
   shell = NULL;
                    
   status = MrmFetchWidget(*hierarchie, "presentResultCont",
                          Widgetliste[PRES_RES_SHELL], &shell, &widgetType);

    if (status != MrmSUCCESS) {
        fprintf(stderr, "PresentResultCont nicht geladen\n");
        exit(BAD);
       }                  

   XtManageChild(shell);
   
   /*
    * Deaktivierung der Delete-Funktion im Windowmanagermenue (nur moeglich 
    * bei einer Anwendung unter Kontrolle des Motif Window Managers
    */
   if (XmIsMotifWMRunning(parent)) {
       fprintf(stderr, "Ausgabe unter Kontrolle des MotifWindowManagers\n");

       XtVaSetValues(parent,
                    XmNdeleteResponse, XmDO_NOTHING,
                    NULL);

       XtVaSetValues(Widgetliste[NODE_MODIFY_SHELL],
                    XmNdeleteResponse, XmDO_NOTHING,
                    NULL);

       XtVaSetValues(Widgetliste[ARC_MODIFY_SHELL],
                    XmNdeleteResponse, XmDO_NOTHING,
                    NULL);
                          
       XtVaSetValues(Widgetliste[FILE_NAME_SHELL],
                    XmNdeleteResponse, XmDO_NOTHING,
                    NULL);
                          
       XtVaSetValues(Widgetliste[PRES_RES_SHELL],
                    XmNdeleteResponse, XmDO_NOTHING,
                    NULL);
      }
   else fprintf(stderr, "Ausgabe ist nicht unter Kontrolle des MotifWindowManagers\n");
}

     

/*ARGSUSED*/
void MemCB (widget, clientDaten, aufrufDaten)
     Widget       widget;      /* nimmt die Widget-ID aus UIL-Datei auf */
     caddr_t      clientDaten, aufrufDaten;
/*
 * AU: Widget-IDs von Elementen der Oberflaeche registrieren
 */
{
   int n = *((int*)clientDaten);
     
   /*
    * Uebernahme der ID's der Widgets aus der UIL-Datei
    */
   Widgetliste[n] = widget;
}



void FetchDialogWidgets(parent_Widget, app_context, hierarchie)
     Widget       parent_Widget;
     XtAppContext *app_context;
     MrmHierarchy *hierarchie;
/*
 * AU: Dialogfenster aus der UIL-Beschreibung einlesen
 */
{
   MrmType widgetType;
   Widget  zwischenWidget = NULL;
     
   /*
    * ID der File-Selection-Box der UIL-Datei entnehmen
    */
   if ((MrmFetchWidget(*hierarchie, "openfsdDialog",
                       parent_Widget, &zwischenWidget,
                       &widgetType) != MrmSUCCESS)) {
       XtAppError(*app_context, "openfsdDialog nicht geladen");
       }
   
   Widgetliste[FILE_SELECT] = zwischenWidget;
    
   zwischenWidget = NULL;

   /*
    * ID einer Message-Box der UIL-Datei entnehmen, in UIL spezifizierte
    * Werte ueberschreiben  und mit einem eigenen Namen versehen 
    */
   if ((MrmFetchWidgetOverride(*hierarchie, "messageDialog",
                               parent_Widget, "helpmessageDialog",
                               NULL, 0, &zwischenWidget,
                               &widgetType) != MrmSUCCESS)) {
       XtAppError(*app_context, "HilfeDateiauswahlDialog nicht geladen");
       }

   Widgetliste[HELP_FILE_MESSAGE] = zwischenWidget;

   XtUnmanageChild(XmMessageBoxGetChild(Widgetliste[HELP_FILE_MESSAGE],
                   XmDIALOG_CANCEL_BUTTON));
   XtUnmanageChild(XmMessageBoxGetChild(Widgetliste[HELP_FILE_MESSAGE],
                   XmDIALOG_HELP_BUTTON));
        
   zwischenWidget = NULL;


   /*
    * ID einer Message-Box der UIL-Datei entnehmen, in UIL spezifizierte 
    * Werte ueberschreiben  und mit einem eigenen Namen versehen
    */  	          
   if ((MrmFetchWidgetOverride(*hierarchie, "hilfeDialog",
                               parent_Widget, "helpmenueDialog",
                               NULL, 0, &zwischenWidget,
                               &widgetType) != MrmSUCCESS)) {
       XtAppError(*app_context, "HilfeMenueDialog nicht geladen");
       }
   
   Widgetliste[HELP_MENU_MESSAGE] = zwischenWidget;
                               
   XtUnmanageChild(XmMessageBoxGetChild(Widgetliste[HELP_MENU_MESSAGE],
                   XmDIALOG_CANCEL_BUTTON));
   XtUnmanageChild(XmMessageBoxGetChild(Widgetliste[HELP_MENU_MESSAGE],
                   XmDIALOG_HELP_BUTTON));

   zwischenWidget = NULL;
                               
   /*
    * ID einer Message-Box der UIL-Datei entnehmen, in UIL spezifizierte 
    * Werte ueberschreiben  und mit einem eigenen Namen versehen
    */
   if ((MrmFetchWidgetOverride(*hierarchie, "messageDialog",
                               parent_Widget, "nodehelpmessageDialog",
                               NULL, 0, &zwischenWidget,
                               &widgetType) != MrmSUCCESS)) {
       XtAppError(*app_context, "NodeHilfeDialog nicht geladen");
       }
                               
   Widgetliste[NODE_MESSAGE] = zwischenWidget;
                 
   XtUnmanageChild(XmMessageBoxGetChild(Widgetliste[NODE_MESSAGE],
                   XmDIALOG_HELP_BUTTON));
   
   XtAddCallback(Widgetliste[NODE_MESSAGE], XmNokCallback,
                 (XtCallbackProc)resultNodeNameCB, (caddr_t)NULL);
 
   XtAddCallback(Widgetliste[NODE_MESSAGE], XmNcancelCallback,
                 (XtCallbackProc)resultNodeNameCB, (caddr_t)NULL);
                               
   zwischenWidget = NULL;
               
   /*
    * ID eines PromptDialogs zur Texteingabe (Bezeichnung eines virtuellen
    * Rechners) der UIL-Datei entnehmen
    */       
   if ((MrmFetchWidgetOverride(*hierarchie, "nameDialog",
                               parent_Widget, "nodeNameDialog",
                               NULL, 0, &zwischenWidget,
                               &widgetType) != MrmSUCCESS)) {
       XtAppError(*app_context, "nodeNameDialog nicht geladen");
       }
        
   Widgetliste[NODE_NAME_PROMPT] = zwischenWidget;
                 
   XtAddCallback(Widgetliste[NODE_NAME_PROMPT], XmNhelpCallback,  
                 (XtCallbackProc)nodeNameDialogCB, (caddr_t)NULL);   
 
   XtAddCallback(Widgetliste[NODE_NAME_PROMPT], XmNokCallback,
                 (XtCallbackProc)nodeNameDialogCB, (caddr_t)NULL);
   
   XtAddCallback(Widgetliste[NODE_NAME_PROMPT], XmNcancelCallback,
                 (XtCallbackProc)nodeNameDialogCB, (caddr_t)NULL);
   
   zwischenWidget = NULL;      
   
   /*
    * ID eines PromptDialogs zur Texteingabe (Dateinamen) der 
    * UIL-Datei entnehmen
    */
   if ((MrmFetchWidgetOverride(*hierarchie, "nameDialog",
                               parent_Widget, "fileNameDialog",
                               NULL, 0, &zwischenWidget,
                               &widgetType) != MrmSUCCESS)) {
       XtAppError(*app_context, "fileNameDialog nicht geladen");
       }
   
   Widgetliste[FILE_NAME_PROMPT] = zwischenWidget;
                               
   XtUnmanageChild(XmSelectionBoxGetChild(Widgetliste[FILE_NAME_PROMPT],
                                          XmDIALOG_HELP_BUTTON));
        
   XtAddCallback(Widgetliste[FILE_NAME_PROMPT], XmNokCallback,
                 (XtCallbackProc)store_freeGraphicCB, (caddr_t)NULL);

   XtAddCallback(Widgetliste[FILE_NAME_PROMPT], XmNcancelCallback,
                 (XtCallbackProc)store_freeGraphicCB, (caddr_t)NULL);
}

#if 1
/* XtError() ist in Intrinsics.h, aber als 'obsolete' geflagt ... (?!?)
 * ohne diese Version stuerzt das Programm bei 'Rechner veraendern' in XtAppError()
 * per SEGFAULT ab...
 * siehe auch Kommentare in point_in_modifynode() in main.c
 */
void XtError(message)
     _Xconst _XtString message; /* Nachrichtentext */
/*
 * AU: Ausgabe von Fehlermeldungen auf Intrinsics-Ebene 
 */
{       
  XBell(display, 0);
  fprintf(stderr, errorXtMessage, message); /* fatal error */
}
#endif                 
              
int XError(display, error_event)
    Display     *display;
    XErrorEvent *error_event;
/*
 * AU: Ausgabe von Fehlermeldungen auf Xlib-Ebene 
 */
{   
  char  buf[BUFSIZ];
             
  XGetErrorText(display, error_event->error_code, buf, sizeof buf);
  XBell(display, 0);
  fprintf(stderr, errorXMessage, buf); /* non-fatal error */
  
  return 0; /* rueckgabe wird ignoriert, da handler nicht fuer krit., fehler
             * aufgerufen wird */
}    
                       

void wprint(const char *va_alist, ...)
#if 0
     va_dcl
#endif
/*
 * AU: dient der Anzeige von Nachrichten in einem Textfenster waehrend
 *     der Laufzeit des Programms
 *     nach jedem Aufruf ist jeweils die letzte Meldung sichtbar
 * VB: die Funktion nimmt eine variable Liste von Argumenten des Typs String
 */
{
 char                  msgbuf[BUFSIZ];
 char                  *fmt;
 static int            lines = 0;
 static XmTextPosition wpr_position; /* Textposition */
 
 va_list args;                       /*
                                      * Zeiger auf das erste Argument in
                                      * der Liste
                                      */

 va_start(args, va_alist);           /* Argumentzeiger initialisieren */

 fmt = va_arg(args, char *);         /*
                                      * Zugriff auf das naechste Element
                                      * in der Argumentliste
                                      */

 (void) vsprintf(msgbuf, fmt, args); /* lesen der restlichen Argumente */

 va_end(args);                       /*
                                      * Aufraeumarbeiten vor dem
                                      * Verlassen der Funktion
                                      */
 
 /* Meldung msgbuf wird an das Ende des Text-Widgets angefuegt */
 XmTextInsert(Widgetliste[MESSAGE_AREA], wpr_position, msgbuf);
 
 /* Anzahl der angezeigten Meldungen erhoehen */
 lines++;    
                                      
 /* Start am Anfang des Textfeldes */   
 XmTextShowPosition(Widgetliste[MESSAGE_AREA], 0);
 
 /* Text im Text-Widget wird um die Anzahl der Meldungen gescrollt */
 XmTextScroll(Widgetliste[MESSAGE_AREA], lines);

 /* Textposition um die Laenge der Nachricht erhoehen */
 wpr_position = wpr_position + strlen(msgbuf);
}


/***************************************************************************/
/*                    Ende der Funktionen                                  */
/***************************************************************************/
