/*
Name:   typedef.h
Zweck:  Definition von Datenstrukturen
Autor:  Andreas Seifert
Datum:
Update:
*/

#ifndef _TYPEDEF_H_
#define _TYPEDEF_H_

/*
 * programmweit verfuegbare Struktur zur Buendelung globaler Variablen, die
 * Werte bis zur naechsten Benutzeraktion zwischenspeichern 
 */
struct global_data {
   Bool         callbacks_added;
   Bool         virtualNode;
   int          start_x, start_y, last_x, last_y;       /* Start- und 
                                                         * Endkoordinaten eines
                                                         * Rechners */
   int          node_version;                           /* Versionsbezeichnung
                                                         * virtueller Rechner */
   int          node_vote;                              /* Stimmenanzahl 
                                                         * Rechner */
   int          arc_value[5];                           /* Wertebelegung 
                                                         * Rechnerbeziehung */
   int          node_quor[5];                           /* Rechnerquoren und 
                                                         * Rechnerstimme */
   int          numb_quor;                              /* Quorenanzahl 
                                                         * Votingstruktur */
   int          draw_xsize, draw_ysize;                 /* Abmessungen 
                                                         * Zeichenflaeche */
   double       reliability[5];                         /* Wahrscheinlichkeit f.
                                                         * d. erfolgreiche 
                                                         * Ausfuehrung eines 
                                                         * Dienstes */
   char         identity[40];                           /* eindeutiger 
                                                         * Rechnername */
   char         name[40];                               /* Rechnerbezeichnung */
   char         title[40];                              /* verwendeter 
                                                         * Listeneintrag */
   char         filename_TOP[255];                      /* Dateiname 
                                                         * Rechnernetz */
   char         filename_VOT[255];                      /* Dateiname 
                                                         * Votingstruktur */
   char         filename_DRW[255];                      /* Dateiname 
                                                         * Zeichendaten */
   char         pathname[255];                          /* Name 
                                                         * Arbeitsverzeichnis */
   char         host[255];                              /* Name
                                                         * Hostrechner */
   char         operation[5][50];                       /* Bezeichnung d. 
                                                         * Operationen */
   TYPE_OF_FILE dateityp;                               /* eingelesener 
                                                         * Dateityp */
   struct IdentGraphicObject *node_ident;               /* Liste bestehend aus 
                                                         * Zeigern auf die 
                                                         * Reraesentanten eines
                                                         * virt. Rechners */
   struct GraphicObject      *starting_node, *end_node; /* Zeiger auf Ausgangs- 
                                                         * und Endpunkt einer 
                                                         * Rechnerbeziehung */
};


/*
 * Struktur zur Aufnahme von Grafikkontexten fuer Zeichenoperationen unter den
 * versch. Modi, Pixmaps der drei Rechnersymbole und der Zeichensaetze
 */
struct drawtype {
       GC               writeGCs[5];     /* Grafikkontexete */
       Pixmap           terminals[3];    /* Bitmaps der Rechnersymbole */
       XFontStruct      *font_struct[2]; /* Zeichensaetze */
};

/*
 * Struktur zur Darstellung eines Rechners/Knoten in der Topologie des
 * Netzes
 */
struct graphtype {
       char             ident[40];     /* eindeutiger Rechnername */    
       char             name[40];      /* frei vergebener Name */     
       double           reliability;   /* Ausfallwahrscheinlichkeit */  
       STAT_OF_OBJECT   nodeStatus;    /* Traversierung des Rechners */
       COND_OF_OBJECT   nodeCondition; /* Zustand des Rechners */
       struct graphtype *nextnode;     /* naechster Rechner des Netzes */
       struct arctype   *arcptr;       /* Liste der verbundenen Rechner */
};

/*
 * Struktur zur Darstellung einer Verbindung zwischen Rechnern/Knoten
 * der Netztopologie
 */
struct arctype {
       double           reliability;  /* Ausfallwahrscheinlichkeit */ 
       COND_OF_OBJECT   arcCondition; /* Zustand der Verbindung */
       TYPE_OF_OBJECT   arcType;      /* Markierung fuer Traversierung */
       struct graphtype *ndptr;       /* Zeiger zu Endknoten */
       struct graphtype *fatherndptr; /* Zeiger zu Ausgangsknoten */
       struct arctype   *nextarc;     /* naechste Verbindung */
};

/*
 * Struktur zur Bildung eines Vektors, wobei jeder Eintrag ein Elemente
 * der Netztopologie (Kanten und Knoten) darstellt und die Zustaende
 * ausgefallen oder funktionsfaehig annehmen kann. Dadurch lassen sich 
 * verschiedene Zustaende des Netzes darstellen. 
 */
struct arraytype {
       struct graphtype *pointtonode;    /* Symbolisierter Rechner */   
       struct arctype   *pointtoarcs[2]; /* Symbolisierte Verbindung */
       double           reliability;     /* Ausfallwahrscheinlichkeit */
       COND_OF_OBJECT   condition;       /* Zustand */
};


  
/*
 * Struktur zur Darstellung der Rechner der Voting-Struktur
 * 
 * An jedem virtuellen Knoten haengt die Liste seiner Repraesentanten.
 * Damit lassen sich Bezeichnungen fuer virtuelle Knoten vergleichen. 
 * Hierdurch wird ebenfalls die Ausgabe in die Strategiedatei
 * erleichtert. Physikalische Rechner sind durch eine 0 als 
 * Versionsnummer gekennzeichnet.
 */
   
struct GraphicObject {
   char                      ident[40]; /* eindeutiger Rechnername */
   char                      name[40];  /* frei vergebener Name */

   /* top-? und bottom_? sind die Ausgabgs- und Zielpunkte der
    * Kanten zwischen den Rechnern */ 
   int                       top_x, top_y, bottom_x, bottom_y;
   int                       pos_x, pos_y; /* Symbolposition */
   int                       vote;         /* Stimme */
   int                       quor[5];      /* Quoren */

   /* Die Versionsnummer unterscheidet virtuelle Rechner, sonst 0 */
   int                       version; /* 0 bei normalen rechnern */
   STAT_OF_OBJECT            graphObjStatus; /* Zustand */
   TYPE_OF_OBJECT            graphObjTyp;    /* Traversierung */
   struct GraphicObject      *next;          /* naechster Rechner */  
   struct GraphicObject      *previous;      /* Vorgaengerrechner */
   struct ArcGraphicObject   *first_arc;     /* Liste Verbindungen */
   struct IdentGraphicObject *firstIdent;    /* Repraesentanten */

   /* Liste der Kombinationen aus Nachfolgern, deren aufsummierte
    * Anzahl an Stimmen das Quorum des Vaterknotens erreichen */
   struct SetGraphicObject   *firstNodeSetHead[5];
};

/*
 * Struktur zur Darstellung der Verbindung (Kooperationsrelation
 * mit der Zugriffsreihenfolge) zwischen den Rechnern der 
 * Voting-Struktur
 */

struct ArcGraphicObject {
   struct GraphicObject    *arc_to_object; /* Zeiger auf Rechner */
   struct ArcGraphicObject *next_arc;     /* naechste Verbindung */
   int                     arc_value[5]; /* Zugriffsreihenfolge */
};
  
/*
 * Struktur zur Darstellung der Repraesentanten
 * eines virtuellen Rechners (= physikalische Rechner)
 */
struct IdentGraphicObject {
   char                      identItem[5]; /* Name Repraesentant */
   struct IdentGraphicObject *nextListIdent; /* naechster R. */
};
  
/*
 * Struktur zur Aufnahme eines Zeigers auf eine Kombination
 * aus Rechnern deren aufsummierte Anzahl an Stimmen das Quorum 
 * des Vaterknotens erreichen 
 */
struct SetGraphicObject {
   struct SetGraphicObject        *nextNodeSetHead;
   struct SetElementGraphicObject *firstNodeSetElement;
};

/*
 * Kombination aus Rechnern deren aufsummierte Anzahl an Stimmen 
 * das Quorum des Vaterknotens erreichen 
 */
struct SetElementGraphicObject {
   struct GraphicObject           *ptrSetItem;
   struct SetElementGraphicObject *nextNodeSetElement;
};

/*
 * Struktur zur Aufnahme eines Zeigers Quorenliste
 * Gruppe
 */
struct SetVQM {
   struct SetVQM        *nextVQMSetHead;
   struct SetElementVQM *firstVQMSetElement;
};

/*
 * Quorenliste
 */
struct SetElementVQM {
   char                 identVQMSet[5];
   struct SetElementVQM *nextVQMSetElement;
};

/*
 * Struktur zur Aufnahme einer virtuellen Gruppe
 */
struct SetElementReference {
   struct GraphicObject       *ptrRefItem;
   struct SetElementReference *beforeRefSetElement;
   struct SetElementReference *nextRefSetElement;
};


typedef struct drawtype                DRAWINGKIT;
typedef struct graphtype               NODE;
typedef struct arctype                 ARC; 
typedef struct arraytype               VECTOR;
typedef struct global_data             GLOBAL_DATA;
typedef struct GraphicObject           GRAPHIC_NODE;
typedef struct ArcGraphicObject        GRAPHIC_ARC; 
typedef struct IdentGraphicObject      GRAPHIC_IDENT;
typedef struct SetGraphicObject        GRAPHIC_SET;
typedef struct SetElementGraphicObject GRAPHIC_SET_ELEMENT;
typedef struct SetVQM                  VQM_SET;
typedef struct SetElementVQM           VQM_SET_ELEMENT;
typedef struct SetElementReference     REF_SET_ELEMENT;


#endif /* _TYPEDEF_H_ */


